SystemOrganization addCategory: #'XML-Parser'!
SystemOrganization addCategory: 'XML-Parser-DOM'!
SystemOrganization addCategory: 'XML-Parser-DTD'!
SystemOrganization addCategory: 'XML-Parser-Exceptions'!
SystemOrganization addCategory: 'XML-Parser-File'!
SystemOrganization addCategory: 'XML-Parser-HTTP'!
SystemOrganization addCategory: 'XML-Parser-Namespaces'!
SystemOrganization addCategory: 'XML-Parser-Streams'!
SystemOrganization addCategory: 'XML-Parser-URI'!
SystemOrganization addCategory: 'XML-Parser-Validation'!
SystemOrganization addCategory: 'XML-Parser-Validation-DFA'!

Error subclass: #XMLParserException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLParserException commentStamp: 'monty 1/23/2014 17:59' prior: 0!
A base class for XML exceptions to make catching XML errors easier.!

XMLParserException subclass: #XMLDOMException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLDOMException commentStamp: 'monty 1/16/2014 19:40' prior: 0!
Invoked when a DOM tree is manipulated wrong.!

XMLParserException subclass: #XMLFileException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLFileException commentStamp: 'monty 1/16/2014 19:40' prior: 0!
Invoked for file IO errors.!

XMLParserException subclass: #XMLHTTPException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLHTTPException commentStamp: 'monty 1/16/2014 19:40' prior: 0!
Invoked for HTTP errors.!

XMLParserException subclass: #XMLLimitException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLLimitException commentStamp: 'monty 1/25/2014 00:19' prior: 0!
Invoked when a security limit on input is exceeded. Use the messages in XMLParserLimits and DTDExternalEntityLimits to configure these limits or the removeLimits message to remove them completely.!

XMLParserException subclass: #XMLNamespaceException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLNamespaceException commentStamp: 'monty 1/16/2014 19:41' prior: 0!
Invoked when namespaces are used improperly and namespace checking is on.!

!XMLParserException methodsFor: 'appending' stamp: 'monty 2/3/2014 18:45'!
appendLineNumber: aLineNumber
	self messageText: self messageText, ' (line ', aLineNumber printString, ')'! !

XMLParserException subclass: #XMLValidationException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLValidationException commentStamp: 'monty 1/16/2014 19:41' prior: 0!
Invoked when invalid XML is detected and validation is on.!

!XMLValidationException class methodsFor: 'instance creation' stamp: 'monty 2/2/2014 07:23'!
undeclaredGeneralEntity: aName
	^ self
		undeclaredGeneralEntity: aName
		isParsed: true! !

!XMLValidationException class methodsFor: 'instance creation' stamp: 'monty 2/2/2014 07:24'!
undeclaredGeneralEntity: aName isParsed: aBoolean
	^ self new
		messageText:
			'Undeclared ',
			(aBoolean ifTrue: ['general'] ifFalse: ['unparsed']),
			' entity reference &', aName, ';';
		yourself! !

!XMLValidationException class methodsFor: 'instance creation' stamp: 'monty 1/24/2014 22:28'!
undeclaredParameterEntity: aName
	^ self new
		messageText: 'Undeclared parameter entity reference %', aName, ';';
		yourself! !

!XMLValidationException class methodsFor: 'instance creation' stamp: 'monty 2/2/2014 07:23'!
undeclaredUnparsedEntity: aName
	^ self
		undeclaredGeneralEntity: aName
		isParsed: false! !

XMLParserException subclass: #XMLWellFormednessException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLWellFormednessException commentStamp: 'monty 1/16/2014 19:41' prior: 0!
Invoked when XML markup is not well-formed.!

!String methodsFor: '*XML-Parser' stamp: 'JAAyer 4/14/2010 21:26'!
asXMLString
	^ XMLString string: self! !

!String methodsFor: '*XML-Parser' stamp: 'monty 1/10/2014 00:56'!
asXMLURI
	^ XMLURI fromString: self! !

!String methodsFor: '*XML-Parser' stamp: 'monty 10/8/2014 00:09'!
isQualifiedName: aQualifiedName orLocalName: isLocalName
	^ self = aQualifiedName
		or: [(aQualifiedName size > isLocalName size)
			and: [self = isLocalName]]! !

!String methodsFor: '*XML-Parser' stamp: 'monty 10/18/2014 05:47'!
parseXML
	^ XMLDOMParser parse: self! !

!String methodsFor: '*XML-Parser' stamp: 'monty 10/3/2014 20:26'!
qualifiedNameLocalPart
	^ self qualifiedNameLocalPartFromPrefix: self qualifiedNamePrefix! !

!String methodsFor: '*XML-Parser' stamp: 'monty 10/8/2014 20:57'!
qualifiedNameLocalPartFromPrefix: aPrefix
	"optimized"
	| prefixSize |

	(prefixSize := aPrefix size) > 0
		ifTrue: [^ self copyFrom: prefixSize + 2 to: self size]
		ifFalse: [^ self]! !

!String methodsFor: '*XML-Parser' stamp: 'monty 10/8/2014 23:48'!
qualifiedNamePrefix
	"Split the name into namespace prefix
	(Optimized code)"
	| i |

	i := self size.
	[i > 1]
		whileTrue: [
			((self at: i) == $:)
				ifTrue: [^ (self copyFrom: 1 to: i - 1)].
			i := i - 1].
	^ ''.! !

Collection subclass: #XMLNFAStateSet
	instanceVariableNames: 'set isAccepting'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAStateSet commentStamp: 'monty 1/24/2014 22:44' prior: 0!
This class is a set of NFA states. It wraps an IdentitySet and keeps track of if any of its states is accepting.!

!XMLNFAStateSet methodsFor: 'comparing' stamp: 'monty 1/9/2014 22:12'!
= anObject
	^ self == anObject
		or: [self class == anObject class
			and: [set = anObject set]]! !

!XMLNFAStateSet methodsFor: 'adding' stamp: 'monty 12/28/2013 18:52'!
add: aState
	isAccepting
		ifFalse: [isAccepting := aState isAccepting].

	^ set add: aState.! !

!XMLNFAStateSet methodsFor: 'enumerating' stamp: 'monty 12/28/2013 18:47'!
do: aBlock
	set do: aBlock! !

!XMLNFAStateSet methodsFor: 'comparing' stamp: 'monty 12/28/2013 18:50'!
hash
	^ set hash! !

!XMLNFAStateSet methodsFor: 'testing' stamp: 'monty 12/28/2013 18:48'!
includes: aState
	^ set includes: aState! !

!XMLNFAStateSet methodsFor: 'initialization' stamp: 'monty 12/28/2013 18:52'!
initialize
	super initialize.

	set := IdentitySet new.
	isAccepting := false.! !

!XMLNFAStateSet methodsFor: 'testing' stamp: 'monty 12/28/2013 18:53'!
isAccepting
	^ isAccepting! !

!XMLNFAStateSet methodsFor: 'removing' stamp: 'monty 12/28/2013 18:46'!
remove: aState ifAbsent: anExceptionBlock
	set
		remove: aState
		ifAbsent: [^ anExceptionBlock value].

	isAccepting
		ifTrue: [isAccepting := set anySatisfy: [:each | each isAccepting]].

	^ aState.! !

!XMLNFAStateSet methodsFor: 'accessing' stamp: 'monty 12/28/2013 18:50'!
set
	^ set! !

!XMLNFAStateSet methodsFor: 'accessing' stamp: 'monty 12/28/2013 19:01'!
size
	^ set size! !

!SequenceableCollection methodsFor: '*XML-Parser' stamp: 'monty 10/19/2014 06:23'!
lastOrNil
	"for speed to avoid at:ifAbsent: closure"
	| lastOrNilOffset |

	(lastOrNilOffset := self size) > 0
		ifTrue: [^ self at: lastOrNilOffset]
		ifFalse: [^ nil]! !

SequenceableCollection subclass: #XMLOrderedList
	instanceVariableNames: 'collection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLOrderedList commentStamp: '<historical>' prior: 0!
This class functions as an OrderedCollection that will not raise exceptions when attempts are made to access absent objects. This means messages like #at:, #after:, and #first will return nil when the requested object cannot be retrieved. Messages that modify the collection based on the presence or absence of other objects such as #add:after: or #at:put: will still raise exceptions as usual.

!

XMLOrderedList subclass: #XMLObservableList
	instanceVariableNames: 'observer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLObservableList commentStamp: '<historical>' prior: 0!
This class is an is an ordered list that sends notifications to observers when objects are added to it or removed from it. Subclasses should override the #beforeAdding:, #afterAdding: and #afterRemoving: messages to send custom notifications.!

XMLObservableList subclass: #XMLAttributeList
	instanceVariableNames: 'attributesByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLAttributeList commentStamp: '<historical>' prior: 0!
This class represents an ordered list of attribute nodes that also keeps a dictionary mapping attribute names to nodes for faster lookups. You usually shouldn't have to use this class directly.!

!XMLAttributeList methodsFor: 'adding' stamp: 'JAAyer 10/15/2010 12:41'!
addName: aName value: aValue
	^ self addLast:
		(XMLAttribute
			name: aName
			value: aValue)! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 10/18/2014 01:45'!
afterAdding: anObject
	self attributesByName
		at: anObject name ifPresent: [self removeAttributeNodeAt: anObject name];
		at: anObject name put: anObject.

	observer ifNotNil: [observer addedAttribute: anObject].! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 10/18/2014 01:46'!
afterRemoving: anAttribute
	self attributesByName removeKey: anAttribute name.
	observer ifNotNil: [observer removedAttribute: anAttribute].! !

!XMLAttributeList methodsFor: 'converting' stamp: 'monty 2/20/2014 01:32'!
asDictionary
	| dictionary |

	dictionary := OrderPreservingDictionary new: self size withDefaultValue: ''.
	self do: [:each | dictionary at: each name put: each value].
	^ dictionary.! !

!XMLAttributeList methodsFor: 'converting' stamp: 'JAAyer 6/25/2010 22:15'!
asNameValueAssociations
	^ self collect: [:each | each name -> each value]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:08'!
attributeNames
	^ self collect: [:each | each name]! !

!XMLAttributeList methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:14'!
attributeNamesAndValuesDo: aTwoArgumentBlock
	self do: [:each | aTwoArgumentBlock value: each name value: each value]! !

!XMLAttributeList methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:09'!
attributeNamesDo: aBlock
	self do: [:each | aBlock value: each name]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:27'!
attributeNodeAt: aName
	^ self attributeNodeAt: aName ifAbsent: [self defaultValue]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:27'!
attributeNodeAt: aName ifAbsent: aBlock
	^ self attributesByName at: aName ifAbsent: aBlock! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:28'!
attributeNodeAt: aLocalName namespaceURI: aUri
	^ self attributeNodeAt: aLocalName namespaceURI: aUri ifAbsent: [self defaultValue]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:28'!
attributeNodeAt: aLocalName namespaceURI: aUri ifAbsent: aBlock
	^ self
		detect: [:each | (each isNamed: aLocalName) and: [each namespaceURI = aUri]]
		ifNone: aBlock! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:11'!
attributeValueAt: aName
	^ self attributeValueAt: aName ifAbsent: [self defaultValue]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:11'!
attributeValueAt: aName ifAbsent: aBlock
	^ (self attributesByName
		at: aName
		ifAbsent: [^ aBlock value]) value! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 7/11/2010 03:34'!
attributeValueAt: aName ifAbsentPut: aBlock
	^ self
		attributeValueAt: aName
		ifAbsent: [self attributeValueAt: aName put: aBlock value]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 12/6/2010 21:23'!
attributeValueAt: aName put: aValue
	(self includesAttribute: aName)
		ifTrue: [(self attributeNodeAt: aName) value: aValue]
		ifFalse: [self addName: aName value: aValue].
	^ aValue.! !

!XMLAttributeList methodsFor: 'private' stamp: 'JAAyer 6/10/2010 05:51'!
attributesByName
	^ attributesByName ifNil: [attributesByName := Dictionary new]! !

!XMLAttributeList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
beforeAdding: anObject
	anObject element
		ifNotNil: [XMLDOMException signal: 'Cannot add attribute node from other element']! !

!XMLAttributeList methodsFor: 'testing' stamp: 'JAAyer 12/6/2010 21:23'!
includesAttribute: aName
	^ self attributesByName includesKey: aName! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 12/6/2010 21:24'!
removeAttributeNodeAt: aName
	^ self remove: (self attributeNodeAt: aName).! !

!XMLAttributeList methodsFor: 'notifications' stamp: 'monty 10/18/2014 05:56'!
renamedAttribute: anAttribute from: anOldName to: aNewName
	self attributesByName
		removeKey: anOldName;
		at: aNewName ifPresent: [:attribute | self remove: attribute];
		at: aNewName put: anAttribute! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 10/18/2014 06:08'!
setFromDictionary: anOrderPreservingDictionary
	| association |

	"hack to reuse a dictionary for speed"
	attributesByName := anOrderPreservingDictionary dictionary.

	"add the XMLAttributes in order directly to the collection
	to avoid sending notifications and also replace the value in
	the attributesByName association with the XMLAttribute"
	anOrderPreservingDictionary keysDo: [:each |
		(association := attributesByName associationAt: each ifAbsent: nil)
			value:
				(collection addLast: 
					(XMLAttribute new
						setName: each
						value: association value
						element: observer))].! !

!XMLAttributeList methodsFor: 'sorting' stamp: 'monty 10/19/2014 08:04'!
sort
	self sort: [:a :b | a name <= b name]! !

!XMLAttributeList methodsFor: 'sorting' stamp: 'monty 10/19/2014 08:07'!
sorted
	^ self sorted: [:a :b | a name <= b name]! !

XMLObservableList subclass: #XMLInternalSubsetList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLInternalSubsetList commentStamp: 'JAAyer 7/27/2010 12:45' prior: 0!
This class represents the list of nodes that comprise a document's internal DTD subset.!

!XMLInternalSubsetList methodsFor: 'private' stamp: 'monty 10/18/2014 00:54'!
afterAdding: aNode
	observer ifNotNil: [observer addedNode: aNode]! !

!XMLInternalSubsetList methodsFor: 'private' stamp: 'monty 10/18/2014 00:54'!
afterRemoving: aNode
	observer ifNotNil: [observer removedNode: aNode]! !

!XMLInternalSubsetList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
beforeAdding: aNode! !

XMLObservableList subclass: #XMLNodeList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeList commentStamp: '<historical>' prior: 0!
This class represents an ordered list of child nodes for some kind of node with elements, probably either an XML document or element object. Non-modifying accessing messages like #first and #last will never raise exceptions and return nil instead.!

XMLNodeList subclass: #XMLCachingNodeList
	instanceVariableNames: 'elements elementsByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLCachingNodeList commentStamp: '<historical>' prior: 0!
This is a node list that caches child elements by name, keeping both an ordered list of elements and a dictionary that allows for faster retrieval of elements by qualified or local name.

You can inject it into the DOM parser like this:
parser := XMLDOMParser on: xmlSource.
parser nodeListClass: XMLCachingNodeList.!

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 10/18/2014 00:49'!
afterAdding: aNode
	aNode isElement
		ifTrue: [
			((self last == aNode) or: [(self findElementAfter: aNode) isNil])
				ifTrue: [self elementAddedAtEnd: aNode]
				ifFalse: [self elementInsertedBeforeEnd: aNode]].
	super afterAdding: aNode.
			! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 10/18/2014 01:28'!
afterRemoving: aNode
	aNode isElement
		ifTrue: [
			self elements remove: aNode.
			self elementsByNameAt: aNode name removeElement: aNode.
			aNode hasPrefix
				ifTrue: [self elementsByNameAt: aNode localName removeElement: aNode]].
	super afterRemoving: aNode.! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
elementAddedAtEnd: anElement
	self elements add: anElement.
	self
		elementsByNameAt: anElement name
		addElement: anElement.
	anElement hasPrefix
		ifTrue: [
			self
				elementsByNameAt: anElement localName
				addElement: anElement]! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:01'!
elementAt: aName ifAbsent: aBlock
	| elementsWithName |

	elementsWithName := self elementsByName at: aName ifAbsent: [^ aBlock value].

	^ elementsWithName isCollection
		ifTrue: [elementsWithName first]
		ifFalse: [elementsWithName]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
elementInsertedBeforeEnd: anElement
	| elementAfter |

	elementAfter := self findElementAfter: anElement.
	self elements add: anElement before: elementAfter.

	self elementsByNameAt: anElement name insertElement: anElement.
	anElement hasPrefix
		ifTrue: [
			self
				elementsByNameAt: anElement localName
				insertElement: anElement]! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:01'!
elements
	^ elements ifNil: [elements := self species new]! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:01'!
elementsAt: aName
	| elementsWithName |

	elementsWithName :=
		self elementsByName
			at: aName
			ifAbsent: [^ self species new].

	^ elementsWithName isCollection
		ifTrue: [elementsWithName]
		ifFalse: [
			self elementsByName
				at: aName
				put: (self species with: elementsWithName)]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
elementsByName
	^ elementsByName ifNil: [elementsByName := Dictionary new].! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
elementsByNameAt: aName addElement: anElement
	| elementsWithName |

	elementsWithName := 
		self elementsByName
			at: aName
			ifAbsent: [
				self elementsByName at: aName put: anElement.
				^ self].

	elementsWithName isCollection
		ifTrue: [elementsWithName add: anElement]
		ifFalse: [
			self elementsByName
				at: aName
				put: (self species with: elementsWithName with: anElement)]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
elementsByNameAt: aName insertElement: anElement
	| elementsWithName elementAfter |

	(elementAfter := self findElementAfter: anElement withName: aName)
		ifNil: [^ self elementsByNameAt: aName addElement: anElement].

	(elementsWithName := self elementsByName at: aName)
		isCollection
			ifTrue: [elementsWithName add: anElement before: elementAfter]
			ifFalse: [
				self elementsByName
					at: aName
					put: (self species with: anElement with: elementAfter)]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
elementsByNameAt: aName removeElement: anElement
	| elementsWithName |

	(elementsWithName := self elementsByName at: aName)
		isCollection
			ifTrue: [
				elementsWithName remove: anElement.
				elementsWithName ifEmpty: [self elementsByName removeKey: aName]]
			ifFalse: [self elementsByName removeKey: aName]! !

!XMLCachingNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:02'!
elementsDetect: aBlock ifNone: anExceptionBlock
	^ self elements detect: aBlock ifNone: anExceptionBlock! !

!XMLCachingNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:02'!
elementsDo: aBlock
	self elements do: aBlock! !

!XMLCachingNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:02'!
elementsSelect: aBlock
	^ self elements select: aBlock! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
findElementAfter: anElement
	| previousElement |

	previousElement := nil.
	self do: [:each |
		each isElement
			ifTrue: [
				previousElement == anElement
					ifTrue: [^ each].
				previousElement := each]].
	^ nil.! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
findElementAfter: anElement withName: aName
	| previousElementWithName |

	previousElementWithName := nil.
	self do: [:each |
		(each isElement and: [each isNamed: aName])
			ifTrue: [
				(previousElementWithName == anElement)
					ifTrue: [^ each].
				previousElementWithName := each]].
	^ nil.! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 2/10/2014 23:11'!
firstElement
	^ self elements first! !

!XMLCachingNodeList methodsFor: 'testing' stamp: 'monty 11/26/2013 22:01'!
hasElements
	^ elements notNil and: [elements notEmpty]! !

!XMLCachingNodeList methodsFor: 'testing' stamp: 'monty 11/26/2013 22:01'!
includesElementAt: aName
	^ self elementsByName includesKey: aName! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 2/10/2014 23:11'!
lastElement
	^ self elements last! !

!XMLCachingNodeList methodsFor: 'notifications' stamp: 'monty 10/3/2014 20:26'!
renamedElement: anElement from: anOldName to: aNewName
	| oldPrefix |

	self elementsByNameAt: anOldName removeElement: anElement.
	(oldPrefix := anOldName qualifiedNamePrefix)
		ifNotEmpty: [
			self
				elementsByNameAt: (anOldName qualifiedNameLocalPartFromPrefix: oldPrefix)
				removeElement: anElement].

	self elementsByNameAt: anElement name insertElement: anElement.
	anElement hasPrefix
		ifTrue: [
			self
				elementsByNameAt: anElement localName
				insertElement: anElement].! !

!XMLCachingNodeList methodsFor: 'as yet unclassified' stamp: 'monty 10/19/2014 07:33'!
sort: aSortBlock
	super sort: aSortBlock.
	self elements sort: aSortBlock.
	self elementsByName valuesDo: [:each |
		each isCollection
			ifTrue: [each sort: aSortBlock]].! !

!XMLNodeList methodsFor: 'private' stamp: 'monty 10/18/2014 00:55'!
afterAdding: aNode
	observer ifNil: [^ self].

	aNode isElement
		ifTrue: [observer addedElement: aNode]
		ifFalse: [observer addedNode: aNode].! !

!XMLNodeList methodsFor: 'private' stamp: 'monty 10/18/2014 00:55'!
afterRemoving: aNode
	observer ifNil: [^ self].

	aNode isElement
		ifTrue: [observer removedElement: aNode]
		ifFalse: [observer removedNode: aNode].! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 11/22/2010 15:42'!
beforeAdding: aNode
	aNode hasParent
		ifTrue: [XMLDOMException signal: 'Cannot add child node that already has a parent']! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 2/14/2014 05:42'!
elementAfter: anElement
	| last |

	self elementsDo: [:each |
		last == anElement
			ifTrue: [^ each].
		last := each].
	^ nil.! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName
	^ self elementAt: aName ifAbsent: [self defaultValue]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:13'!
elementAt: aName ifAbsent: aBlock
	^ self
		detect: [:each | each isElement and: [each isNamed: aName]]
		ifNone: aBlock! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName namespaceURI: aUri
	^ self elementAt: aName namespaceURI: aUri ifAbsent: [self defaultValue]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName namespaceURI: aUri ifAbsent: aBlock
	^ (self elementsAt: aName)
		detect: [:each | each namespaceURI = aUri]
		ifNone: aBlock! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 2/14/2014 05:46'!
elementBefore: anElement
	| last |
	
	self elementsDo: [:each |
		each == anElement
			ifTrue: [^ last].
		last := each].
	^ nil.! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:07'!
elements
	^ self select: [:each | each isElement]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:12'!
elementsAt: aName
	^ self elementsSelect: [:each | each isNamed: aName]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:33'!
elementsAt: aName namespaceURI: aUri
	^ (self elementsAt: aName) select: [:each | each namespaceURI = aUri]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:34'!
elementsCollect: aBlock
	^ self elements collect: aBlock! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsDetect: aBlock
	^ self elementsDetect: aBlock ifNone: [self defaultValue]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:10'!
elementsDetect: aBlock ifNone: anExceptionBlock
	^ self
		detect: [:each | each isElement and: [aBlock value: each]]
		ifNone: anExceptionBlock! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:11'!
elementsDo: aBlock
	self do: [:each | each isElement ifTrue: [aBlock value: each]]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:08'!
elementsSelect: aBlock
	^ self select: [:each | each isElement and: [aBlock value: each]]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 2/10/2014 23:09'!
firstElement
	^ self
		detect: [:each | each isElement]
		ifNone: [nil]! !

!XMLNodeList methodsFor: 'testing' stamp: 'monty 11/26/2013 22:03'!
hasElements
	^ self anySatisfy: [:each | each isElement]! !

!XMLNodeList methodsFor: 'testing' stamp: 'monty 11/26/2013 22:04'!
includesElementAt: aName
	^ (self elementAt: aName) notNil! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 2/10/2014 23:10'!
lastElement
	self reverseDo: [:each |
		each isElement
			ifTrue: [^ each]].
	^ nil.! !

!XMLNodeList methodsFor: 'notifications' stamp: 'monty 10/18/2014 01:06'!
renamedElement: anElement from: anOldName to: aNewName! !

!XMLNodeList methodsFor: 'sorting' stamp: 'monty 10/19/2014 08:06'!
sort
	self sort: [:a :b |
		(a isElement & b isElement) not
			or: [a name <= b name]]! !

!XMLNodeList methodsFor: 'sorting' stamp: 'monty 10/19/2014 08:06'!
sorted
	^ self sorted: [:a :b |
		(a isElement & b isElement) not
			or: [a name <= b name]]! !

!XMLObservableList class methodsFor: 'instance creation' stamp: 'monty 10/18/2014 01:02'!
observer: anObject
	^ self new observer: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 14:46'!
add: anObject after: aPriorObject
	^ self with: [super add: anObject after: aPriorObject] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 10:56'!
add: anObject afterIndex: anIndex
	^ self with: [super add: anObject afterIndex: anIndex] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 14:46'!
add: anObject before: aSubsequentObject
	^ self with: [super add: anObject before: aSubsequentObject] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 10:57'!
add: anObject beforeIndex: anIndex
	^ self with: [super add: anObject beforeIndex: anIndex] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 10:57'!
addFirst: anObject
	^ self with: [super addFirst: anObject] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'monty 2/6/2014 01:02'!
addLast: anObject
	"inlined for speed"
	self beforeAdding: anObject.
	collection addLast: anObject.
	self afterAdding: anObject.
	^ anObject.! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 10/18/2014 00:51'!
afterAdding: anObject
	observer ifNotNil: [observer added: anObject to: self]! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 10/18/2014 00:51'!
afterRemoving: anObject
	observer ifNotNil: [observer removed: anObject from: self]! !

!XMLObservableList methodsFor: 'accessing' stamp: 'JAA 3/21/2011 20:08'!
at: anIndex put: anObject
	| presentObject |

	(presentObject := self at: anIndex)
		ifNotNil: [
			self
				with: [super at: anIndex put: nil]
				remove: presentObject].

	^ self with: [super at: anIndex put: anObject] add: anObject.! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 10/18/2014 00:51'!
beforeAdding: anObject
	observer ifNotNil: [observer adding: anObject to: self]! !

!XMLObservableList methodsFor: 'observing' stamp: 'monty 10/18/2014 00:50'!
observer
	^ observer! !

!XMLObservableList methodsFor: 'observing' stamp: 'monty 10/18/2014 00:50'!
observer: anObserver
	observer := anObserver! !

!XMLObservableList methodsFor: 'removing' stamp: 'JAAyer 7/22/2010 10:58'!
remove: anObject ifAbsent: aBlock
	^ self with: [super remove: anObject ifAbsent: [^ aBlock value]] remove: anObject! !

!XMLObservableList methodsFor: 'removing' stamp: 'JAAyer 6/11/2010 09:03'!
removeAt: anIndex
	^ self with: [super removeAt: anIndex] remove: (self at: anIndex)! !

!XMLObservableList methodsFor: 'removing' stamp: 'monty 10/19/2014 07:11'!
reset
	self removeAll! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 2/2/2014 21:19'!
with: aBlock add: anObject
	self beforeAdding: anObject.
	aBlock value.
	self afterAdding: anObject.
	^ anObject.! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 2/2/2014 21:20'!
with: aBlock remove: anObject
	aBlock value.
	self afterRemoving: anObject.
	^ anObject.! !

!XMLOrderedList class methodsFor: 'private' stamp: 'monty 1/29/2014 01:12'!
collectionClass
	^ OrderedCollection! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'monty 1/28/2014 05:54'!
new
	^ self new: 5! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'monty 1/29/2014 01:13'!
new: aSize
	^ self basicNew setCollection: (self collectionClass new: aSize)! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'monty 1/29/2014 01:13'!
new: aSize withAll: anObject
	^ self basicNew setCollection: (self collectionClass new: aSize withAll: anObject)! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'monty 1/29/2014 01:13'!
newFrom: aCollection
	^ self basicNew setCollection: (self collectionClass newFrom: aCollection)! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'JAAyer 8/4/2010 14:43'!
ofSize: aSize
	^ self new: aSize withAll: nil! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:01'!
add: anObject
	^ self addLast: anObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:08'!
add: anObject after: aPriorObject
	^ collection add: anObject after: aPriorObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:09'!
add: anObject afterIndex: anIndex
	^ collection add: anObject afterIndex: anIndex! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:09'!
add: anObject before: aSubsequentObject
	^ collection add: anObject before: aSubsequentObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:10'!
add: anObject beforeIndex: anIndex
	^ collection add: anObject beforeIndex: anIndex! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:02'!
addAll: aCollection
	^ self addAllLast: aCollection! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:10'!
addAllFirst: aCollection
	aCollection reverseDo: [:each | self addFirst: each].
	^ aCollection.! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:11'!
addAllLast: aCollection
	aCollection do: [:each | self addLast: each].
	^ aCollection.! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:11'!
addFirst: anObject
	^ collection addFirst: anObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:12'!
addLast: anObject
	^ collection addLast: anObject! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 12:48'!
after: anObject
	^ collection after: anObject ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'converting' stamp: 'JAAyer 8/4/2010 12:53'!
asArray
	^ collection asArray! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 12:49'!
at: anIndex
	^ collection at: anIndex ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:12'!
at: anIndex ifAbsentPut: aValue
	^ self
		at: anIndex
		ifAbsent: [self at: anIndex put: aValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAA 3/25/2011 14:32'!
at: anIndex put: aValue
	^ collection at: anIndex put: aValue! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/5/2010 00:55'!
atLast: anIndexFromEnd
	^ self atLast: anIndexFromEnd ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 12:50'!
before: anObject
	^ collection before: anObject ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 13:14'!
capacity
	^ collection capacity! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/19/2014 06:27'!
collect: aBlock
	^ self speciesNewWithCollection: (collection collect: aBlock)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/19/2014 06:27'!
collect: aBlock from: aStartIndex to: anEndIndex
	^ self speciesNewWithCollection:
		(collection collect: aBlock from: aStartIndex to: anEndIndex)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/19/2014 06:27'!
collect: aCollectBlock thenSelect: aSelectBlock
	^ self speciesNewWithCollection:
		(collection collect: aCollectBlock thenSelect: aSelectBlock)! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAA 3/25/2011 15:35'!
collection
	^ collection! !

!XMLOrderedList methodsFor: 'copying' stamp: 'JAAyer 8/4/2010 14:54'!
copy
	^ self species basicNew setCollection: collection copy! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 10/19/2014 07:14'!
copyEmpty
	^ self species new! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 10/19/2014 06:27'!
copyFrom: aStart to: anEnd
	^ self speciesNewWithCollection:
		(collection copyFrom: (aStart max: 1) to: (self size min: anEnd))! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 9/29/2014 00:38'!
copyReplaceFrom: aStart to: aStop with: aReplacementCollection
	"adapted from OrderedCollection"
	| newList delta startIndex stopIndex |

	delta := 0.
	startIndex := aStart.
	stopIndex := aStop.
	aStart < 1
		ifTrue: [startIndex := stopIndex := 0]
		ifFalse: [startIndex > self size
				ifTrue: [startIndex := stopIndex := self size + 1]
				ifFalse: 
					[(stopIndex < (startIndex - 1) or: [stopIndex > self size])
						ifTrue: [self errorOutOfBounds].
					delta := stopIndex - startIndex + 1]].

	newList := self species new: self size + aReplacementCollection size - delta.
	1 to: startIndex - 1 do: [:index |
		newList add: (self at: index)].
	1 to: aReplacementCollection size do: [:index |
		newList add: (aReplacementCollection at: index)].
	stopIndex + 1 to: self size do: [:index |
		newList add: (self at: index)].

	^ newList.! !

!XMLOrderedList methodsFor: 'copying' stamp: 'JAA 3/25/2011 15:18'!
copyWith: anElement 
	^ self copy
	 	add: anElement;
		yourself! !

!XMLOrderedList methodsFor: 'defaults' stamp: 'JAAyer 3/12/2010 06:48'!
defaultValue
	^ nil! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 8/4/2010 12:49'!
do: aBlock
	collection do: aBlock! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/19/2014 06:27'!
reject: aBlock
	^ self speciesNewWithCollection: (collection reject: aBlock)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/19/2014 06:27'!
reject: aRejectBlock thenCollect: aCollectBlock
	^ self speciesNewWithCollection:
		(collection
			reject: aRejectBlock
			thenCollect: aCollectBlock)! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 13:33'!
remove: anObject ifAbsent: aBlock
	^ collection remove: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 16:00'!
removeAll
	[self isEmpty] whileFalse: [self removeLast]! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 15:46'!
removeAt: anIndex
	^ collection removeAt: anIndex! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 1/12/2014 20:25'!
removeFirst
	^ self removeAt: 1! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 1/12/2014 20:32'!
removeFirst: aNumber
	| removed |

	removed := self copyFrom: 1 to: aNumber.
	aNumber timesRepeat: [self removeFirst].
	^ removed.! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 1/12/2014 20:24'!
removeLast
	^ self removeAt: self size! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 1/12/2014 20:33'!
removeLast: aNumber
	| removed |

	removed := self copyFrom: self size - (aNumber - 1) to: self size.
	aNumber timesRepeat: [self removeLast].
	^ removed.! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 1/12/2014 18:26'!
replaceFrom: aStartingIndex to: anEndingIndex with: aReplacementCollection startingAt: aReplacementIndex
	| offset |

	offset := 0.
	[aStartingIndex + offset <= anEndingIndex]
		whileTrue: [
			self
				at: aStartingIndex + offset
				put: (aReplacementCollection at: aReplacementIndex + offset).
			offset := offset + 1].! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 10/19/2014 06:31'!
reset
	collection reset! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 8/4/2010 13:35'!
reverseDo: aBlock
	collection reverseDo: aBlock! !

!XMLOrderedList methodsFor: 'converting' stamp: 'JAA 3/26/2011 18:54'!
reversed
	| reversed |

	reversed := self species new.
	self reverseDo: [:each | reversed addLast: each].
	^ reversed.
! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/19/2014 06:27'!
select: aBlock
	^ self speciesNewWithCollection: (collection select: aBlock)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/19/2014 06:27'!
select: aSelectBlock thenCollect: aCollectBlock
	^ self speciesNewWithCollection:
		(collection select: aSelectBlock thenCollect: aCollectBlock)! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAAyer 8/4/2010 13:38'!
setCollection: aCollection
	collection := aCollection! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 12:49'!
size
	^ collection size! !

!XMLOrderedList methodsFor: 'sorting' stamp: 'JAAyer 8/4/2010 13:43'!
sort
	collection sort! !

!XMLOrderedList methodsFor: 'sorting' stamp: 'JAAyer 8/4/2010 13:43'!
sort: aSortBlock
	collection sort: aSortBlock! !

!XMLOrderedList methodsFor: 'sorting' stamp: 'monty 10/19/2014 07:46'!
sorted
	self copy sort! !

!XMLOrderedList methodsFor: 'sorting' stamp: 'monty 10/19/2014 08:07'!
sorted: aSortBlock
	^ self copy sort: aSortBlock! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAAyer 3/12/2010 11:02'!
species
	^ XMLOrderedList! !

!XMLOrderedList methodsFor: 'private' stamp: 'monty 10/19/2014 06:27'!
speciesNewWithCollection: aCollection
	^ self species basicNew setCollection: aCollection! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/19/2014 06:27'!
with: aCollection collect: aBlock
	^ self speciesNewWithCollection: (collection with: aCollection collect: aBlock)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/19/2014 06:27'!
withIndexCollect: aTwoArgumentBlock
	^ self speciesNewWithCollection: (collection withIndexCollect: aTwoArgumentBlock)! !

Object subclass: #DTDAttributeDefinitionParser
	instanceVariableNames: 'element attribute type defaultPragma defaultValue'
	classVariableNames: 'DefaultValueValidators NonEnumerationValidators'
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDAttributeDefinitionParser commentStamp: '<historical>' prior: 0!
This class parses an attribute definition in an ATTLIST declaration and creates a validator for it.!

!DTDAttributeDefinitionParser class methodsFor: 'instance creation' stamp: 'monty 12/30/2013 23:53'!
element: anElement attribute: anAttribute type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	^ self new
		setElement: anElement
		attribute: anAttribute
		type: aType
		defaultPragma: aDefaultPragma
		defaultValue: aDefaultValue! !

!DTDAttributeDefinitionParser class methodsFor: 'class initialization' stamp: 'monty 12/31/2013 02:54'!
initialize
	"self initialize"

	super initialize.
	self
		initializeDefaultValueValidators;
		initializeNonEnumerationValidators.! !

!DTDAttributeDefinitionParser class methodsFor: 'class initialization' stamp: 'monty 12/31/2013 02:53'!
initializeDefaultValueValidators
	(DefaultValueValidators := Dictionary new)
		at: 'FIXED' put: XMLFixedValueAttributeValidator;
		at: 'IMPLIED' put: XMLImpliedValueAttributeValidator;
		at: 'REQUIRED' put: XMLRequiredValueAttributeValidator! !

!DTDAttributeDefinitionParser class methodsFor: 'class initialization' stamp: 'monty 12/31/2013 02:54'!
initializeNonEnumerationValidators
	(NonEnumerationValidators := Dictionary new)
		at: 'CDATA' put: XMLCDataAttributeValidator;
		at: 'ID' put: XMLIDAttributeValidator;
		at: 'IDREF' put: XMLIDRefAttributeValidator;
		at: 'IDREFS' put: XMLIDRefsAttributeValidator;
		at: 'ENTITY' put: XMLEntityAttributeValidator;
		at: 'ENTITIES' put: XMLEntitiesAttributeValidator;
		at: 'NMTOKEN' put: XMLNameAttributeValidator;
		at: 'NMTOKENS' put: XMLNamesAttributeValidator! !

!DTDAttributeDefinitionParser methodsFor: 'private ' stamp: 'monty 1/25/2014 01:25'!
hasNotationType
	^ type beginsWith: 'NOTATION '! !

!DTDAttributeDefinitionParser methodsFor: 'parsing' stamp: 'monty 1/25/2014 01:28'!
parse
	| defaultValueValidator |

	defaultValueValidator :=
		(DefaultValueValidators
			at: defaultPragma
			ifAbsent: [XMLDefaultValueAttributeValidator])
				attribute: attribute
				defaultValue: defaultValue.

	^ self validatorForTypeWithDefaultValueValidator: defaultValueValidator.! !

!DTDAttributeDefinitionParser methodsFor: 'initialization' stamp: 'monty 12/30/2013 23:52'!
setElement: anElement attribute: anAttribute type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	element := anElement.
	attribute := anAttribute.
	type := aType.
	defaultPragma := aDefaultPragma.
	defaultValue := aDefaultValue.! !

!DTDAttributeDefinitionParser methodsFor: 'private ' stamp: 'monty 1/25/2014 01:26'!
validatorClassForType
	type ifEmpty: [XMLValidationException signal: 'Missing attribute type'].

	^ NonEnumerationValidators 
		at: type
		ifAbsent: [
			(self hasNotationType)
				ifTrue: [XMLNotationListAttributeValidator]
				ifFalse: [XMLValueListAttributeValidator]]! !

!DTDAttributeDefinitionParser methodsFor: 'private ' stamp: 'monty 1/25/2014 01:29'!
validatorForTypeWithDefaultValueValidator: aDefaultValueValidator
	| attributeValidator |

	attributeValidator :=
		self validatorClassForType
			element: element
			attribute: attribute
			defaultValueValidator: aDefaultValueValidator.
	attributeValidator isValueListValidator
		ifTrue: [attributeValidator setAllowedValuesFromString: type].

	^ attributeValidator.! !

Object subclass: #DTDContentModelParser
	instanceVariableNames: 'element contentModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDContentModelParser commentStamp: '<historical>' prior: 0!
This is a base class for DTD content model parsers that turn element declarations into validators.!

!DTDContentModelParser class methodsFor: 'instance creation' stamp: 'monty 12/21/2013 03:14'!
element: aName contentModel: aContentModel
	^ self new
		setElement: aName
		contentModel: aContentModel! !

!DTDContentModelParser class methodsFor: 'parsing' stamp: 'monty 12/21/2013 20:00'!
parserForContentModel: aContentModel
	"the tokenizer removes all spaces from the content model"
	^ (aContentModel notEmpty and: [aContentModel first == $(])
		ifTrue: [
			(aContentModel size > 1 and: [aContentModel second == $#])
				ifTrue: [DTDMixedContentModelParser]
				ifFalse: [DTDDFAContentModelParser]]
		ifFalse: [DTDLiteralContentModelParser]! !

!DTDContentModelParser methodsFor: 'parsing' stamp: 'monty 12/21/2013 19:57'!
parse
	self subclassResponsibility! !

!DTDContentModelParser methodsFor: 'initialization' stamp: 'monty 12/21/2013 03:13'!
setElement: aName contentModel: aContentModel
	element := aName.
	contentModel := aContentModel.! !

DTDContentModelParser subclass: #DTDDFAContentModelParser
	instanceVariableNames: 'contentModelStream nameStream operatorStack nfaBuilder'
	classVariableNames: 'OperatorApplicators SpecialCharacterParsers'
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDDFAContentModelParser commentStamp: 'monty 12/29/2013 04:00' prior: 0!
This class parses a content model using postfix conversion and an NFABuilder to build a NFA before converting it to a DFA.!

!DTDDFAContentModelParser class methodsFor: 'class initialization' stamp: 'monty 1/16/2014 05:10'!
initialize
	"self initialize"

	super initialize.
	self
		initializeOperatorApplicators;
		initializeSpecialCharacterParsers.! !

!DTDDFAContentModelParser class methodsFor: 'class initialization' stamp: 'monty 12/29/2013 05:08'!
initializeOperatorApplicators
	"use Array for faster lookup"
	(OperatorApplicators := Array new: 128)
		at: $, asciiValue put: #applyConcatenation;
		at: $| asciiValue put: #applyBranch;
		at: $? asciiValue put: #applyOptional;
		at: $* asciiValue put: #applyStar;
		at: $+ asciiValue put: #applyPlus! !

!DTDDFAContentModelParser class methodsFor: 'class initialization' stamp: 'monty 12/29/2013 05:08'!
initializeSpecialCharacterParsers
	"use Array for faster lookup"
	(SpecialCharacterParsers := Array new: 128)
		at: $( asciiValue put: #parseParen:;
		at: $) asciiValue put: #parseParen:;
		at: $, asciiValue put: #parseBinaryOperator:;
		at: $| asciiValue put: #parseBinaryOperator:;
		at: $? asciiValue put: #parseUnaryOperator:;
		at: $* asciiValue put: #parseUnaryOperator:;
		at: $+ asciiValue put: #parseUnaryOperator:! !

!DTDDFAContentModelParser methodsFor: 'evaluation' stamp: 'monty 12/28/2013 19:46'!
applyBinaryOperatorOnStack
	nfaBuilder totalFragments > 1
		ifFalse: [self errorMissingOperand].

	self applyOperator: operatorStack removeLast.! !

!DTDDFAContentModelParser methodsFor: 'evaluation' stamp: 'monty 12/30/2013 22:29'!
applyOperator: aCharacter
	nfaBuilder perform: (OperatorApplicators at: aCharacter asciiValue)! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 12/18/2013 22:44'!
errorMismatchedParens
	XMLValidationException signal: 'Mismatched parentheses in content model'! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 12/18/2013 22:44'!
errorMissingOperand
	XMLValidationException signal: 'Missing element name in content model'! !

!DTDDFAContentModelParser methodsFor: 'testing' stamp: 'monty 12/29/2013 02:37'!
hasOperatorOnStack
	^ operatorStack notEmpty and: [operatorStack last ~~ $(]! !

!DTDDFAContentModelParser methodsFor: 'testing' stamp: 'monty 12/18/2013 22:44'!
isBinaryOperator: aFirstOperator higherPrecedenceThan: aSecondOperator
	^ aFirstOperator == $, and: [aSecondOperator == $|]! !

!DTDDFAContentModelParser methodsFor: 'testing' stamp: 'monty 1/16/2014 05:09'!
isSpecialCharacter: aCharacter
	^ (self parserForSpecialCharacter: aCharacter) notNil! !

!DTDDFAContentModelParser methodsFor: 'parsing' stamp: 'monty 12/30/2013 22:28'!
parse
	contentModelStream := contentModel readStream.
	nameStream := WriteStream on: (String new: 16).
	operatorStack := OrderedCollection new.
	nfaBuilder := XMLNFABuilder new.

	[contentModelStream atEnd]
		whileFalse: [ 
			(self parserForSpecialCharacter: contentModelStream peek)
				ifNotNil: [:selector |
					self
						perform: selector
						with: contentModelStream next]
				ifNil: [self parseElementName]].
	self popRemainingOperators.

	^ XMLDFAElementValidator
		element: element
		dfa: nfaBuilder firstStateOfAcceptingNFA asDFAState.! !

!DTDDFAContentModelParser methodsFor: 'parsing' stamp: 'monty 12/29/2013 02:23'!
parseBinaryOperator: aCharacter
	self hasOperatorOnStack
		ifTrue: [
			(self
				isBinaryOperator: aCharacter
				higherPrecedenceThan: operatorStack last)
					ifFalse: [self applyBinaryOperatorOnStack]].

	operatorStack addLast: aCharacter.! !

!DTDDFAContentModelParser methodsFor: 'parsing' stamp: 'monty 1/16/2014 05:10'!
parseElementName
	| matcher |

	nameStream reset.
	[contentModelStream atEnd
		or: [self isSpecialCharacter: contentModelStream peek]]
		whileFalse: [nameStream nextPut: contentModelStream next].

	matcher := nameStream contents.
	matcher = '#PCDATA'
		ifTrue: [XMLValidationException signal: 'Unexpected #PCDATA in content model'].

	nfaBuilder pushMatcher: matcher.! !

!DTDDFAContentModelParser methodsFor: 'parsing' stamp: 'monty 12/29/2013 02:24'!
parseParen: aCharacter
	aCharacter == $)
		ifTrue: [self popParenthesizedOperators]
		ifFalse: [operatorStack addLast: $(].! !

!DTDDFAContentModelParser methodsFor: 'parsing' stamp: 'monty 12/29/2013 02:23'!
parseUnaryOperator: aCharacter
	nfaBuilder hasFragments
		ifFalse: [self errorMissingOperand].

	self applyOperator: aCharacter.! !

!DTDDFAContentModelParser methodsFor: 'parsing' stamp: 'monty 1/10/2014 17:07'!
parserForSpecialCharacter: aCharacter
	| asciiValue |

	^ ((asciiValue := aCharacter asciiValue) > 0
		and: [asciiValue <= SpecialCharacterParsers size])
			ifTrue: [SpecialCharacterParsers at: asciiValue]
			ifFalse: [nil]! !

!DTDDFAContentModelParser methodsFor: 'evaluation' stamp: 'monty 12/28/2013 23:02'!
popParenthesizedOperators
	[operatorStack notEmpty]
		whileTrue: [
			operatorStack last == $(
				ifTrue: [
					operatorStack removeLast.
					^ self]
				ifFalse: [self applyBinaryOperatorOnStack]].

	self errorMismatchedParens.! !

!DTDDFAContentModelParser methodsFor: 'evaluation' stamp: 'monty 12/18/2013 22:44'!
popRemainingOperators
	[operatorStack notEmpty]
		whileTrue: [
			operatorStack last == $(
				ifTrue: [self errorMismatchedParens].
			self applyBinaryOperatorOnStack].

	nfaBuilder totalFragments = 1
		ifFalse: [self errorMissingOperand].! !

DTDContentModelParser subclass: #DTDLiteralContentModelParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDLiteralContentModelParser commentStamp: 'monty 2/2/2014 06:12' prior: 0!
Parser for ANY and EMPTY elements.!

!DTDLiteralContentModelParser methodsFor: 'parsing' stamp: 'monty 12/21/2013 03:13'!
parse
	contentModel = 'ANY'
		ifTrue: [^ XMLAnyElementValidator element: element].
	contentModel = 'EMPTY'
		ifTrue: [^ XMLEmptyElementValidator element: element].

	XMLValidationException signal: 'Invalid content model "', contentModel, '"'.! !

DTDContentModelParser subclass: #DTDMixedContentModelParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDMixedContentModelParser commentStamp: 'monty 2/2/2014 06:13' prior: 0!
This class parses mixed content declarations like "(#PCDATA|one|two)" using a DTDMixedContentListParser.!

!DTDMixedContentModelParser methodsFor: 'parsing' stamp: 'monty 12/29/2013 00:44'!
parse
	^ XMLMixedContentElementValidator
		element: element
		allowedElements: self parseAllowedElements! !

!DTDMixedContentModelParser methodsFor: 'parsing' stamp: 'monty 12/29/2013 00:43'!
parseAllowedElements
	| allowedElements |

	allowedElements := (DTDMixedContentListParser on: contentModel) parse.
	allowedElements remove: '#PCDATA'.

	(allowedElements isEmpty or: [contentModel last == $*])
		ifFalse: [
			XMLValidationException signal:
				'Missing * after mixed content list with elements'].

	^ allowedElements.! !

Object subclass: #DTDDoctypeDefinition
	instanceVariableNames: 'rootElement externalSubsetUri isStandalone elementValidators attributeValidators generalEntities parameterEntities unparsedEntities notations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDDoctypeDefinition commentStamp: '<historical>' prior: 0!
This class represents the parsed internal and external subset of a DTD.!

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 1/16/2014 05:41'!
addExternalGeneralEntity: aName resolver: aResolver uri: aSystemUri from: aDocumentOrEntityUri
	self generalEntities
		at: aName
		ifAbsentPut: [
			aResolver
				resolveExternalGeneralEntity: aName
				uri: aSystemUri
				from: aDocumentOrEntityUri]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 1/16/2014 05:41'!
addExternalParameterEntity: aName resolver: aResolver uri: aSystemUri from: aDocumentOrEntityUri
	self parameterEntities
		at: aName
		ifAbsentPut: [
			aResolver
				resolveExternalParameterEntity: aName
				uri: aSystemUri
				from: aDocumentOrEntityUri]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 1/4/2014 03:44'!
addGeneralEntity: aName replacement: aReplacement from: aDocumentOrEntityUri
	self generalEntities
		at: aName
		ifAbsentPut: [
			DTDGeneralEntity 
				name: aName
				replacement: aReplacement
				uri: aDocumentOrEntityUri]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 11/14/2013 23:59'!
addNotation: aName
	self notations add: aName! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 1/4/2014 03:45'!
addParameterEntity: aName replacement: aReplacement from: aDocumentOrEntityUri
	self parameterEntities
		at: aName
		ifAbsentPut: [
			DTDParameterEntity
				name: aName
				replacement: aReplacement
				uri: aDocumentOrEntityUri]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 1/4/2014 03:49'!
addUnparsedEntity: aName ndata: aNotation
	self unparsedEntities
		at: aName
		ifAbsentPut: [aNotation]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 2/2/2014 23:31'!
addValidatorForAttributeIn: anElement named: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	| attributeValidatorsForElement validator |

	attributeValidatorsForElement :=
		self attributeValidators
			at: anElement
			ifAbsentPut: [Dictionary new].
	(attributeValidatorsForElement includesKey: aName)
		ifTrue: [^ self].

	validator :=
		(DTDAttributeDefinitionParser
			element:  anElement
			attribute: aName
			type: aType
			defaultPragma: aDefaultPragma
			defaultValue: aDefaultValue) parse.

	validator
		validateAndAddTo: attributeValidatorsForElement
		usingDeclarationsFrom: self.! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 12/21/2013 03:14'!
addValidatorForElement: anElement contentModel: aContentModel
	self elementValidators
		at: anElement
		ifAbsentPut: [
			((DTDContentModelParser parserForContentModel: aContentModel)
				element: anElement
				contentModel: aContentModel) parse]! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/19/2013 01:01'!
attributeValidators
	^ attributeValidators ifNil: [attributeValidators := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/18/2013 19:26'!
elementValidators
	^ elementValidators ifNil: [elementValidators := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 1/4/2014 03:48'!
externalSubsetURI
	^ externalSubsetUri ifNil: [externalSubsetUri := '']! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/14/2013 20:51'!
generalEntities
	^ generalEntities ifNil: [generalEntities := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/14/2013 20:50'!
generalEntityAt: aName
	^ self generalEntities at: aName ifAbsent: [nil]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 11/18/2013 19:45'!
hasAttributeValidators
	^ attributeValidators notNil and: [attributeValidators notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 11/18/2013 19:45'!
hasElementValidators
	^ elementValidators notNil and: [elementValidators notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 1/4/2014 03:49'!
hasExternalSubset
	^ self isStandalone not and: [self externalSubsetURI notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 11/15/2013 00:00'!
includesNotation: aName
	^ self notations includes: aName! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 11/22/2013 09:11'!
isStandalone
	^ isStandalone ifNil: [isStandalone := false]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/22/2013 09:11'!
isStandalone: aBoolean
	isStandalone := aBoolean! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/14/2013 23:59'!
notations
	^ notations ifNil: [notations := Set new]! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/14/2013 20:51'!
parameterEntities
	^ parameterEntities ifNil: [parameterEntities := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/14/2013 20:51'!
parameterEntityAt: aName
	^ self parameterEntities at: aName ifAbsent: [nil]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/19/2013 23:42'!
rootElement
	^ rootElement ifNil: ['']! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 1/4/2014 03:48'!
rootElement: aRootElement externalSubsetURI: aUri
	rootElement := aRootElement.
	externalSubsetUri := aUri.! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/14/2013 23:50'!
unparsedEntities
	^ unparsedEntities ifNil: [unparsedEntities := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/14/2013 23:55'!
unparsedEntityAt: aName
	^ self unparsedEntities at: aName ifAbsent: [nil]! !

Object subclass: #DTDExternalEntityLimits
	instanceVariableNames: 'maxExternalEntities maxExternalEntitySize resolvesLocalFromNonLocal'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalEntityLimits commentStamp: 'monty 1/25/2014 04:01' prior: 0!
This class places security limits on the number and size of external parsed entities. You can set these using the configuration messages in DTDExternalEntityResolver or remove them using removeLimits.!

!DTDExternalEntityLimits class methodsFor: 'accessing' stamp: 'monty 10/3/2014 23:23'!
defaultMaxExternalEntitySize
	^ XMLParserLimits defaultExternalSubsetReadLimit! !

!DTDExternalEntityLimits methodsFor: 'initialization' stamp: 'monty 1/26/2014 04:19'!
initialize
	super initialize.

	maxExternalEntities := 12.
	maxExternalEntitySize := self class defaultMaxExternalEntitySize.
	resolvesLocalFromNonLocal := false.! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 06:14'!
maxExternalEntities
	^ maxExternalEntities! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 06:14'!
maxExternalEntities: aLimit
	maxExternalEntities := aLimit! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 06:14'!
maxExternalEntitySize
	^ maxExternalEntitySize! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 06:14'!
maxExternalEntitySize: aLimit
	maxExternalEntitySize := aLimit! !

!DTDExternalEntityLimits methodsFor: 'removing' stamp: 'monty 1/24/2014 06:48'!
removeAll
	maxExternalEntities := nil.
	maxExternalEntitySize := nil.
	resolvesLocalFromNonLocal := nil.! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/24/2014 06:47'!
resolvesLocalFromNonLocal
	^ resolvesLocalFromNonLocal! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/24/2014 06:47'!
resolvesLocalFromNonLocal: aBoolean
	resolvesLocalFromNonLocal := aBoolean! !

Object subclass: #DTDExternalEntityResolver
	instanceVariableNames: 'externalEntityLimits totalResolvedEntities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalEntityResolver commentStamp: 'monty 1/23/2014 05:48' prior: 0!
This class resolves external entity references using XMLURI, XMLHTTPRequest, and XMLFileHandle.!

DTDExternalEntityResolver subclass: #DTDCachingEntityResolver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

DTDCachingEntityResolver class
	instanceVariableNames: 'entityCache maxCachedEntities'!

!DTDCachingEntityResolver commentStamp: 'monty 1/10/2014 08:16' prior: 0!
This resolver can cache entities so they don't have to be retrieved again (like over HTTP).!

DTDCachingEntityResolver class
	instanceVariableNames: 'entityCache maxCachedEntities'!

!DTDCachingEntityResolver class methodsFor: 'clearing' stamp: 'monty 1/10/2014 08:24'!
clearEntityCache
	"self clearEntityCache"
	self entityCache removeAll! !

!DTDCachingEntityResolver class methodsFor: 'private' stamp: 'monty 1/10/2014 07:22'!
entityCache
	^ entityCache ifNil: [entityCache := Dictionary new]! !

!DTDCachingEntityResolver class methodsFor: 'private' stamp: 'monty 1/10/2014 16:32'!
entityCacheAt: aUri ifAbsentPut: aBlock
	^ self entityCache
		at: aUri
		ifAbsent: [
			self
				entityCacheAt: aUri
				put: aBlock value]! !

!DTDCachingEntityResolver class methodsFor: 'private' stamp: 'monty 1/10/2014 07:27'!
entityCacheAt: aUri put: anEntity
	self entityCache size >= self maxCachedEntities
		ifTrue: [self clearEntityCache].

	^ self entityCache
		at: aUri
		put: anEntity.! !

!DTDCachingEntityResolver class methodsFor: 'accessing' stamp: 'monty 1/10/2014 07:57'!
maxCachedEntities
	^ maxCachedEntities ifNil: [maxCachedEntities := 64]! !

!DTDCachingEntityResolver class methodsFor: 'accessing' stamp: 'monty 1/10/2014 07:22'!
maxCachedEntities: anInteger
	maxCachedEntities := anInteger! !

!DTDCachingEntityResolver methodsFor: 'clearing' stamp: 'monty 1/13/2014 03:45'!
clearEntityCache
	self class clearEntityCache! !

!DTDCachingEntityResolver methodsFor: 'accessing' stamp: 'monty 1/13/2014 17:52'!
maxCachedEntities
	^ self class maxCachedEntities! !

!DTDCachingEntityResolver methodsFor: 'accessing' stamp: 'monty 1/13/2014 17:53'!
maxCachedEntities: anInteger
	self class maxCachedEntities: anInteger! !

!DTDCachingEntityResolver methodsFor: 'resolving' stamp: 'monty 1/16/2014 06:38'!
resolveEntityURI: aUri
	| uri |

	^ (uri := aUri asXMLURI) isLocalURI
		ifTrue: [super resolveEntityURI: uri]
		ifFalse: [
			ReadStream on:
				(self class
					entityCacheAt: uri
					ifAbsentPut: [(super resolveEntityURI: uri) contents])]! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 1/23/2014 05:43'!
checkExternalEntityLimit
	(self maxExternalEntities notNil
		and: [self totalResolvedEntities + 1 > self maxExternalEntities])
		ifTrue: [XMLLimitException signal: 'External entity limit exceeded'].! !

!DTDExternalEntityResolver methodsFor: 'combining' stamp: 'monty 1/24/2014 06:49'!
combineURI: aBaseUri with: aRelativeUri
	| baseUri combinedUri |

	baseUri := XMLURI fromString: aBaseUri.
	combinedUri := baseUri / aRelativeUri.
	(combinedUri isLocalURI
		and: [baseUri isLocalURI not
			and: [self resolvesLocalFromNonLocal notNil
				and: [self resolvesLocalFromNonLocal not]]])
		ifTrue: [XMLLimitException signal: 'Local URI referenced from non-local URI'].

	^ combinedUri.! !

!DTDExternalEntityResolver methodsFor: 'accessing' stamp: 'monty 1/22/2014 18:55'!
defaultDocumentURI
	^ XMLFileHandle currentWorkingDirectory, '/default'! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 1/16/2014 06:15'!
externalEntityLimits
	^ externalEntityLimits ifNil: [externalEntityLimits := DTDExternalEntityLimits new]! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 1/23/2014 05:45'!
incrementEntityTotal
	totalResolvedEntities := (totalResolvedEntities ifNil: [0]) + 1! !

!DTDExternalEntityResolver methodsFor: 'configuration' stamp: 'monty 1/16/2014 06:17'!
maxExternalEntities
	^ self externalEntityLimits maxExternalEntities! !

!DTDExternalEntityResolver methodsFor: 'configuration' stamp: 'monty 1/16/2014 06:54'!
maxExternalEntities: aLimit
	self externalEntityLimits maxExternalEntities: aLimit! !

!DTDExternalEntityResolver methodsFor: 'configuration' stamp: 'monty 1/16/2014 06:20'!
maxExternalEntitySize
	^ self externalEntityLimits maxExternalEntitySize! !

!DTDExternalEntityResolver methodsFor: 'configuration' stamp: 'monty 1/16/2014 06:54'!
maxExternalEntitySize: aLimit
	self externalEntityLimits maxExternalEntitySize: aLimit! !

!DTDExternalEntityResolver methodsFor: 'configuration' stamp: 'monty 1/16/2014 06:16'!
removeLimits
	self externalEntityLimits removeAll! !

!DTDExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 1/23/2014 05:44'!
resolveEntityURI: aUri
	| readStream |

	self checkExternalEntityLimit.
	readStream := aUri asXMLURI getUpToLimit: self maxExternalEntitySize.
	self incrementEntityTotal.

	^ readStream.! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 2/18/2014 04:58'!
resolveExternalEntity: aName uri: anEntityUri from: aSourceUri class: aClass
	| combinedUri replacement replacementStream |

	combinedUri := self combineURI: aSourceUri with: anEntityUri.

	replacementStream := self resolveEntityURI: combinedUri.
	replacement  := replacementStream contents.
	replacementStream close.

	^ aClass
		name: aName
		replacement: replacement
		uri: combinedUri asString.! !

!DTDExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 1/16/2014 05:41'!
resolveExternalGeneralEntity: aName uri: anEntityUri from: aSourceUri
	^ self
		resolveExternalEntity: aName
		uri: anEntityUri
		from: aSourceUri
		class: DTDExternalGeneralEntity! !

!DTDExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 1/16/2014 05:41'!
resolveExternalParameterEntity: aName uri: anEntityUri from: aSourceUri
	^ self
		resolveExternalEntity: aName
		uri: anEntityUri
		from: aSourceUri
		class: DTDExternalParameterEntity! !

!DTDExternalEntityResolver methodsFor: 'configuration' stamp: 'monty 1/24/2014 06:50'!
resolvesLocalFromNonLocal
	^ self externalEntityLimits resolvesLocalFromNonLocal! !

!DTDExternalEntityResolver methodsFor: 'configuration' stamp: 'monty 1/24/2014 06:50'!
resolvesLocalFromNonLocal: aBoolean
	self externalEntityLimits resolvesLocalFromNonLocal: aBoolean! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 1/16/2014 06:40'!
totalResolvedEntities
	^ totalResolvedEntities ifNil: [totalResolvedEntities := 0]! !

Object subclass: #DTDParsedEntity
	instanceVariableNames: 'name replacement uri'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDParsedEntity commentStamp: '<historical>' prior: 0!
This is a base class for internal and external parsed entities with replacements that can be inserted into an XML document.!

DTDParsedEntity subclass: #DTDGeneralEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDGeneralEntity commentStamp: 'monty 1/5/2014 00:54' prior: 0!
This class is for general parsed entities with replacements.
!

DTDGeneralEntity subclass: #DTDBypassedGeneralEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDBypassedGeneralEntity commentStamp: 'monty 10/7/2014 02:10' prior: 0!
A bypassed general entity is replaced by an escaped reference to itself.!

!DTDBypassedGeneralEntity methodsFor: 'replacement' stamp: 'monty 12/7/2013 19:13'!
replacement
	^ self asEscapedReference! !

DTDGeneralEntity subclass: #DTDExternalGeneralEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalGeneralEntity commentStamp: 'monty 10/7/2014 02:19' prior: 0!
A general entity with an external replacement.!

!DTDExternalGeneralEntity methodsFor: 'testing' stamp: 'monty 11/16/2013 02:57'!
isExternal
	^ true! !

!DTDGeneralEntity methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:03'!
escapedReferenceSymbol
	^ '&amp;'! !

!DTDGeneralEntity methodsFor: 'accessing' stamp: 'monty 11/12/2013 20:09'!
referenceSymbol
	^ $&! !

!DTDGeneralEntity methodsFor: 'replacement' stamp: 'monty 11/14/2013 20:22'!
replacementForContentContext
	^ self replacement! !

DTDParsedEntity subclass: #DTDParameterEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDParameterEntity commentStamp: 'monty 1/5/2014 00:54' prior: 0!
This class is for paramter entities with replacements.!

DTDParameterEntity subclass: #DTDBypassedParameterEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDBypassedParameterEntity commentStamp: 'monty 10/7/2014 02:18' prior: 0!
A bypassed parameter entity is replaced by an escaped reference inside entity declaration replacements or an empty string outside replacements in DTD subsets.!

!DTDBypassedParameterEntity methodsFor: 'replacement' stamp: 'monty 12/7/2013 19:14'!
replacement
	^ self asEscapedReference! !

!DTDBypassedParameterEntity methodsFor: 'replacement' stamp: 'monty 12/7/2013 19:14'!
replacementForDTDContext
	^ ''! !

DTDParameterEntity subclass: #DTDExternalParameterEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalParameterEntity commentStamp: 'monty 10/7/2014 02:19' prior: 0!
A parameter entity with an external replacement.!

!DTDExternalParameterEntity methodsFor: 'testing' stamp: 'monty 11/16/2013 02:58'!
isExternal
	^ true! !

!DTDParameterEntity methodsFor: 'accessing' stamp: 'monty 12/7/2013 20:23'!
escapedReferenceSymbol
	^ '&#37;'! !

!DTDParameterEntity methodsFor: 'accessing' stamp: 'monty 11/12/2013 20:09'!
referenceSymbol
	^ $%! !

!DTDParameterEntity methodsFor: 'replacement' stamp: 'monty 12/7/2013 19:41'!
replacementForDTDContext
	^ (self replacement copyWithFirst: Character space) copyWith: Character space.! !

!DTDParsedEntity class methodsFor: 'instance creation' stamp: 'monty 11/16/2013 06:02'!
name: aString replacement: aValueString
	^ self new
		setName: aString
		replacement: aValueString
		uri: nil! !

!DTDParsedEntity class methodsFor: 'instance creation' stamp: 'monty 11/16/2013 02:55'!
name: aString replacement: aValueString uri: aUri
	^ self new
		setName: aString
		replacement: aValueString
		uri: aUri! !

!DTDParsedEntity class methodsFor: 'instance creation' stamp: 'monty 12/7/2013 19:37'!
named: aString
	^ self
		name: aString
		replacement: ''! !

!DTDParsedEntity methodsFor: 'converting' stamp: 'monty 12/7/2013 19:01'!
asEscapedReference
	^ self asReferenceStartingWith: self escapedReferenceSymbol! !

!DTDParsedEntity methodsFor: 'converting' stamp: 'monty 12/7/2013 19:11'!
asReference
	^ self asReferenceStartingWith: self referenceSymbol asString! !

!DTDParsedEntity methodsFor: 'private' stamp: 'monty 12/9/2013 01:29'!
asReferenceStartingWith: aString
	^ (aString, self name) copyWith: $;! !

!DTDParsedEntity methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:02'!
escapedReferenceSymbol
	self subclassResponsibility! !

!DTDParsedEntity methodsFor: 'testing' stamp: 'monty 1/16/2014 09:33'!
hasTextDeclaration
	^ self replacement beginsWith: '<?xml '! !

!DTDParsedEntity methodsFor: 'testing' stamp: 'monty 11/16/2013 02:56'!
isExternal
	^ false! !

!DTDParsedEntity methodsFor: 'accessing' stamp: 'monty 11/13/2013 23:45'!
name
	^ name ifNil: [name := '']! !

!DTDParsedEntity methodsFor: 'printing' stamp: 'monty 11/20/2013 01:23'!
printOn: aStream
	self asReference printOn: aStream
	! !

!DTDParsedEntity methodsFor: 'accessing' stamp: 'monty 11/12/2013 20:09'!
referenceSymbol
	self subclassResponsibility! !

!DTDParsedEntity methodsFor: 'replacement' stamp: 'monty 11/13/2013 19:15'!
replacement
	^ replacement ifNil: [replacement := '']! !

!DTDParsedEntity methodsFor: 'replacement' stamp: 'monty 12/29/2013 04:33'!
replacementForLiteralContext
	| aReplacement |

	aReplacement := (String new: self replacement size) writeStream.
	self replacement do: [:each |
		each ==  $"
			ifTrue: [aReplacement nextPutAll: '&#34;']
			ifFalse: [
				each == $'
					ifTrue: [aReplacement nextPutAll: '&#39;']
					ifFalse: [aReplacement nextPut: each]]].
	^ aReplacement contents.! !

!DTDParsedEntity methodsFor: 'initialization' stamp: 'monty 11/16/2013 02:55'!
setName: aName replacement: aReplacement uri: aUri
	name := aName.
	replacement := aReplacement.
	uri := aUri.! !

!DTDParsedEntity methodsFor: 'accessing' stamp: 'monty 11/16/2013 02:55'!
uri
	^ uri ifNil: [uri := '']! !

Object subclass: #DTDValueListParser
	instanceVariableNames: 'valueListStream valueStream listValues'
	classVariableNames: 'ListValueDelimiters'
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDValueListParser commentStamp: '<historical>' prior: 0!
This is an abstract parser of lists in the form of "(one | two | three)"!

DTDValueListParser subclass: #DTDEnumerationListParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDEnumerationListParser commentStamp: 'monty 1/25/2014 03:20' prior: 0!
This parses attribute value enumeration lists.!

!DTDEnumerationListParser methodsFor: 'testing' stamp: 'monty 1/25/2014 03:38'!
isValidListValue: aValue
	^ true! !

DTDValueListParser subclass: #DTDMixedContentListParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDMixedContentListParser commentStamp: 'monty 1/25/2014 03:19' prior: 0!
This parses element mixed content  (#PCDATA| ... ) lists.!

!DTDMixedContentListParser methodsFor: 'testing' stamp: 'monty 2/10/2014 02:43'!
isValidListValue: aValue
	^ aValue = '#PCDATA' or: [XMLWellFormedParserTokenizer isName: aValue]! !

!DTDMixedContentListParser methodsFor: 'parsing' stamp: 'monty 12/31/2013 02:46'!
parseAfterList
	(valueListStream atEnd
		or: [valueListStream next == $*
			and: [valueListStream atEnd]])
		ifFalse: [
			XMLValidationException signal:
				'Mixed content list can only be followed by "*"'].! !

DTDValueListParser subclass: #DTDNotationListParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDNotationListParser commentStamp: 'monty 1/25/2014 03:36' prior: 0!
This class parses notation attribute value lists.!

!DTDNotationListParser methodsFor: 'testing' stamp: 'monty 2/10/2014 02:43'!
isValidListValue: aValue
	^ XMLWellFormedParserTokenizer isName: aValue! !

!DTDNotationListParser methodsFor: 'accessing' stamp: 'monty 1/4/2014 04:45'!
valueList: aString
	super valueList:
		(aString
			copyFrom: (aString indexOf: $()
			to: aString size)! !

!DTDValueListParser class methodsFor: 'class initialization' stamp: 'monty 1/21/2014 23:44'!
initialize
	"self initialize"

	super initialize.

	(ListValueDelimiters := BitmapCharacterSet new)
		add: $|;
		add: $);
		addAll: Character separators! !

!DTDValueListParser class methodsFor: 'instance creation' stamp: 'monty 12/9/2013 02:27'!
on: aString
	^ self new valueList: aString! !

!DTDValueListParser methodsFor: 'testing' stamp: 'monty 12/31/2013 02:08'!
isValidListValue: aValue
	self subclassResponsibility! !

!DTDValueListParser methodsFor: 'parsing' stamp: 'monty 1/16/2014 09:13'!
parse
	(valueListStream atEnd not
		and: [valueListStream next == $(])
		ifFalse: [XMLValidationException signal: 'Expected value list'].

	[valueListStream atEnd
		or: [valueListStream peek == $)]]
		whileFalse: [
			valueListStream skipSeparators.
			self parseListValue.
			valueListStream skipSeparators.

			(valueListStream atEnd not
				and: [valueListStream peek == $|])
				ifTrue: [valueListStream next]].
	self parseListTerminator.

	^ listValues.! !

!DTDValueListParser methodsFor: 'parsing' stamp: 'monty 12/31/2013 02:46'!
parseAfterList! !

!DTDValueListParser methodsFor: 'parsing' stamp: 'monty 12/31/2013 02:46'!
parseListTerminator
	(valueListStream atEnd not
		and: [valueListStream next == $)])
		ifFalse: [XMLValidationException signal: 'Missing list terminator ")"'].

	self parseAfterList.! !

!DTDValueListParser methodsFor: 'parsing' stamp: 'monty 2/2/2014 07:59'!
parseListValue
	| value oldSize |

	valueStream reset.
	[valueListStream atEnd
		or: [ListValueDelimiters includes: valueListStream peek]]
		whileFalse: [valueStream nextPut: valueListStream next].

	(value := valueStream contents)
		ifEmpty: [XMLValidationException signal: 'Empty list value'].
	(self isValidListValue: value)
		ifFalse: [XMLValidationException signal: 'Invalid value "', value, '" in list value'].

	"checking the size avoids includes: hash lookup"
	oldSize := listValues size.
	listValues add: value.
	(listValues size > oldSize)
		ifFalse: [XMLValidationException signal: 'Duplicate "', value, '" in list'].! !

!DTDValueListParser methodsFor: 'accessing' stamp: 'monty 12/29/2013 04:07'!
valueList: aString
	valueListStream := ReadStream on: aString.
	valueStream := WriteStream on: (String new: 16).
	listValues := Set new.! !

Object subclass: #SAXHandler
	instanceVariableNames: 'input documentUri configuration isInDtd isInCDataSection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAXHandler commentStamp: 'monty 10/3/2014 23:17' prior: 0!
This class is an XML parser that uses event handling. The acronym "SAX" refers to Java's "Simple API for XML," on which SAXHandler's API is based. To use this class, create a subclass and override handlers under the "content" and "lexical" categories as needed. By default, namespace support and validation are enabled (but not external entity resolution).

To instantiate a SAXHandler, send any of the #on: or #parse- messages to a subclass.

The class-side #parse- messages take some input source and parse it immediately, returning the result of #parseDocument:
	SAXHandlerSubclass parse: xml.
	SAXHandlerSubclass parseURL: aUrl. 
	SAXHandlerSubclass parse: xml usingNamespaces: false

The #on: messages create new parsers on the given input string, stream, URL, or file and return it; the instance can then be further configured and ultimately sent #parseDocument to parse the given input:
	(SAXHandlerSubclass on: xml)
		isValidating: true;
		resolvesExternalEntities: true;
		parseDocument.

There is also #parseDocumentWhile: to stop parsing before the end.
	(SAXHandlerSubclass on: xml)
		parseDocumentWhile: [self shouldKeepParsing].

To parse incrementally, send #parser to an instance to get the underlying XMLParser object and send it #parseToken repeatedly:
	(handler := SAXHandlerSubclass on: xml)
		isValidating: false; 
		preservesUndeclaredEntityReferences: true.
	parser := handler parser.

	"Only parse the first 10 tokens:"
	10 timesRepeat: [parser parseToken].

There are security limits on input you can remove with #removeLimits or change with messages like #documentReadLimit:
	(SAXHandlerSubclass on: xml)
		removeLimits;
		documentReadLimit: newReadLimit;
		maxEntityReplacementDepth: newMaxEntityDepth;
		parseDocument.!

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/27/2010 14:06'!
on: aStringOrStream
	^ self new input: aStringOrStream! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 11/15/2013 21:13'!
on: aStringOrStream documentURI: aUri
	^ self new
		input: aStringOrStream
		documentURI: aUri! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 1/21/2014 23:36'!
onFileNamed: aFileName
	^ self
		on: (XMLFileHandle path: aFileName) readStream
		documentURI: aFileName! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 1/25/2014 19:03'!
onURL: aUrlString
	^ self
		onURL: aUrlString
		upToLimit: XMLParserLimits defaultDocumentReadLimit! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 1/25/2014 19:09'!
onURL: aUrlString upToLimit: aLimit
	^ (self
		on: ((XMLHTTPRequest url: aUrlString) getUpToLimit: aLimit)
		documentURI: aUrlString)
		documentReadLimit: aLimit! !

!SAXHandler class methodsFor: 'parsing' stamp: 'JAAyer 11/10/2010 05:21'!
parse: aStringOrStream
	^ (self on: aStringOrStream) parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'JAAyer 11/10/2010 05:30'!
parse: aStringOrStream usingNamespaces: aBoolean
	^ (self on: aStringOrStream)
		usesNamespaces: aBoolean;
		parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 11/15/2013 23:33'!
parse: aStringOrStream usingNamespaces: aNamespaceBoolean validation: aValidationBoolean
	^ (self on: aStringOrStream)
		usesNamespaces: aNamespaceBoolean;
		isValidating: aValidationBoolean;
		parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 11/16/2013 07:30'!
parse: aStringOrStream usingNamespaces: aNamespaceBoolean validation: aValidationBoolean externalEntities: anEntityBoolean
	^ (self on: aStringOrStream)
		usesNamespaces: aNamespaceBoolean;
		isValidating: aValidationBoolean;
		resolvesExternalEntities: anEntityBoolean;
		parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'JAAyer 11/13/2010 20:43'!
parseFileNamed: aFileName
	^ (self onFileNamed: aFileName) parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 11/15/2013 23:35'!
parseURL: aUrl
	^ (self onURL: aUrl) parseDocument! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 12/30/2013 23:57'!
attributeDeclaration: anElement name: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue! !

!SAXHandler methodsFor: 'content' stamp: 'mir 1/8/2002 18:27'!
characters: aString
	"This call corresponds to the Java SAX call
	characters(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 11/10/2010 19:23'!
comment: aCommentString
	"This call corresponds to the Java SAX ext call
	comment(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:37'!
configuration
	^ configuration ifNil: [configuration := self configurationClass new]! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:13'!
configuration: aConfiguration
	configuration := aConfiguration! !

!SAXHandler methodsFor: 'private' stamp: 'JAAyer 11/23/2010 15:35'!
configurationClass
	^ XMLConfiguration! !

!SAXHandler methodsFor: 'private' stamp: 'monty 12/27/2013 03:36'!
currentScope: aNamespaceScope! !

!SAXHandler methodsFor: 'private' stamp: 'monty 1/9/2014 05:28'!
defaultDocumentURI
	^ self externalEntityResolver defaultDocumentURI! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 1/19/2014 03:51'!
documentReadLimit
	^ self parserLimits documentReadLimit! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 10/3/2014 00:11'!
documentReadLimit: aLimit
	"change the document read limit or nil to disable"

	self parserLimits documentReadLimit: aLimit! !

!SAXHandler methodsFor: 'accessing' stamp: 'monty 11/15/2013 21:14'!
documentURI
	^ documentUri ifNil: [documentUri := self defaultDocumentURI]! !

!SAXHandler methodsFor: 'private' stamp: 'monty 2/9/2014 02:43'!
driverClass
	^ SAXParserDriver! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 11/13/2013 23:54'!
elementDeclaration: aName contentModel: aContentModel
! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 2/21/2010 15:13'!
endCData
	"This call corresponds to the Java SAX ext call
	endCData()."! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 11/12/2013 18:01'!
endDTD
! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/27/2010 06:50'!
endDocument
	"This call corresponds to the Java SAX call
	endDocument()."! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/10/2010 19:23'!
endElement: aQualifiedName
! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 3/24/2010 16:50'!
endElement: aQualifiedName prefix: aPrefix uri: aUri
	"This call corresponds to the Java SAX call
	endElement(java.lang.String namespaceURI, java.lang.String localName, java.lang.String qName).
	By default this call is mapped to the following more convenient call:"

	self endElement: aQualifiedName! !

!SAXHandler methodsFor: 'content' stamp: 'monty 11/14/2013 00:13'!
endElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName
	"This call corresponds to the Java SAX call
	endElement(java.lang.String namespaceURI, java.lang.String localName, java.lang.String qName).
	By default this call is mapped to the following more convenient call:"

	self endElement: aQualifiedName prefix: aPrefix uri:  aUri! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 11/10/2010 19:23'!
endEntity: anEntityName
	"This call corresponds to the Java SAX ext call
	endEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'content' stamp: 'JAA 3/22/2011 11:32'!
endPrefixMapping: aPrefix
	"This call corresonds to the Java SAX call
	endPrefixMapping(java.lang.String prefix)."! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/8/2013 02:33'!
externalEntityResolver
	^ self configuration externalEntityResolver! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/29/2013 01:55'!
externalEntityResolver: anExternalEntityResolver
	"This can be used to set a different DTDExternalEntityResolver"

	self configuration externalEntityResolver: anExternalEntityResolver! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 1/19/2014 03:52'!
externalSubsetReadLimit
	^ self parserLimits externalSubsetReadLimit! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 10/3/2014 00:11'!
externalSubsetReadLimit: aLimit
	"change the external subset read limit or nil to disable"

	self parserLimits externalSubsetReadLimit: aLimit! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 11/14/2013 04:01'!
generalEntityDeclaration: aName publicID: aPublicId systemID: aSystemId ndata: aNotation! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 11/14/2013 04:02'!
generalEntityDeclaration: aName replacement: aReplacement! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/11/2000 16:25'!
ignorableWhitespace: aString
	"This call corresonds to the Java SAX call
	ignorableWhitespace(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 14:05'!
input
	^ input! !

!SAXHandler methodsFor: 'private' stamp: 'JAAyer 11/27/2010 14:05'!
input: aStringOrStream
	input := aStringOrStream.! !

!SAXHandler methodsFor: 'private' stamp: 'monty 11/15/2013 21:13'!
input: aStringOrStream documentURI: aUri
	input := aStringOrStream.
	documentUri := aUri.! !

!SAXHandler methodsFor: 'private' stamp: 'monty 10/4/2014 02:55'!
isInCDataSection
	^ isInCDataSection ifNil: [isInCDataSection := false]! !

!SAXHandler methodsFor: 'private' stamp: 'monty 10/4/2014 06:59'!
isInCDataSection: aBoolean
	isInCDataSection := aBoolean! !

!SAXHandler methodsFor: 'private' stamp: 'monty 10/5/2014 20:29'!
isInDTD
	^ isInDtd ifNil: [isInDtd := false]! !

!SAXHandler methodsFor: 'private' stamp: 'monty 10/5/2014 20:29'!
isInDTD: aBoolean
	isInDtd := aBoolean! !

!SAXHandler methodsFor: 'configuration' stamp: 'JAAyer 11/23/2010 14:20'!
isValidating
	^ self configuration isValidating! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/29/2013 03:00'!
isValidating: aBoolean
	"If true and document contstraints are present (in a DTD),
	the document will be checked against them (default).

	Enabling disables preservesUndeclaredEntityReferences.
	Disabling disables requiresDocumentConstraints."

	self configuration isValidating: aBoolean! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 1/19/2014 03:52'!
maxEntityReplacementDepth
	^ self parserLimits maxEntityReplacementDepth! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 10/3/2014 00:11'!
maxEntityReplacementDepth: aLimit
	"change the max replacement depth or nil to disable"

	self parserLimits maxEntityReplacementDepth: aLimit! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 10/4/2014 03:34'!
notationDeclaration: aName publicID: aPublicId systemID: aSystemId
! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 11/14/2013 04:05'!
parameterEntityDeclaration: aName publicID: aPublicId systemID: aSystemId! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 11/14/2013 04:05'!
parameterEntityDeclaration: aName replacement: aReplacement! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/19/2013 00:37'!
parseDocument
	self parser parseDocument.

	^ self parsingResult.! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/19/2013 07:17'!
parseDocumentWhile: aBlock
	self parser parseDocumentWhile: aBlock.

	^ self parsingResult.! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 10/3/2014 00:18'!
parser
	self configuration parserHandlerClass: self class.

	^ self parserClass
		driver: (self driverClass saxHandler: self)
		on: input.! !

!SAXHandler methodsFor: 'private' stamp: 'monty 10/3/2014 00:18'!
parserClass
	^ XMLParser! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 1/16/2014 18:46'!
parserLimits
	^ self configuration parserLimits! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/19/2013 00:37'!
parsingResult
	^ self! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/7/2013 19:18'!
preservesUndeclaredEntityReferences
	^ self configuration preservesUndeclaredEntityReferences! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/29/2013 03:01'!
preservesUndeclaredEntityReferences: aBoolean
	"If true, references to undeclared entities in content are
	preserved (not default).
	Disables validation"

	self configuration preservesUndeclaredEntityReferences: aBoolean! !

!SAXHandler methodsFor: 'content' stamp: 'JAA 3/22/2011 11:31'!
processingInstruction: aTarget data: aDataString
	"This call corresonds to the Java SAX call
	processingInstruction(java.lang.String target, java.lang.String data)."! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 10/3/2014 00:16'!
removeLimits
	"remove all input security limits (unsafe)"

	self parserLimits removeAll.
	self externalEntityResolver removeLimits.! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/21/2013 20:07'!
requiresDocumentConstraints
	^ self configuration requiresDocumentConstraints! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/29/2013 03:23'!
requiresDocumentConstraints: aBoolean
	"If true, constraints on elements and attributes (in a DTD)
	will be required (not default).
	Enables validation."

	self configuration requiresDocumentConstraints: aBoolean! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 11/16/2013 07:33'!
resolvesExternalEntities
	^ self configuration resolvesExternalEntities! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 1/24/2014 23:52'!
resolvesExternalEntities: aBoolean
	"When true, external DTDs and parsed entities will be
	fetched (not default)."

	self configuration resolvesExternalEntities: aBoolean! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/10/2010 19:22'!
skippedEntity: anEntityName
	"This call corresonds to the Java SAX call
	skippedEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'lexical' stamp: 'mir 8/11/2000 18:53'!
startCData
	"This call corresponds to the Java SAX ext call
	startCData()."! !

!SAXHandler methodsFor: 'declaration' stamp: 'JAAyer 11/10/2010 19:21'!
startDTD: aRootElementName publicID: aPublicId systemID: aSystemId
	"This call corresponds to the Java SAX ext call
	startDTD(java.lang.String name, java.lang.String publicId, java.lang.String systemId)."! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/11/2000 16:45'!
startDocument
	"This call corresonds to the Java SAX call
	startDocument()."! !

!SAXHandler methodsFor: 'content' stamp: 'monty 11/14/2013 00:13'!
startElement: aQualifiedName attributes: aDictionary
! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 3/24/2010 16:47'!
startElement: aQualifiedName prefix: aPrefix uri: aUri attributes: aDictionary
	self startElement: aQualifiedName attributes: aDictionary! !

!SAXHandler methodsFor: 'content' stamp: 'monty 11/14/2013 00:14'!
startElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName attributes: aDictionary
	"This call corresonds to the Java SAX call
	startElement(java.lang.String namespaceURI, java.lang.String localName,
		java.lang.String qName, Attributes atts)."

	self
		startElement: aQualifiedName
		prefix: aPrefix
		uri: aUri
		attributes: aDictionary! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 11/10/2010 19:22'!
startEntity: anEntityName
	"This call corresponds to the Java SAX ext call
	startEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/10/2010 19:22'!
startPrefixMapping: aPrefix uri: aUri
	"This call corresonds to the Java SAX call
	startPrefixMapping(java.lang.String prefix, java.lang.String uri)."! !

!SAXHandler methodsFor: 'configuration' stamp: 'JAAyer 11/23/2010 14:20'!
usesNamespaces
	^ self configuration usesNamespaces! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/29/2013 03:03'!
usesNamespaces: aBoolean
	"Namespace checking when true (default)"

	self configuration usesNamespaces: aBoolean! !

!SAXHandler methodsFor: 'content' stamp: 'monty 11/14/2013 00:14'!
xmlVersion: aVersionFloat encoding: anEncoding standalone: aBoolean
! !

SAXHandler subclass: #XMLDOMParser
	instanceVariableNames: 'document nodeStack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLDOMParser commentStamp: 'monty 12/31/2013 00:36' prior: 0!
This class is an XML parser that produces a tree of nodes representing the parsed document's structure. The root of this tree will typically be a document node, and it will have a single child element node as its root. The root element can have any number of additional elements as its children, as well as non-element child nodes. Those elements can themselves have an arbitrary number of children, and so on. To see what type of node objects a node tree created by this parser will contain and for how to manipulate them browse the XML-Parser-DOM category.

By default, CDATA sections are treated as regular string nodes and comments are ignored entirely. To suppress this behavior, enable #preservesCDataNodes: or #preservesCommentNodes: before parsing. 

Lastly. you can control what node classes the DOM parser uses to create node objects by injecting a custom node factory with #nodeFactory: prior to parsing. See XMLNodeFactory for more information.!

!XMLDOMParser class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 04:09'!
on: aStringOrStream withRootNode: aRootNode
	^ (self on: aStringOrStream)
		pushNode: aRootNode;
		yourself! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/31/2013 00:02'!
attributeDeclaration: anElement name: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	self topNode addNode:
		(self nodeFactory attributeDeclarationClass
			element: anElement
			attribute: aName
			type: aType
			defaultPragma: aDefaultPragma
			defaultValue: aDefaultValue)! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 11/23/2010 14:19'!
characters: aString
	(self preservesCDataNodes and: [self isInCDataSection])
		ifTrue: [self topNode addNode: (self nodeFactory cdataClass string: aString)]
		ifFalse: [self topNode addContent: (self nodeFactory stringClass string: aString)]
! !

!XMLDOMParser methodsFor: 'lexical' stamp: 'JAAyer 11/23/2010 14:43'!
comment: aComment
	self preservesCommentNodes
		ifTrue: [self topNode addNode: (self nodeFactory commentClass string: aComment)]! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 11/23/2010 14:37'!
configurationClass
	^ XMLDOMConfiguration! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 12/27/2013 03:43'!
currentScope: aNamespaceScope
	self topNode setNamespaceScope: aNamespaceScope! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 3/9/2010 01:38'!
document
	^ document! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 12:31'!
document: aDocument
	(document := aDocument) configuration: self configuration! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/21/2013 03:14'!
elementDeclaration: aName contentModel: aContentModel
	self topNode addNode:
		(self nodeFactory elementDeclarationClass
			element: aName
			contentModel: aContentModel)! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 11/12/2013 23:00'!
endDTD
	self popNode! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 11/27/2010 06:53'!
endDocument
	self popNode! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAA 3/21/2011 19:45'!
endElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName
	self popNode! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/21/2013 03:01'!
generalEntityDeclaration: aName publicID: aPublicId systemID: aSystemId ndata: aNotation
	self topNode addNode:
		(self nodeFactory generalEntityDeclarationClass
			name: aName
			publicID: aPublicId
			systemID: aSystemId
			ndata: aNotation)! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/21/2013 03:02'!
generalEntityDeclaration: aName replacement: aReplacement
	self topNode addNode:
		(self nodeFactory generalEntityDeclarationClass
			name: aName
			replacement: aReplacement)! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
nodeFactory
	^ self configuration nodeFactory! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
nodeFactory: aNodeFactory
	self configuration nodeFactory: aNodeFactory! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'monty 12/9/2013 01:37'!
nodeListClass
	^ self configuration nodeListClass! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'monty 12/9/2013 01:37'!
nodeListClass: aClass
	self configuration nodeListClass: aClass! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 2/1/2014 01:32'!
nodeStack
	^ nodeStack ifNil: [nodeStack := OrderedCollection new: 10]! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/21/2013 03:03'!
notationDeclaration: aName publicID: aPublicId systemID: aSystemId
	self topNode addNode:
		(self nodeFactory notationDeclarationClass
			name: aName
			publicID: aPublicId
			systemID: aSystemId)! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/21/2013 03:02'!
parameterEntityDeclaration: aName publicID: aPublicId systemID: aSystemId
	self topNode addNode:
		(self nodeFactory parameterEntityDeclarationClass
			name: aName
			publicID: aPublicId
			systemID: aSystemId)! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/21/2013 03:02'!
parameterEntityDeclaration: aName replacement: aReplacement
	self topNode addNode:
		(self nodeFactory parameterEntityDeclarationClass
			name: aName
			replacement: aReplacement)! !

!XMLDOMParser methodsFor: 'parsing' stamp: 'monty 10/13/2014 19:45'!
parser
	| parser |

	parser := super parser.
	(self nodeStack notEmpty and: [self topNode isElement])
		ifTrue: [
			self
				configuration: self topNode configuration;
				document: self topNode document.
			parser driver
				rootElement: self topNode name;
				currentScope: self topNode namespaceScope].

	^ parser.! !

!XMLDOMParser methodsFor: 'parsing' stamp: 'monty 12/19/2013 00:38'!
parsingResult
	^ self document! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 3/16/2010 09:31'!
popNode
	^ self nodeStack removeLast! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
preservesCDataNodes
	^ self configuration preservesCDataNodes! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:17'!
preservesCDataNodes: aBoolean
	self configuration preservesCDataNodes: aBoolean! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
preservesCommentNodes
	^ self configuration preservesCommentNodes! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
preservesCommentNodes: aBoolean
	self configuration preservesCommentNodes: aBoolean! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAA 3/22/2011 11:31'!
processingInstruction: aTarget data: aDataString
	self topNode addNode: (self nodeFactory piClass target: aTarget data: aDataString)! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 2/4/2014 01:33'!
pushElement: anElement
	self topNode addNode: anElement.
	^ self pushNode: anElement.! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 11/28/2010 13:25'!
pushNode: aNode
	^ self nodeStack addLast: aNode
! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 11/12/2013 23:02'!
startDTD: aRootElementName publicID: aPublicId systemID: aSystemId
	| doctypeDeclaration |

	doctypeDeclaration := 
		self nodeFactory doctypeDeclarationClass
			root: aRootElementName
			publicID: aPublicId
			systemID: aSystemId.
	self pushNode: doctypeDeclaration.
	self document doctypeDeclaration: doctypeDeclaration.! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 11/28/2010 13:25'!
startDocument
	self topNode ifNotNil: [^ self].

	self document: self nodeFactory documentClass new.
	self pushNode: self document ! !

!XMLDOMParser methodsFor: 'content' stamp: 'monty 2/14/2014 05:55'!
startElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName attributes: aDictionary
	self pushElement:
		((self nodeFactory
			classForElement: aQualifiedName
			prefix: aPrefix
			uri: aUri
			localName: aLocalName) new
				setQualifiedName: aQualifiedName
				prefix: aPrefix
				localName: aLocalName
				attributes: aDictionary)! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 10/7/2014 03:06'!
topNode
	^ self nodeStack lastOrNil! !

!XMLDOMParser methodsFor: 'content' stamp: 'monty 11/19/2013 05:37'!
xmlVersion: aVersionFloat encoding: anEncoding standalone: aBoolean
	self document ifNotNil: [
		self document
			version: aVersionFloat;
			encoding: anEncoding;
			isStandalone: aBoolean]! !

Object subclass: #XMLAttributeDefaultValidator
	instanceVariableNames: 'defaultValue attribute'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLAttributeDefaultValidator commentStamp: '<historical>' prior: 0!
This is a base class for attribute default value validators that implement the constraints (like #REQUIRED) from the default declaration.!

!XMLAttributeDefaultValidator class methodsFor: 'instance creation' stamp: 'monty 1/21/2014 05:35'!
attribute: anAttribute
	^ self attribute: anAttribute defaultValue: ''! !

!XMLAttributeDefaultValidator class methodsFor: 'instance creation' stamp: 'monty 12/10/2013 06:16'!
attribute: anAttribute defaultValue: aDefaultValue
	^ self new setAttribute: anAttribute defaultValue: aDefaultValue! !

!XMLAttributeDefaultValidator methodsFor: 'initialization' stamp: 'monty 12/10/2013 06:17'!
setAttribute: anAttribute defaultValue: aDefaultValue
	attribute := anAttribute.
	defaultValue := aDefaultValue.! !

!XMLAttributeDefaultValidator methodsFor: 'validation' stamp: 'monty 12/28/2013 20:02'!
validateAttributeDefaultIn: anAttributeDictionary
	self subclassResponsibility! !

XMLAttributeDefaultValidator subclass: #XMLDefaultValueAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLDefaultValueAttributeValidator commentStamp: 'monty 2/2/2014 04:47' prior: 0!
This class will put a default value for an attribute if it has none.!

!XMLDefaultValueAttributeValidator methodsFor: 'validation' stamp: 'monty 12/11/2013 02:23'!
validateAttributeDefaultIn: anAttributeDictionary
	((anAttributeDictionary at: attribute ifAbsent: ['']) isEmpty 
		and: [defaultValue notEmpty])
		ifTrue: [anAttributeDictionary at: attribute put: defaultValue]! !

XMLDefaultValueAttributeValidator subclass: #XMLFixedValueAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLFixedValueAttributeValidator commentStamp: 'monty 2/2/2014 08:40' prior: 0!
This class fixes the value of an attribute to a default.!

!XMLFixedValueAttributeValidator methodsFor: 'validation' stamp: 'monty 12/11/2013 02:23'!
validateAttributeDefaultIn: anAttributeDictionary
	anAttributeDictionary at: attribute put: defaultValue! !

XMLAttributeDefaultValidator subclass: #XMLImpliedValueAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLImpliedValueAttributeValidator commentStamp: 'monty 2/2/2014 06:07' prior: 0!
Attributes declared #IMPLIED are optional.
!

!XMLImpliedValueAttributeValidator methodsFor: 'validation' stamp: 'monty 12/27/2013 15:36'!
validateAttributeDefaultIn: anAttributeDictionary! !

XMLAttributeDefaultValidator subclass: #XMLRequiredValueAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLRequiredValueAttributeValidator commentStamp: 'monty 2/2/2014 16:37' prior: 0!
Attributes declared #REQUIRED require values.!

!XMLRequiredValueAttributeValidator methodsFor: 'validation' stamp: 'monty 12/11/2013 02:23'!
validateAttributeDefaultIn: anAttributeDictionary
	(anAttributeDictionary includesKey: attribute)
		ifFalse: [XMLValidationException signal: 'Required attribute "', attribute, '" is missing']! !

Object subclass: #XMLAttributeValidator
	instanceVariableNames: 'element attribute defaultValueValidator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLAttributeValidator commentStamp: '<historical>' prior: 0!
This is a base class for attribute validators.!

!XMLAttributeValidator class methodsFor: 'instance creation' stamp: 'monty 12/10/2013 06:15'!
element: anElement attribute: anAttribute defaultValueValidator: aDefaultValueValidator
	^ self new
		setElement: anElement
		attribute: anAttribute
		defaultValueValidator: aDefaultValueValidator! !

!XMLAttributeValidator methodsFor: 'accessing' stamp: 'monty 12/10/2013 06:17'!
attribute
	^ attribute! !

!XMLAttributeValidator methodsFor: 'accessing' stamp: 'monty 12/10/2013 06:17'!
element
	^ element! !

!XMLAttributeValidator methodsFor: 'normalization' stamp: 'monty 2/10/2014 19:58'!
furtherNormalizeAttributeValueIn: anAttributeDictionary
	| readStream writeStream |

	readStream := (anAttributeDictionary at: attribute ifAbsent: [^ self]) readStream.
	writeStream := (String new: readStream size) writeStream.
	readStream skipSeparators.
	[readStream atEnd]
		whileFalse: [
			readStream peek isSeparator
				ifTrue: [
					readStream skipSeparators.
					readStream atEnd
						ifFalse: [writeStream space]]
				ifFalse: [writeStream nextPut: readStream next]].
	anAttributeDictionary
		at: attribute
		put: writeStream contents.! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/29/2013 02:57'!
hasIDRefs
	^ false! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/29/2013 02:57'!
hasIDs
	^ false! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 1/4/2014 05:02'!
hasNotationValues
	^ false! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/10/2013 06:06'!
isIDValidator
	^ false! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/9/2013 02:24'!
isValueListValidator
	^ false! !

!XMLAttributeValidator methodsFor: 'private' stamp: 'monty 2/10/2014 19:50'!
listValuesIn: aSpaceSeparatedList do: aBlock
	Character space
		split: aSpaceSeparatedList
		do: aBlock! !

!XMLAttributeValidator methodsFor: 'initialization' stamp: 'monty 12/10/2013 06:15'!
setElement: anElement attribute: anAttribute defaultValueValidator: aDefaultValueValidator
	element := anElement.
	attribute := anAttribute.
	defaultValueValidator := aDefaultValueValidator.! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/11/2013 02:14'!
shouldFurtherNormalizeAttributeValue
	^ true! !

!XMLAttributeValidator methodsFor: 'validation' stamp: 'monty 1/4/2014 22:49'!
validateAndAddTo: anAttributeValidatorDictionary usingDeclarationsFrom: aDeclarationDelegate
	anAttributeValidatorDictionary
		at: attribute
		put: self! !

!XMLAttributeValidator methodsFor: 'validation' stamp: 'monty 2/10/2014 19:58'!
validateAttributeDefaultIn: anAttributeDictionary
	defaultValueValidator validateAttributeDefaultIn: anAttributeDictionary.

	self shouldFurtherNormalizeAttributeValue
		ifTrue: [self furtherNormalizeAttributeValueIn: anAttributeDictionary].! !

!XMLAttributeValidator methodsFor: 'validation' stamp: 'monty 12/28/2013 20:03'!
validateAttributeValue: aValue
	self subclassResponsibility! !

!XMLAttributeValidator methodsFor: 'validation' stamp: 'monty 1/21/2014 05:29'!
validateAttributes: anAttributeDictionary
	self
		validateAttributeDefaultIn: anAttributeDictionary;
		validateAttributeValue: (anAttributeDictionary at: attribute ifAbsent: [^ self])! !

!XMLAttributeValidator methodsFor: 'validation' stamp: 'monty 2/10/2014 02:43'!
validateID: anId
	(XMLWellFormedParserTokenizer isName: anId)
		ifFalse: [XMLValidationException signal: 'Invalid ID ', anId, ' in attribute "', attribute, '"']! !

!XMLAttributeValidator methodsFor: 'validation' stamp: 'monty 2/10/2014 02:43'!
validateName: aName
	(XMLWellFormedParserTokenizer isNmtoken: aName)
		ifFalse: [XMLValidationException signal: 'Invalid name "', aName, '" in attribute "', attribute, '"']! !

XMLAttributeValidator subclass: #XMLCDataAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLCDataAttributeValidator commentStamp: 'monty 2/2/2014 08:12' prior: 0!
CDATA attribute values are left alone (whitespace is not normalized further).!

!XMLCDataAttributeValidator methodsFor: 'testing' stamp: 'monty 12/11/2013 02:14'!
shouldFurtherNormalizeAttributeValue
	^ false! !

!XMLCDataAttributeValidator methodsFor: 'validation' stamp: 'monty 12/28/2013 20:04'!
validateAttributeValue: aValue
	! !

XMLAttributeValidator subclass: #XMLEntityAttributeValidator
	instanceVariableNames: 'unparsedEntities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLEntityAttributeValidator commentStamp: 'monty 2/2/2014 16:44' prior: 0!
This class validates attributes that contain unparsed entity names (that must be declared).!

XMLEntityAttributeValidator subclass: #XMLEntitiesAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLEntitiesAttributeValidator commentStamp: 'monty 2/2/2014 16:44' prior: 0!
This class validates attributes with lists of unparsed entity name values separated by spaces.!

!XMLEntitiesAttributeValidator methodsFor: 'validation' stamp: 'monty 2/10/2014 19:50'!
validateAttributeValue: aValue
	self
		listValuesIn: aValue
		do: [:each | self validateEntityReference: each]! !

!XMLEntityAttributeValidator methodsFor: 'testing' stamp: 'monty 11/19/2013 22:00'!
isEntityValidator
	^ true! !

!XMLEntityAttributeValidator methodsFor: 'accessing' stamp: 'monty 11/19/2013 18:54'!
unparsedEntities
	^ unparsedEntities ifNil: [unparsedEntities := Dictionary new]! !

!XMLEntityAttributeValidator methodsFor: 'validation' stamp: 'monty 1/4/2014 22:50'!
validateAndAddTo: anAttributeValidatorDictionary usingDeclarationsFrom: aDeclarationDelegate
	unparsedEntities := aDeclarationDelegate unparsedEntities.
	super
		validateAndAddTo: anAttributeValidatorDictionary
		usingDeclarationsFrom: aDeclarationDelegate.! !

!XMLEntityAttributeValidator methodsFor: 'validation' stamp: 'monty 12/11/2013 02:12'!
validateAttributeValue: aValue
	self validateEntityReference: aValue! !

!XMLEntityAttributeValidator methodsFor: 'validation' stamp: 'monty 12/10/2013 06:14'!
validateEntityReference: aName
	self validateName: aName.
	(self unparsedEntities includesKey: aName)
		ifFalse: [XMLValidationException signal: 'Unrecognized unpasred entity ', aName, ' in attribute "', attribute, '"']! !

XMLAttributeValidator subclass: #XMLIDAttributeValidator
	instanceVariableNames: 'ids'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLIDAttributeValidator commentStamp: 'monty 2/2/2014 05:57' prior: 0!
This class validates ID attributes that must be unique in each element of the type.!

!XMLIDAttributeValidator methodsFor: 'testing' stamp: 'monty 12/29/2013 02:57'!
hasIDs
	^ self ids notEmpty! !

!XMLIDAttributeValidator methodsFor: 'accessing' stamp: 'monty 11/19/2013 08:18'!
ids
	^ ids ifNil: [ids := Set new]
	! !

!XMLIDAttributeValidator methodsFor: 'testing' stamp: 'monty 12/10/2013 06:07'!
isIDValidator
	^ true! !

!XMLIDAttributeValidator methodsFor: 'validation' stamp: 'monty 1/4/2014 22:50'!
validateAndAddTo: anAttributeValidatorDictionary usingDeclarationsFrom: aDeclarationDelegate
	anAttributeValidatorDictionary valuesDo: [:each |
		each isIDValidator
			ifTrue: [
				XMLValidationException signal:
					'Element <', element, '> has multiple ID attributes']].
	
	super
		validateAndAddTo: anAttributeValidatorDictionary
		usingDeclarationsFrom: aDeclarationDelegate! !

!XMLIDAttributeValidator methodsFor: 'validation' stamp: 'monty 1/21/2014 04:05'!
validateAttributeValue: aValue
	self validateID: aValue.
	(self ids includes: aValue)
		ifTrue: [
			XMLValidationException signal:
				'ID "', aValue, '" used more than once for attribute "', attribute, '"'].

	self ids add: aValue.! !

XMLAttributeValidator subclass: #XMLIDRefAttributeValidator
	instanceVariableNames: 'idrefs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLIDRefAttributeValidator commentStamp: 'monty 2/2/2014 16:44' prior: 0!
This class validates attributes that refer to an ID of another element with a ID attribute.!

!XMLIDRefAttributeValidator methodsFor: 'testing' stamp: 'monty 12/29/2013 02:58'!
hasIDRefs
	^ self idRefs notEmpty! !

!XMLIDRefAttributeValidator methodsFor: 'accessing' stamp: 'monty 12/29/2013 02:58'!
idRefs
	^ idrefs ifNil: [idrefs := Set new].! !

!XMLIDRefAttributeValidator methodsFor: 'validation' stamp: 'monty 12/29/2013 02:58'!
validateAttributeValue: aValue
	self validateID: aValue.
	self idRefs add: aValue.! !

XMLIDRefAttributeValidator subclass: #XMLIDRefsAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLIDRefsAttributeValidator commentStamp: 'monty 2/2/2014 08:13' prior: 0!
This class validates attributes with lists of ID ref values separated by spaces.!

!XMLIDRefsAttributeValidator methodsFor: 'validation' stamp: 'monty 2/10/2014 19:50'!
validateAttributeValue: aValue
	self
		listValuesIn: aValue
		do: [:each |
			self validateID: each.
			self idRefs add: each].! !

XMLAttributeValidator subclass: #XMLNameAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLNameAttributeValidator commentStamp: 'monty 2/2/2014 06:06' prior: 0!
This class validates name attributes.!

!XMLNameAttributeValidator methodsFor: 'validation' stamp: 'monty 12/11/2013 02:12'!
validateAttributeValue: aValue
	self validateName: aValue! !

XMLNameAttributeValidator subclass: #XMLNamesAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLNamesAttributeValidator commentStamp: 'monty 2/2/2014 06:07' prior: 0!
This class validates attributes with lists of name values separated by spaces.!

!XMLNamesAttributeValidator methodsFor: 'validation' stamp: 'monty 2/10/2014 19:50'!
validateAttributeValue: aValue
	self
		listValuesIn: aValue
		do: [:each | self validateName: each]! !

XMLAttributeValidator subclass: #XMLValueListAttributeValidator
	instanceVariableNames: 'allowedValues'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLValueListAttributeValidator commentStamp: 'monty 2/2/2014 08:13' prior: 0!
This class validates attributes that can have any value taken from a list like "(one|two|thee)".!

XMLValueListAttributeValidator subclass: #XMLNotationListAttributeValidator
	instanceVariableNames: 'notations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLNotationListAttributeValidator commentStamp: 'monty 2/2/2014 08:10' prior: 0!
This class validates attributes that can have NOTATION names from a list as values.!

!XMLNotationListAttributeValidator methodsFor: 'testing' stamp: 'monty 1/4/2014 04:58'!
hasNotationValues
	^ true! !

!XMLNotationListAttributeValidator methodsFor: 'accessing' stamp: 'monty 1/4/2014 05:00'!
notations
	^ notations ifNil: [notations := Set new]! !

!XMLNotationListAttributeValidator methodsFor: 'validation' stamp: 'monty 1/4/2014 22:50'!
validateAndAddTo: anAttributeValidatorDictionary usingDeclarationsFrom: aDeclarationDelegate
	anAttributeValidatorDictionary valuesDo: [:each |
		each hasNotationValues
			ifTrue: [
				XMLValidationException signal:
					'Element <', element, '> has multiple NOTATION list attributes']].

	notations := aDeclarationDelegate notations.
	super
		validateAndAddTo: anAttributeValidatorDictionary
		usingDeclarationsFrom: aDeclarationDelegate.! !

!XMLNotationListAttributeValidator methodsFor: 'validation' stamp: 'monty 1/4/2014 08:16'!
validateAttributeValue: aValue
	super validateAttributeValue: aValue.

	(self notations includes: aValue)
		ifFalse: [
			XMLValidationException signal:
				'Undeclared NOTATION "', aValue, '" in attribute "', attribute, '"'].! !

!XMLNotationListAttributeValidator methodsFor: 'private ' stamp: 'monty 1/25/2014 01:17'!
valueListParserClass
	^ DTDNotationListParser! !

!XMLValueListAttributeValidator methodsFor: 'accessing' stamp: 'monty 12/9/2013 02:17'!
allowedValues
	^ allowedValues ifNil: [allowedValues := Set new]! !

!XMLValueListAttributeValidator methodsFor: 'testing' stamp: 'monty 12/9/2013 02:24'!
isValueListValidator
	^ true! !

!XMLValueListAttributeValidator methodsFor: 'accessing' stamp: 'monty 1/25/2014 01:24'!
setAllowedValuesFromString: aString
	allowedValues := (self valueListParserClass on: aString) parse! !

!XMLValueListAttributeValidator methodsFor: 'validation' stamp: 'monty 12/11/2013 02:12'!
validateAttributeValue: aValue
	(self allowedValues includes: aValue)
		ifFalse: [XMLValidationException signal: 'Invalid value "', aValue, '" for attribute "', attribute, '"']! !

!XMLValueListAttributeValidator methodsFor: 'private ' stamp: 'monty 1/25/2014 01:17'!
valueListParserClass
	^ DTDEnumerationListParser! !

Object subclass: #XMLConfiguration
	instanceVariableNames: 'isValidating usesNamespaces resolvesExternalEntities preservesUndeclaredEntityReferences externalEntityResolver parserHandlerClass parserLimits requiresDocumentConstraints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLConfiguration commentStamp: '<historical>' prior: 0!
This class stores the shared settings and components for a parser.!

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/10/2014 08:08'!
externalEntityResolver
	^ externalEntityResolver ifNil: [externalEntityResolver := DTDCachingEntityResolver new]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 11/18/2013 22:18'!
externalEntityResolver: anExternalEntityResolver
	externalEntityResolver := anExternalEntityResolver! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:09'!
isValidating
	^ isValidating ifNil: [isValidating := true]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 12/29/2013 02:42'!
isValidating: aBoolean
	(isValidating := aBoolean)
		ifTrue: [self preservesUndeclaredEntityReferences: false]
		ifFalse: [self requiresDocumentConstraints: false]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 11/16/2013 03:10'!
parserHandlerClass
	^ parserHandlerClass ifNil: [parserHandlerClass := SAXHandler]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 11/16/2013 03:10'!
parserHandlerClass: aClass
	parserHandlerClass := aClass! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/16/2014 21:27'!
parserLimits
	^ parserLimits ifNil: [parserLimits := XMLParserLimits new]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:16'!
preservesUndeclaredEntityReferences
	^ preservesUndeclaredEntityReferences ifNil: [preservesUndeclaredEntityReferences := false]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 12/29/2013 02:42'!
preservesUndeclaredEntityReferences: aBoolean
	(preservesUndeclaredEntityReferences := aBoolean)
		ifTrue: [self isValidating: false]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/25/2014 21:13'!
requiresDocumentConstraints
	^ requiresDocumentConstraints ifNil: [requiresDocumentConstraints := false]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/25/2014 21:13'!
requiresDocumentConstraints: aBoolean
	(requiresDocumentConstraints := aBoolean)
		ifTrue: [self isValidating: true]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/24/2014 23:52'!
resolvesExternalEntities
	^ resolvesExternalEntities ifNil: [resolvesExternalEntities := false]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 11/13/2013 22:18'!
resolvesExternalEntities: aBoolean
	resolvesExternalEntities := aBoolean! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:07'!
usesNamespaces
	^ usesNamespaces ifNil: [usesNamespaces := true]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:06'!
usesNamespaces: aBoolean
	usesNamespaces := aBoolean! !

XMLConfiguration subclass: #XMLDOMConfiguration
	instanceVariableNames: 'preservesCDataNodes preservesCommentNodes nodeFactory nodeListClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLDOMConfiguration commentStamp: '<historical>' prior: 0!
This class stores additional settings and components relevant to the DOM parser and DOM nodes.!

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:10'!
nodeFactory
	^ nodeFactory ifNil: [nodeFactory := XMLNodeFactory new]! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:10'!
nodeFactory: aFactory
	nodeFactory := aFactory! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:22'!
nodeListClass
	^ nodeListClass ifNil: [nodeListClass := XMLNodeList]! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:22'!
nodeListClass: aClass
	nodeListClass := aClass! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCDataNodes
	^ preservesCDataNodes ifNil: [preservesCDataNodes := false]! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCDataNodes: aBoolean
	preservesCDataNodes := aBoolean! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCommentNodes
	^ preservesCommentNodes ifNil: [preservesCommentNodes := false]! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCommentNodes: aBoolean
	preservesCommentNodes := aBoolean! !

Object subclass: #XMLDFAState
	instanceVariableNames: 'nfaStates transitions dfaStateCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLDFAState commentStamp: 'monty 12/28/2013 22:34' prior: 0!
This class is a DFA state made by grouping one or more NFA states.!

!XMLDFAState class methodsFor: 'instance creation' stamp: 'monty 12/29/2013 01:11'!
nfaStates: anNfaStateSet
	^ self
		nfaStates: anNfaStateSet
		dfaStateCache: nil! !

!XMLDFAState class methodsFor: 'instance creation' stamp: 'monty 12/29/2013 01:09'!
nfaStates: anNfaStateSet dfaStateCache: aDfaStateCache
	^ self new
		setNFAStates: anNfaStateSet
		dfaStateCache: aDfaStateCache! !

!XMLDFAState methodsFor: 'enumerating' stamp: 'monty 12/28/2013 00:21'!
detectNextStateMatching: aMatcher
	^ self detectNextStateMatching: aMatcher ifNone: [nil]! !

!XMLDFAState methodsFor: 'enumerating' stamp: 'monty 12/28/2013 19:54'!
detectNextStateMatching: aMatcher ifNone: aNoneBlock
	^ (self transitions
		at: aMatcher
		ifAbsentPut: [self dfaStateMatching: aMatcher])
			ifNil: [aNoneBlock value]! !

!XMLDFAState methodsFor: 'accessing' stamp: 'monty 12/29/2013 03:33'!
dfaStateMatching: aMatcher
	| matchingNfaStates |

	matchingNfaStates := XMLNFAStateSet new.
	nfaStates do: [:state |
		(state matches: aMatcher)
			ifTrue: [state nextState addTo: matchingNfaStates]].

	^ matchingNfaStates
		ifNotEmpty: [dfaStateCache dfaStateFor: matchingNfaStates]
		ifEmpty: [nil].! !

!XMLDFAState methodsFor: 'testing' stamp: 'monty 12/28/2013 18:54'!
isAccepting
	^ nfaStates isAccepting! !

!XMLDFAState methodsFor: 'accessing' stamp: 'monty 12/29/2013 01:10'!
nfaStates
	^ nfaStates! !

!XMLDFAState methodsFor: 'initialization' stamp: 'monty 12/29/2013 03:25'!
setNFAStates: anNfaStateSet dfaStateCache: aDfaStateCache
	nfaStates := anNfaStateSet.
	dfaStateCache := (aDfaStateCache ifNil: [XMLDFAStateCache firstState: self]).! !

!XMLDFAState methodsFor: 'accessing' stamp: 'monty 12/28/2013 01:09'!
transitions
	^ transitions ifNil: [transitions := Dictionary new]! !

Object subclass: #XMLDFAStateCache
	instanceVariableNames: 'firstState cachedStates'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLDFAStateCache commentStamp: 'monty 12/29/2013 03:32' prior: 0!
This class uses a dictionary with NFAStateSet keys to cache DFA states.!

!XMLDFAStateCache class methodsFor: 'instance creation' stamp: 'monty 12/29/2013 01:02'!
firstState: aFirstState
	^ self new firstState: aFirstState! !

!XMLDFAStateCache methodsFor: 'private' stamp: 'monty 12/29/2013 03:29'!
clearCache
	cachedStates
		removeAll;
		at: firstState nfaStates
		put: firstState! !

!XMLDFAStateCache methodsFor: 'accessing' stamp: 'monty 12/29/2013 06:33'!
dfaStateFor: aStateSet
	^ cachedStates
		at: aStateSet
		ifAbsent: [
			cachedStates size >= self maxCachedStates
				ifTrue: [self clearCache].

			cachedStates
				at: aStateSet
				put: (self newDFAStateFor: aStateSet)].! !

!XMLDFAStateCache methodsFor: 'accessing' stamp: 'monty 12/29/2013 03:24'!
firstState: aFirstState
	firstState := aFirstState.

	(cachedStates := Dictionary new)
		at: firstState nfaStates
		put: firstState.! !

!XMLDFAStateCache methodsFor: 'private' stamp: 'monty 12/29/2013 01:07'!
maxCachedStates
	^ 32! !

!XMLDFAStateCache methodsFor: 'private' stamp: 'monty 12/29/2013 03:29'!
newDFAStateFor: aStateSet
	^ XMLDFAState
		nfaStates: aStateSet
		dfaStateCache: self.! !

Object subclass: #XMLDocumentValidator
	instanceVariableNames: 'elementValidatorStack xmlIdValidator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLDocumentValidator commentStamp: '<historical>' prior: 0!
This is a base class for XML document validators.!

XMLDocumentValidator subclass: #DTDDocumentValidator
	instanceVariableNames: 'dtd'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDDocumentValidator commentStamp: '<historical>' prior: 0!
This class validates documents using a DTD.!

!DTDDocumentValidator class methodsFor: 'instance creation' stamp: 'monty 11/18/2013 05:13'!
dtd: aDtd
	^ self new dtd: aDtd! !

!DTDDocumentValidator methodsFor: 'enumerating' stamp: 'monty 12/9/2013 02:29'!
allAttributeValidatorsDo: aBlock
	^ self dtd attributeValidators valuesDo: [:each | each valuesDo: aBlock]! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 12/29/2013 05:04'!
attributeValidatorsFor: anElement
	^ dtd attributeValidators
		at: anElement
		ifAbsent: [nil]! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/18/2013 05:13'!
dtd
	^ dtd! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/18/2013 05:13'!
dtd: aDtd
	dtd := aDtd! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/20/2013 01:20'!
elementValidatorFor: aName
	^ (dtd elementValidators
		at: aName
		ifAbsent: [nil]) copy! !

!DTDDocumentValidator methodsFor: 'testing' stamp: 'monty 11/19/2013 01:03'!
hasAttributeValidators
	^ dtd hasAttributeValidators! !

!DTDDocumentValidator methodsFor: 'testing' stamp: 'monty 11/19/2013 01:03'!
hasElementValidators
	^ dtd hasElementValidators! !

!DTDDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 21:07'!
unvalidatedAttributes: anAttributeCollection
	XMLValidationException signal:
		'Undeclared attribute(s) ', (', ' join: anAttributeCollection)! !

!DTDDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 20:29'!
unvalidatedElement: aName
	XMLValidationException signal: 'Undeclared element <', aName, '>'! !

!DTDDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 02:38'!
validateEndDocument
	self hasAttributeValidators
		ifTrue: [self validateIDsAndReferences]	! !

!DTDDocumentValidator methodsFor: 'validation' stamp: 'monty 10/10/2014 23:09'!
validateIDsAndReferences
	| ids references |

	ids := Set new.
	references := Set new.
	self allAttributeValidatorsDo: [:each |
		each hasIDs
			ifTrue: [ids addAll: each ids]
			ifFalse: [
				each hasIDRefs
					ifTrue: [references addAll: each idRefs]]].

	references do: [:each |
		(ids includes: each)
			ifFalse: [XMLValidationException signal: 'Invalid ID reference "', each, '"']].! !

!DTDDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 20:15'!
validateRootElement: anElementName
	(dtd rootElement notEmpty
		and: [dtd rootElement ~= anElementName])
		ifTrue: [
			XMLValidationException signal:
				'Element <', anElementName, '> cannot be root']! !

!XMLDocumentValidator methodsFor: 'enumerating' stamp: 'monty 11/19/2013 18:26'!
allAttributeValidatorsDo: aBlock
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/19/2013 01:04'!
attributeValidatorsFor: anElement
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/18/2013 03:47'!
currentElementValidator
	^ self elementValidatorStack last! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 20:07'!
defaultElementValidatorFor: aName
	^ XMLAnyElementValidator element: aName! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/18/2013 03:57'!
elementValidatorFor: aName
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/18/2013 03:46'!
elementValidatorStack
	^ elementValidatorStack ifNil: [elementValidatorStack := OrderedCollection new]! !

!XMLDocumentValidator methodsFor: 'private' stamp: 'monty 11/28/2013 19:42'!
errorMissingContentFor: aName
	XMLValidationException signal: 'Element <', aName, '> terminated prematurely'! !

!XMLDocumentValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 03:31'!
hasAttributeValidators
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 03:32'!
hasElementValidators
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 20:29'!
unvalidatedAttributes: anAttributeCollection! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 20:29'!
unvalidatedElement: aName! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 1/17/2014 02:46'!
validateAttributes: anAttributeDictionary with: aValidatorDictionary
	| unvalidatedAttributes |

	unvalidatedAttributes := anAttributeDictionary keys asSet.
	aValidatorDictionary
		ifNotNil: [
			aValidatorDictionary keysAndValuesDo: [:attribute :validator |
				validator validateAttributes: anAttributeDictionary.
				unvalidatedAttributes
					remove: attribute
					ifAbsent: []]].

	unvalidatedAttributes
		ifNotEmpty: [self unvalidatedAttributes: unvalidatedAttributes].! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 02:22'!
validateEndDocument
! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/7/2013 19:45'!
validateEndTag: aName
	self hasElementValidators
		ifTrue: [
			(self currentElementValidator element = aName)
				ifTrue: [
					self currentElementValidator atEnd
						ifFalse: [self errorMissingContentFor: aName]].
			self elementValidatorStack removeLast]! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 11/18/2013 03:44'!
validatePCData: aString
	self hasElementValidators
		ifTrue: [self currentElementValidator validatePCData: aString]! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 20:08'!
validateRootElement: anElementName! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 23:56'!
validateStartTag: aName
	self elementValidatorStack
		ifNotEmpty: [self currentElementValidator validateStartTag: aName].

	self elementValidatorStack addLast:
		((self elementValidatorFor: aName)
			ifNil: [
				self
					unvalidatedElement: aName;
					defaultElementValidatorFor: aName]).! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/29/2013 05:05'!
validateStartTag: aName attributes: anAttributeDictionary
	(self hasElementValidators or: [self hasAttributeValidators])
		ifFalse: [^ self].

	self validateStartTag: aName.
	self
		validateAttributes: anAttributeDictionary
		with: (self attributeValidatorsFor: aName).! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 1/21/2014 05:30'!
validateXMLIDAttributeIn: aDictionary
	self xmlIDValidator validateAttributes: aDictionary! !

!XMLDocumentValidator methodsFor: 'private' stamp: 'monty 1/21/2014 05:32'!
xmlIDValidator
	^ xmlIdValidator ifNil: [
		xmlIdValidator :=
			XMLIDAttributeValidator
				element: ''
				attribute: 'xml:id'
				defaultValueValidator:
					(XMLImpliedValueAttributeValidator attribute: 'xml:id')]! !

Object subclass: #XMLElementNester
	instanceVariableNames: 'openElements scopedAttributes rootElement hasExternalRoot'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLElementNester commentStamp: '<historical>' prior: 0!
This is a base class for element nesters.!

!XMLElementNester methodsFor: 'private' stamp: 'monty 1/25/2014 21:08'!
copiedScopedAttributes
	| copiedAttributes |

	copiedAttributes :=
		self hasScopedAttributes
			ifTrue: [self scopedAttributes removeLast copy]
			ifFalse: [Dictionary new].
	self scopedAttributes addLast: copiedAttributes.

	^ copiedAttributes.! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 10/7/2014 03:16'!
currentElement
	^ self openElements lastOrNil! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 1/23/2014 17:52'!
currentScopedAttributes
	^ self scopedAttributes
		ifEmpty: [nil]
		ifNotEmpty: [:attributes | attributes last]! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 12/6/2013 17:25'!
enclosedElementsAsString: aTagName
	| enclosedTags |

	enclosedTags := String new writeStream.
	self
		printOn: enclosedTags
		startingAt: (self openElements lastIndexOf: aTagName) + 1.
	^ enclosedTags contents.! !

!XMLElementNester methodsFor: 'checking' stamp: 'monty 1/20/2014 19:59'!
endElement: anElementName
	self openElements removeLast.
	self hasScopedAttributes
		ifTrue: [scopedAttributes removeLast].! !

!XMLElementNester methodsFor: 'checking' stamp: 'monty 2/2/2014 02:45'!
endNesting! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 12/6/2013 17:30'!
hasExternalRoot
	^ hasExternalRoot ifNil: [hasExternalRoot := false]! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 12/6/2013 17:29'!
hasOpenElement: anElementName
	^ self openElements includes: anElementName! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 12/6/2013 17:29'!
hasOpenElements
	^ self openElements notEmpty! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 1/20/2014 19:59'!
hasScopedAttributes
	^ scopedAttributes notNil and: [scopedAttributes notEmpty]! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 2/10/2014 02:12'!
isCurrentElementRoot
	^ self openElements size = 1! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 2/1/2014 01:32'!
openElements
	^ openElements ifNil: [openElements := OrderedCollection new: 10]! !

!XMLElementNester methodsFor: 'enumerating' stamp: 'monty 2/15/2014 03:58'!
openElementsReverseDo: aBlock
	"reimplemented so it is always safe to remove at the end while scanning backwards"
	| i |

	i := self openElements size.
	[i > 0]
		whileTrue: [
			aBlock value: (openElements at: i).
			i := i - 1].! !

!XMLElementNester methodsFor: 'private' stamp: 'monty 12/6/2013 17:29'!
printLimit
	^ 5! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 12/6/2013 17:29'!
printOn: aStream
	self printOn: aStream startingAt: 1! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 2/10/2014 02:21'!
printOn: aStream startingAt: aPosition
	| endPosition |

	endPosition :=  self openElements size min: (aPosition + self printLimit - 1).
	(self openElements copyFrom: aPosition to: endPosition)
		do: [:each |
			aStream
				nextPut: $<;
				nextPutAll: each;
				nextPut: $>]
		separatedBy: [aStream nextPutAll: ', '].
	(endPosition < self openElements size)
		ifTrue: [aStream nextPutAll: '...'].! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 12/6/2013 17:28'!
rootElement
	^ rootElement ifNil: ['']! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 12/6/2013 17:28'!
rootElement: anElement
	rootElement := anElement.
	hasExternalRoot := true.! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 1/20/2014 20:00'!
scopedAttributeAt: aName
	^ self
		scopedAttributeAt: aName
		ifAbsent: ['']! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 1/21/2014 02:46'!
scopedAttributeAt: aName ifAbsent: aBlock
	^ self currentScopedAttributes
		ifNil: [aBlock value]
		ifNotNil: [:attributes |
			attributes
				at: aName
				ifAbsent: aBlock]! !

!XMLElementNester methodsFor: 'private' stamp: 'monty 1/20/2014 19:33'!
scopedAttributes
	^ scopedAttributes ifNil: [scopedAttributes := OrderedCollection new]! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 1/25/2014 21:24'!
setScopedAttributesNamed: aNameCollection from: aDictionary
	| attributes |

	aNameCollection do: [:each |
		(aDictionary at: each ifAbsent: [nil])
			ifNotNil: [:value |
				(attributes ifNil: [attributes := self copiedScopedAttributes])
					at: each
					put: value]].! !

!XMLElementNester methodsFor: 'checking' stamp: 'monty 1/20/2014 19:59'!
startElement: anElementName
	self openElements addLast: anElementName.
	self hasScopedAttributes
		ifTrue: [scopedAttributes addLast: self currentScopedAttributes].! !

XMLElementNester subclass: #XMLWellFormedElementNester
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLWellFormedElementNester commentStamp: '<historical>' prior: 0!
The class keeps track of elements as they start and end and ensures that they nest properly.!

!XMLWellFormedElementNester methodsFor: 'checking' stamp: 'monty 12/11/2013 21:43'!
endElement: anElementName
	(self currentElement = anElementName)
		ifFalse: [
			^ (self hasOpenElement: anElementName)
				ifTrue: [self errorImproperlyNestedElements: anElementName]
				ifFalse: [self errorUnmatchedEndTag: anElementName]].

	super endElement: anElementName.! !

!XMLWellFormedElementNester methodsFor: 'checking' stamp: 'monty 2/2/2014 02:44'!
endNesting
	self hasOpenElements
		ifTrue: [XMLWellFormednessException signal: 'Unterminated elements: ', self printString]! !

!XMLWellFormedElementNester methodsFor: 'errors' stamp: 'JAAyer 11/28/2010 07:03'!
errorImproperlyNestedElements: anElementName
	XMLWellFormednessException signal:
		('<{1}> element ended before its child {2} element(s)'
			format: (Array with: anElementName with: (self enclosedElementsAsString: anElementName)))! !

!XMLWellFormedElementNester methodsFor: 'errors' stamp: 'JAAyer 11/28/2010 07:03'!
errorUnmatchedEndTag: anElementName
	XMLWellFormednessException signal:
		('End </{1}> tag with no corresponding start <{1}> tag'
			format: (Array with: anElementName))! !

!XMLWellFormedElementNester methodsFor: 'checking' stamp: 'monty 12/11/2013 21:43'!
startElement: anElementName
	rootElement
		ifNil: [rootElement := anElementName]
		ifNotNil: [
			(self hasOpenElements or: [self hasExternalRoot])
				ifFalse: [
					XMLWellFormednessException signal:
						'Document cannot have multiple root elements']].

	super startElement: anElementName.! !

Object subclass: #XMLElementValidator
	instanceVariableNames: 'element'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLElementValidator commentStamp: '<historical>' prior: 0!
This is a base class for element content validators.!

XMLElementValidator subclass: #XMLAnyElementValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLAnyElementValidator commentStamp: '<historical>' prior: 0!
This class validates elements that can contain any content.!

!XMLAnyElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 04:42'!
atEnd
	^ true! !

XMLElementValidator subclass: #XMLDFAElementValidator
	instanceVariableNames: 'dfa currentState'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLDFAElementValidator commentStamp: '<historical>' prior: 0!
This class can use a DFA to validate element content.
!

!XMLDFAElementValidator class methodsFor: 'instance creation' stamp: 'monty 12/7/2013 19:47'!
element: aName dfa: aDfa
	^ self new
		setElement: aName
		dfa: aDfa! !

!XMLDFAElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 05:43'!
atEnd
	^ currentState isAccepting! !

!XMLDFAElementValidator methodsFor: 'copying' stamp: 'monty 11/18/2013 05:41'!
postCopy
	currentState := dfa! !

!XMLDFAElementValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:47'!
setElement: aName dfa: aDfa
	element := aName.
	dfa := aDfa.
	currentState := dfa.! !

!XMLDFAElementValidator methodsFor: 'validation' stamp: 'monty 12/28/2013 00:03'!
validatePCData: aString
	self errorUnexpectedPCData: aString! !

!XMLDFAElementValidator methodsFor: 'validation' stamp: 'monty 12/28/2013 00:21'!
validateStartTag: aName
	currentState :=
		currentState
			detectNextStateMatching: aName
			ifNone: [self errorUnexpectedElement: aName]! !

!XMLElementValidator class methodsFor: 'instance creation' stamp: 'monty 12/7/2013 19:48'!
element: aName
	^ self new element: aName! !

!XMLElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 04:41'!
atEnd
	self subclassResponsibility! !

!XMLElementValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:48'!
element
	^ element! !

!XMLElementValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:48'!
element: aName
	element := aName! !

!XMLElementValidator methodsFor: 'private' stamp: 'monty 12/7/2013 19:44'!
errorUnexpectedElement: aName
	XMLValidationException signal:
		'Unexpected <', aName, '> element in <', self element, '> element'.! !

!XMLElementValidator methodsFor: 'private' stamp: 'monty 12/7/2013 19:44'!
errorUnexpectedPCData: aString
	XMLValidationException signal:
		'Unexpected PCDATA in <', self element, '> element'.! !

!XMLElementValidator methodsFor: 'validation' stamp: 'monty 11/18/2013 05:24'!
validatePCData: aString! !

!XMLElementValidator methodsFor: 'validation' stamp: 'monty 11/18/2013 05:24'!
validateStartTag: aName! !

XMLElementValidator subclass: #XMLEmptyElementValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLEmptyElementValidator commentStamp: '<historical>' prior: 0!
This class validates empty elements.
!

!XMLEmptyElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 04:41'!
atEnd
	^ true! !

!XMLEmptyElementValidator methodsFor: 'private' stamp: 'monty 12/7/2013 19:44'!
errorMustBeEmpty
	XMLValidationException signal: 'Element <', self element, '> must be empty'! !

!XMLEmptyElementValidator methodsFor: 'validation' stamp: 'monty 11/18/2013 04:32'!
validatePCData: aString
	self errorMustBeEmpty! !

!XMLEmptyElementValidator methodsFor: 'validation' stamp: 'monty 11/18/2013 04:34'!
validateStartTag: aName
	self errorMustBeEmpty! !

XMLElementValidator subclass: #XMLMixedContentElementValidator
	instanceVariableNames: 'allowedElements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLMixedContentElementValidator commentStamp: '<historical>' prior: 0!
This class validates elements that can contain a mix of PCDATA and certain elements.!

!XMLMixedContentElementValidator class methodsFor: 'instance creation' stamp: 'monty 12/7/2013 19:50'!
element: aName allowedElements: aNameCollection
	^ self new
		setElement: aName
		allowedElements: aNameCollection! !

!XMLMixedContentElementValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:51'!
allowedElements
	^ allowedElements! !

!XMLMixedContentElementValidator methodsFor: 'testing' stamp: 'monty 11/19/2013 01:14'!
atEnd
	^ true! !

!XMLMixedContentElementValidator methodsFor: 'initialization' stamp: 'monty 12/7/2013 19:50'!
setElement: aName allowedElements: aNameCollection
	element := aName.
	allowedElements := aNameCollection.! !

!XMLMixedContentElementValidator methodsFor: 'validation' stamp: 'monty 12/7/2013 19:51'!
validateStartTag: aName
	(self allowedElements includes: aName)
		ifFalse: [self errorUnexpectedElement: aName]
	! !

Object subclass: #XMLFileHandle
	instanceVariableNames: 'path'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-File'!

!XMLFileHandle commentStamp: 'monty 1/23/2014 15:44' prior: 0!
This is an abstract class for file handles to read/write/delete files. Subclasses need to implement the abstract methods. The class defaultImplementation method should return the subclass to use for this platform.!

!XMLFileHandle class methodsFor: 'accessing' stamp: 'monty 1/23/2014 01:34'!
currentWorkingDirectory
	^ self implementation currentWorkingDirectoryPathString! !

!XMLFileHandle class methodsFor: 'private' stamp: 'monty 1/22/2014 18:50'!
currentWorkingDirectoryPathString
	self subclassResponsibility! !

!XMLFileHandle class methodsFor: 'accessing' stamp: 'monty 1/12/2014 01:40'!
defaultImplementation
	^ XMLFileSystemFileHandle! !

!XMLFileHandle class methodsFor: 'accessing' stamp: 'monty 1/23/2014 02:28'!
implementation
	^ self == XMLFileHandle
		ifTrue: [self defaultImplementation]
		ifFalse: [self]! !

!XMLFileHandle class methodsFor: 'instance creation' stamp: 'monty 1/23/2014 01:35'!
path: aSystemPath
	^ self implementation new path: aSystemPath! !

!XMLFileHandle methodsFor: 'deleting' stamp: 'monty 1/22/2014 22:49'!
delete
	self deleteFileOrDo: [:error | error resignalAs: XMLFileException]! !

!XMLFileHandle methodsFor: 'private' stamp: 'monty 1/21/2014 22:32'!
deleteFileOrDo: aBlock
	self subclassResponsibility! !

!XMLFileHandle methodsFor: 'accessing' stamp: 'monty 1/22/2014 19:07'!
fileSize
	self subclassResponsibility! !

!XMLFileHandle methodsFor: 'requesting' stamp: 'monty 1/13/2014 18:14'!
get
	^ self readStream! !

!XMLFileHandle methodsFor: 'requesting' stamp: 'monty 1/13/2014 18:14'!
getUpToLimit: aMaxSize
	^ self readStreamUpToLimit: aMaxSize! !

!XMLFileHandle methodsFor: 'testing' stamp: 'monty 1/9/2014 01:58'!
isFile
	self subclassResponsibility! !

!XMLFileHandle methodsFor: 'accessing' stamp: 'monty 1/9/2014 04:34'!
path
	^ path! !

!XMLFileHandle methodsFor: 'accessing' stamp: 'monty 1/9/2014 04:23'!
path: aPath
	path := aPath! !

!XMLFileHandle methodsFor: 'reading' stamp: 'monty 1/13/2014 18:13'!
readStream
	^ self readStreamUpToLimit: nil! !

!XMLFileHandle methodsFor: 'private' stamp: 'monty 1/13/2014 20:12'!
readStreamOnFileOrDo: aBlock
	self subclassResponsibility! !

!XMLFileHandle methodsFor: 'reading' stamp: 'monty 1/22/2014 22:49'!
readStreamUpToLimit: aMaxSize
	| fileSize |

	self isFile
		ifFalse: [XMLFileException signal: 'Not a file'].
	
	(fileSize := self fileSize) > 0
		ifFalse: [ ^ '' readStream].
	(aMaxSize notNil and: [fileSize > aMaxSize])
		ifTrue: [XMLLimitException signal: 'File size limit exceeded'].

	^ self readStreamOnFileOrDo: [:error | error resignalAs: XMLFileException]! !

!XMLFileHandle methodsFor: 'writing' stamp: 'monty 1/22/2014 22:49'!
writeStream
	^ self writeStreamOnFileOrDo: [:error | error resignalAs: XMLFileException]! !

!XMLFileHandle methodsFor: 'private' stamp: 'monty 1/21/2014 22:27'!
writeStreamOnFileOrDo: aBlock
	self subclassResponsibility! !

XMLFileHandle subclass: #XMLFileSystemFileHandle
	instanceVariableNames: 'fileReference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-File'!

!XMLFileSystemFileHandle commentStamp: 'monty 1/9/2014 23:29' prior: 0!
This class uses the Pharo FileSystem API to implement file IO.!

!XMLFileSystemFileHandle class methodsFor: 'private' stamp: 'monty 1/22/2014 18:50'!
currentWorkingDirectoryPathString
	| fileSystemPath |

	fileSystemPath := (Smalltalk at: #FileSystem) workingDirectory path.
	^ ($/ join: fileSystemPath segments) copyWithFirst: $/.! !

!XMLFileSystemFileHandle methodsFor: 'private' stamp: 'monty 1/22/2014 19:10'!
deleteFileOrDo: aBlock
	[self fileReference delete]
		on: self fileSystemError 
		do: aBlock! !

!XMLFileSystemFileHandle methodsFor: 'private' stamp: 'monty 1/13/2014 04:37'!
fileReference
	^ fileReference ifNil: [fileReference := path asFileReference]! !

!XMLFileSystemFileHandle methodsFor: 'accessing' stamp: 'monty 1/22/2014 19:07'!
fileSize
	^ self fileReference size! !

!XMLFileSystemFileHandle methodsFor: 'private' stamp: 'monty 1/22/2014 19:09'!
fileSystemError
	^ Smalltalk at: #FileSystemError! !

!XMLFileSystemFileHandle methodsFor: 'testing' stamp: 'monty 1/13/2014 04:37'!
isFile
	^ self fileReference isFile! !

!XMLFileSystemFileHandle methodsFor: 'private' stamp: 'monty 1/22/2014 19:10'!
readStreamOnFileOrDo: aBlock
	^ [self fileReference readStream]
		on: self fileSystemError
		do: aBlock! !

!XMLFileSystemFileHandle methodsFor: 'private' stamp: 'monty 1/22/2014 19:10'!
writeStreamOnFileOrDo: aBlock
	^ [self fileReference writeStream]
		on: self fileSystemError
		do: aBlock! !

Object subclass: #XMLHTTPRequest
	instanceVariableNames: 'url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-HTTP'!

!XMLHTTPRequest commentStamp: 'monty 1/23/2014 15:44' prior: 0!
A base class for HTTP requests. Subclasses need to implement the abstract requesting method to return a ReadStream on the respone or raise an exception if it is too large or an HTTP error happens. The class defaultImplementation method should return the subclass to use for this platform.!

!XMLHTTPRequest class methodsFor: 'accessing' stamp: 'monty 1/12/2014 01:37'!
defaultImplementation
	^ XMLZincHTTPRequest! !

!XMLHTTPRequest class methodsFor: 'accessing' stamp: 'monty 1/23/2014 02:28'!
implementation
	^ self == XMLHTTPRequest
		ifTrue: [self defaultImplementation]
		ifFalse: [self]! !

!XMLHTTPRequest class methodsFor: 'instance creation' stamp: 'monty 1/23/2014 01:35'!
url: aUrl
	^ self implementation new url: aUrl! !

!XMLHTTPRequest methodsFor: 'private' stamp: 'monty 1/13/2014 21:11'!
defaultTimeout
	^ 10! !

!XMLHTTPRequest methodsFor: 'requesting' stamp: 'monty 1/9/2014 05:08'!
get
	^ self getUpToLimit: nil! !

!XMLHTTPRequest methodsFor: 'private' stamp: 'monty 1/13/2014 19:42'!
getResponseUpToLimit: aMaxSize withTimeout: aTimeout onError: aBlock
	self subclassResponsibility! !

!XMLHTTPRequest methodsFor: 'requesting' stamp: 'monty 1/22/2014 22:50'!
getUpToLimit: aMaxSize
	^ self
		getResponseUpToLimit: aMaxSize
		withTimeout: self defaultTimeout
		onError: [:error |
			(error isKindOf: XMLLimitException)
				ifTrue: [
					error
						messageText: 'HTTP reponse limit exceeded';
						pass]
				ifFalse: [error resignalAs: XMLHTTPException]]! !

!XMLHTTPRequest methodsFor: 'accessing' stamp: 'monty 1/10/2014 00:10'!
url
	^ url! !

!XMLHTTPRequest methodsFor: 'accessing' stamp: 'monty 1/10/2014 00:10'!
url: aUrl
	url := aUrl! !

XMLHTTPRequest subclass: #XMLZincHTTPRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-HTTP'!

!XMLZincHTTPRequest commentStamp: '<historical>' prior: 0!
This client uses Zinc-HTTP, but does not refer to Zinc classes directly to avoid compile errors on platforms without Zinc.!

!XMLZincHTTPRequest methodsFor: 'private' stamp: 'monty 1/23/2014 15:52'!
getResponseUpToLimit: aMaxSize withTimeout: aTimeout onError: aBlock
	^ (Smalltalk at: #ZnMaximumEntitySize)
		value: aMaxSize
		during: [
			((Smalltalk at: #ZnClient) new
				timeout: aTimeout;
				ifFail: [:error |
					(error isKindOf: (Smalltalk at: #ZnEntityTooLarge))
						ifTrue: [error resignalAs: XMLLimitException]
						ifFalse: [error pass]];
				url: url;
				get) readStream]! !

Object subclass: #XMLIntegerReader
	instanceVariableNames: 'stream'
	classVariableNames: 'HexDigitCharacterMap'
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLIntegerReader commentStamp: 'monty 1/21/2014 01:36' prior: 0!
This class reads hex or decimal integers from a minimal stream supporting next, peek, and atEnd.!

!XMLIntegerReader class methodsFor: 'class initialization' stamp: 'monty 1/21/2014 01:35'!
initialize
	"self initialize"

	super initialize.
	HexDigitCharacterMap := Dictionary new.
	($0 to: $9) do: [:each |
		HexDigitCharacterMap
			at: each
			put: each asciiValue - $0 asciiValue].
	($a to: $f) do: [:each |
		HexDigitCharacterMap
			at: each
			put: each asciiValue - $a asciiValue + 10.
		HexDigitCharacterMap
			at: each asUppercase
			put: (HexDigitCharacterMap at:each)].! !

!XMLIntegerReader class methodsFor: 'instance creation' stamp: 'monty 1/21/2014 01:27'!
on: aStream
	^ self new stream: aStream! !

!XMLIntegerReader class methodsFor: 'instance creation' stamp: 'monty 2/4/2014 00:36'!
readFrom: aStream
	^ (aStream atEnd not
		and: [aStream peek == $x])
		ifTrue: [
			aStream next.
			self readFrom: aStream withBase: 16]
		ifFalse: [self readFrom: aStream withBase: 10]! !

!XMLIntegerReader class methodsFor: 'reading' stamp: 'monty 1/21/2014 02:02'!
readFrom: aStream withBase: aBase
	^ self
		readFrom: aStream
		withBase: aBase
		upTo: nil! !

!XMLIntegerReader class methodsFor: 'reading' stamp: 'monty 1/21/2014 17:42'!
readFrom: aStream withBase: aBase upTo: anInteger
	^ (self on: aStream)
		nextIntegerWithBase: aBase
		upTo: anInteger! !

!XMLIntegerReader methodsFor: 'private' stamp: 'monty 1/21/2014 01:26'!
maxIntegerValue
	"SmallInteger maxValue"
	^ 1073741823! !

!XMLIntegerReader methodsFor: 'private' stamp: 'monty 1/21/2014 17:51'!
nextDigitWithBase: aBase
	| digit nextChar |
	
	nextChar := stream peek.
	aBase = 16
		ifTrue: [digit := HexDigitCharacterMap at: nextChar ifAbsent: [^ nil]]
		ifFalse: [
			digit := nextChar asciiValue - $0 asciiValue.
			(digit >= 0 and: [digit <= 10])
				ifFalse: [^ nil]].
	stream next.

	^ digit.! !

!XMLIntegerReader methodsFor: 'reading' stamp: 'monty 1/21/2014 02:14'!
nextIntegerWithBase: aBase upTo: anInteger
	| integer maxValue maxBaseValue digitValue readLimit |

	maxValue := self maxIntegerValue.
	maxBaseValue := (maxValue / aBase) asInteger.
	readLimit := anInteger.
	[stream atEnd
		or: [(readLimit notNil and: [readLimit <= 0])
			or: [(digitValue := self nextDigitWithBase: aBase) isNil]]]
		whileFalse: [
			integer ifNil: [integer := 0].
	
			integer > maxBaseValue
				ifTrue: [^ integer].
			integer := integer * aBase.

			integer > (maxValue - digitValue)
				ifTrue: [^ integer].
			integer := integer + digitValue.
			readLimit ifNotNil: [readLimit := readLimit - 1]].
	^ integer.! !

!XMLIntegerReader methodsFor: 'reading' stamp: 'monty 1/21/2014 01:28'!
stream: aStream
	stream := aStream! !

Object subclass: #XMLNFABuilder
	instanceVariableNames: 'fragmentStack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFABuilder commentStamp: 'monty 1/7/2014 19:39' prior: 0!
This class builds an NFA using a stack of NFAFragments.!

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/28/2013 01:26'!
applyBranch
	| branchState leftFragment  rightFragment |

	leftFragment := self popFragment.
	rightFragment := self popFragment.
	branchState :=
		XMLNFABranchingState
			leftBranch: leftFragment firstState
			rightBranch: rightFragment firstState.

	self pushFragment:
		(XMLNFAFragment
			firstState: branchState
			terminalStates: (leftFragment combinedTerminalStatesFrom: rightFragment)).! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/27/2013 23:46'!
applyConcatenation
	| firstFragment secondFragment |

	secondFragment := self popFragment.
	firstFragment := self popFragment.

	firstFragment
		connectToFragment: secondFragment;
		terminalStates: secondFragment terminalStates.

	self pushFragment: firstFragment.! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/27/2013 23:43'!
applyOptional
	| optionalFragment branchingState |

	optionalFragment := self popFragment.
	branchingState :=
		XMLNFABranchingState branch: optionalFragment firstState.

	self pushFragment:
		(XMLNFAFragment
			firstState: branchingState
			terminalStates: (optionalFragment terminalStatesWith: branchingState))! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/28/2013 01:19'!
applyPlus
	| plusFragment branchState |

	plusFragment := self topFragment.
	branchState := XMLNFABranchingState branch: plusFragment firstState.
	plusFragment
		connectToState: branchState;
		terminalState: branchState.! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/27/2013 23:55'!
applyStar
	| branchState starFragment |

	starFragment := self popFragment.
	branchState := XMLNFABranchingState branch: starFragment firstState.
	starFragment connectToState: branchState.

	self pushFragment:
		(XMLNFAFragment
			firstState: branchState
			terminalState: branchState)! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/28/2013 22:51'!
firstStateOfAcceptingNFA
	| firstFragment |

	[self totalFragments > 1]
		whileTrue: [self applyConcatenation].

	firstFragment := self popFragment.
	firstFragment connectToState: XMLNFAAcceptingState new.

	^ firstFragment firstState.! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 14:16'!
fragmentStack
	^ fragmentStack ifNil: [fragmentStack := OrderedCollection new]! !

!XMLNFABuilder methodsFor: 'testing' stamp: 'monty 12/6/2013 06:40'!
hasFragments
	^ self fragmentStack notEmpty! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 08:01'!
popFragment
	^ self fragmentStack removeLast! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 08:01'!
pushFragment: aFragment
	self fragmentStack addLast: aFragment! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/27/2013 21:41'!
pushMatcher: aMatcher
	| firstState |

	firstState := XMLNFAMatchingState matcher: aMatcher.
	self pushFragment:
		(XMLNFAFragment
			firstState: firstState
			terminalState: firstState).! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 11/17/2013 12:01'!
topFragment
	^ self fragmentStack last! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 15:04'!
totalFragments
	^ self fragmentStack size! !

Object subclass: #XMLNFAFragment
	instanceVariableNames: 'firstState terminalStates'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAFragment commentStamp: 'monty 12/28/2013 21:51' prior: 0!
This is a fragment used to build an NFA by the NFABuilder.!

!XMLNFAFragment class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 23:50'!
firstState: aFirstState terminalState: aTerminalState
	^ self new
		firstState: aFirstState;
		terminalState: aTerminalState! !

!XMLNFAFragment class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 23:50'!
firstState: aFirstState terminalStates: aTerminalStateCollection
	^ self new
		firstState: aFirstState;
		terminalStates: aTerminalStateCollection! !

!XMLNFAFragment methodsFor: 'converting' stamp: 'monty 12/27/2013 23:14'!
combinedTerminalStatesFrom: aFragment
	^ terminalStates isArray
		ifTrue: [
			aFragment terminalStates asOrderedCollection
				addAll: terminalStates;
				yourself]
		ifFalse: [
			terminalStates
				addAll: aFragment terminalStates;
				yourself]! !

!XMLNFAFragment methodsFor: 'connecting' stamp: 'monty 12/27/2013 23:46'!
connectToFragment: aFragment
	self connectToState: aFragment firstState! !

!XMLNFAFragment methodsFor: 'connecting' stamp: 'monty 12/27/2013 23:46'!
connectToState: aState
	self terminalStates do: [:each | each connectToState: aState]! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 11/17/2013 10:48'!
firstState
	^ firstState! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 11/17/2013 11:51'!
firstState: aState
	firstState := aState! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 12/27/2013 23:50'!
terminalState: aTerminalState
	terminalStates := Array with: aTerminalState! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 12/27/2013 21:46'!
terminalStates
	^ terminalStates! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 12/27/2013 23:50'!
terminalStates: aTerminalStateCollection
	terminalStates := aTerminalStateCollection! !

!XMLNFAFragment methodsFor: 'converting' stamp: 'monty 12/27/2013 23:40'!
terminalStatesWith: aState
	^ terminalStates isArray
		ifTrue: [OrderedCollection with: terminalStates first with: aState]
		ifFalse: [
			terminalStates
				add: aState;
				yourself]! !

Object subclass: #XMLNFAState
	instanceVariableNames: 'lastAddedTo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAState commentStamp: 'monty 12/28/2013 22:34' prior: 0!
This is a base class for NFA states.!

XMLNFAState subclass: #XMLNFAAcceptingState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAAcceptingState commentStamp: 'monty 10/7/2014 01:27' prior: 0!
An NFA in this state is considered to accept the input.!

!XMLNFAAcceptingState methodsFor: 'testing' stamp: 'monty 12/27/2013 21:23'!
isAccepting
	^ true! !

XMLNFAState subclass: #XMLNFABranchingState
	instanceVariableNames: 'leftBranch rightBranch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFABranchingState commentStamp: 'monty 10/7/2014 01:27' prior: 0!
This state can branch off into two different states without matching against anything.!

!XMLNFABranchingState class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 23:43'!
branch: aLeftBranch
	^ self new
		setLeftBranch: aLeftBranch
		rightBranch: nil! !

!XMLNFABranchingState class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 22:04'!
leftBranch: aLeftBranch rightBranch: aRightBranch
	^ self new
		setLeftBranch: aLeftBranch
		rightBranch: aRightBranch! !

!XMLNFABranchingState methodsFor: 'adding' stamp: 'monty 12/29/2013 01:10'!
addTo: aStateSet
	lastAddedTo == aStateSet
		ifFalse: [
			"This avoids getting caught in a loop"
			lastAddedTo := aStateSet.

			leftBranch addTo: aStateSet.
			rightBranch addTo: aStateSet]! !

!XMLNFABranchingState methodsFor: 'connecting' stamp: 'monty 12/27/2013 23:45'!
connectToState: aState
	leftBranch ifNil: [leftBranch := aState].
	rightBranch ifNil: [rightBranch := aState].! !

!XMLNFABranchingState methodsFor: 'initialization' stamp: 'monty 12/28/2013 00:45'!
setLeftBranch: aLeftBranch rightBranch: aRightBranch
	leftBranch := aLeftBranch.
	rightBranch := aRightBranch.! !

XMLNFAState subclass: #XMLNFAMatchingState
	instanceVariableNames: 'matcher nextState'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAMatchingState commentStamp: 'monty 10/9/2014 20:58' prior: 0!
This state can pass to a next state if the input matches against an object.!

!XMLNFAMatchingState class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 21:15'!
matcher: aMatcher
	^ self new matcher: aMatcher! !

!XMLNFAMatchingState methodsFor: 'connecting' stamp: 'monty 12/28/2013 21:52'!
connectToState: aState
	nextState := aState! !

!XMLNFAMatchingState methodsFor: 'accessing' stamp: 'monty 12/27/2013 21:16'!
matcher: aMatcher
	matcher := aMatcher! !

!XMLNFAMatchingState methodsFor: 'testing' stamp: 'monty 12/27/2013 21:15'!
matches: anObject
	^ matcher = anObject! !

!XMLNFAMatchingState methodsFor: 'accessing' stamp: 'monty 12/27/2013 21:13'!
nextState
	^ nextState! !

!XMLNFAState methodsFor: 'adding' stamp: 'monty 12/29/2013 01:10'!
addTo: aStateSet
	lastAddedTo == aStateSet
		ifFalse: [
			"This avoids getting caught in a loop"
			lastAddedTo := aStateSet.

			aStateSet add: self]! !

!XMLNFAState methodsFor: 'converting' stamp: 'monty 12/28/2013 22:33'!
asDFAState
	| nfaStates |

	nfaStates := XMLNFAStateSet new.
	self addTo: nfaStates.

	^ XMLDFAState nfaStates: nfaStates.! !

!XMLNFAState methodsFor: 'connecting' stamp: 'monty 12/27/2013 23:45'!
connectToState: aState! !

!XMLNFAState methodsFor: 'testing' stamp: 'monty 1/7/2014 19:37'!
isAccepting
	^ false! !

!XMLNFAState methodsFor: 'testing' stamp: 'monty 12/27/2013 21:10'!
matches: anObject
	^ false! !

Object subclass: #XMLNamespaceScope
	instanceVariableNames: 'prefixMappings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Namespaces'!

!XMLNamespaceScope commentStamp: 'monty 2/2/2014 06:17' prior: 0!
A namespace scope is a possibly empty set of mappings from namespace prefixes to URIs and an optional default namespace URI to be used for unprefixed elements. Scopes can be created from other scopes by copying.!

!XMLNamespaceScope class methodsFor: 'predefined namespaces' stamp: 'monty 10/8/2014 23:27'!
xmlNamespaceURI
	^ 'http://www.w3.org/XML/1998/namespace'! !

!XMLNamespaceScope class methodsFor: 'predefined namespaces' stamp: 'monty 10/8/2014 23:27'!
xmlnsNamespaceURI
	^ 'http://www.w3.org/2000/xmlns/'! !

!XMLNamespaceScope methodsFor: 'comparing' stamp: 'monty 10/12/2014 06:32'!
= anObject
	self == anObject
		ifTrue: [^ true].
	self class == anObject class
		ifFalse: [^ false].

	^ self hasNamespaces
		ifTrue: [anObject hasNamespaces and: [prefixMappings = anObject prefixMappings]]
		ifFalse: [anObject hasNamespaces not].! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 10/10/2014 05:47'!
defaultNamespace
	^ prefixMappings
		ifNil: ['']
		ifNotNil: [prefixMappings at: '' ifAbsent: ['']]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 10/10/2014 05:46'!
defaultNamespace: aUri
	(aUri = self class xmlNamespaceURI
		or: [aUri = self class xmlnsNamespaceURI])
		ifTrue: [
			XMLNamespaceException signal:
				'Cannot use "xml" or  "xmlns" namespace URI as default namespace'].

	self prefixMappings at: '' put: aUri.! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'monty 10/12/2014 03:21'!
hasDefaultNamespace
	^ self includesPrefix: ''! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'monty 10/10/2014 05:51'!
hasNamespaces
	^ prefixMappings notNil and: [prefixMappings notEmpty]! !

!XMLNamespaceScope methodsFor: 'comparing' stamp: 'monty 10/12/2014 04:00'!
hash
	| hash |

	hash := self class hash.
	self prefixesAndURIsDo: [:prefix :uri | hash := hash bitXor: uri].
	^ hash.! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'monty 10/12/2014 03:19'!
includesPrefix: aPrefix
	self prefixMappingsAt: aPrefix ifAbsent: [^ false].
	^ true.! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'monty 10/12/2014 03:19'!
includesPrefix: aPrefix mappedTo: aUri
	^ (self prefixMappingsAt: aPrefix ifAbsent: [^ false]) = aUri! !

!XMLNamespaceScope methodsFor: 'initializing' stamp: 'monty 10/12/2014 05:05'!
inheritMappingsFrom: anEnclosingScope
	anEnclosingScope hasNamespaces
		ifTrue: [
			anEnclosingScope prefixesAndURIsDo: [:prefix :uri |
				(self includesPrefix: prefix)
					ifFalse: [self mapPrefix: prefix to: uri]]]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 10/12/2014 01:38'!
mapPrefix: aPrefix to: aUri
	aPrefix isEmpty
		ifTrue: [^ self defaultNamespace: aUri].
	(aPrefix at: 1) == $x "skip comparisons below"
		ifTrue: [
			aPrefix = 'xml'
				ifTrue: [
					aUri = self class xmlNamespaceURI
						ifFalse: [
							XMLNamespaceException signal:
								'Cannot map the "xml" prefix to a new namespace URI'].
					self prefixMappings at: 'xml' put: aUri.
					^ self].
			aPrefix = 'xmlns'
				ifTrue: [XMLNamespaceException signal: 'Cannot bind "xmlns" prefix at all']].
	(aPrefix indexOf: $:) > 0 
		ifTrue: [XMLNamespaceException signal: 'Prefix "', aPrefix, '" contains ":" character'].

	aUri isEmpty
		ifTrue: [XMLNamespaceException signal: 'Empty URIs cannot be mapped to prefixes'].
	(aUri = self class xmlNamespaceURI or: [aUri = self class xmlnsNamespaceURI])
		ifTrue: [XMLNamespaceException signal: 'Cannot alias the "xml" or "xmlns" prefixes'].

	self prefixMappings at: aPrefix put: aUri.! !

!XMLNamespaceScope methodsFor: 'copying' stamp: 'monty 10/10/2014 05:51'!
postCopy
	prefixMappings ifNotNil: [prefixMappings := prefixMappings copy].! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 1/30/2014 22:33'!
prefixMappings
	^ prefixMappings ifNil: [prefixMappings := OrderPreservingDictionary new: 5 withDefaultValue: '']! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 10/12/2014 03:10'!
prefixMappingsAt: aPrefix ifAbsent: aBlock
	^ prefixMappings
		ifNil: [aBlock value]
		ifNotNil: [prefixMappings at: aPrefix ifAbsent: aBlock]! !

!XMLNamespaceScope methodsFor: 'enumerating' stamp: 'monty 10/12/2014 04:01'!
prefixesAndURIsDo: aTwoArgumentBlock
	prefixMappings ifNotNil: [prefixMappings keysAndValuesDo: aTwoArgumentBlock]! !

!XMLNamespaceScope methodsFor: 'printing' stamp: 'monty 10/14/2014 05:42'!
printOn: aStream
	self prefixesAndURIsDo: [:prefix :uri |
		aStream nextPutAll: 'xmlns'.
		prefix ifNotEmpty: [
			aStream
				nextPut: $:;
				nextPutAll: prefix].
		aStream
			nextPut: $=;
			nextPut: $";
			nextPutAll: uri;
			nextPut: $";
			space]! !

!XMLNamespaceScope methodsFor: 'resolving' stamp: 'monty 10/12/2014 02:28'!
resolvePrefix: aPrefix
	^ self resolvePrefix: aPrefix ifUnresolvable: ['']! !

!XMLNamespaceScope methodsFor: 'resolving' stamp: 'monty 10/12/2014 05:13'!
resolvePrefix: aPrefix ifUnresolvable: aBlock
	^ self
		prefixMappingsAt: aPrefix
		ifAbsent: [
			aPrefix = 'xml'
				ifTrue: [self class xmlNamespaceURI]
				ifFalse: [
					aPrefix = 'xmlns'
						ifTrue: [self class xmlnsNamespaceURI]
						ifFalse: [aBlock value]]]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 10/10/2014 22:57'!
unmapPrefix: aPrefix
	self prefixMappings
		removeKey: aPrefix
		ifAbsent: [
			XMLNamespaceException signal:
				'Prefix "', aPrefix, '" was not mapped and cannot be unmapped']! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 10/12/2014 06:40'!
validateAndResolvePrefix: aPrefix
	^ self
		prefixMappingsAt: aPrefix
		ifAbsent: [
			aPrefix = 'xml'
				ifTrue: [self class xmlNamespaceURI]
				ifFalse: [
					XMLNamespaceException signal:
						(aPrefix = 'xmlns'
							ifTrue: ['Cannot use reserved "xmlns" prefix for elements or attributes']
							ifFalse: ['Unmapped namespace prefix "', aPrefix, '" in name'])]]! !

!XMLNamespaceScope methodsFor: 'validation' stamp: 'monty 10/9/2014 17:39'!
validateAndResolvePrefix: aPrefix ofElement: aName
	aPrefix isEmpty
		ifTrue: [
			self validateUnprefixedName: aName.
			^ self defaultNamespace]
		ifFalse: [^ self validateAndResolvePrefix: aPrefix]! !

!XMLNamespaceScope methodsFor: 'validation' stamp: 'monty 10/9/2014 17:39'!
validateAttributePrefixes: aDictionary
	"performance critical method"
	| namespacedAttributes |

	aDictionary keysDo: [:each | | prefix |
		(prefix := each qualifiedNamePrefix) isEmpty "avoid ifNotEmpty: closure overhead"
			ifTrue: [self validateUnprefixedName: each]
			ifFalse: [ | namespace oldSize |
				"not initialized unless needed"
				namespacedAttributes
					ifNil: [namespacedAttributes := Set new: aDictionary size].
				namespace := self validateAndResolvePrefix: prefix.

				"checking size after is faster than includes:"
				oldSize := namespacedAttributes size.
				namespacedAttributes add: namespace, (each qualifiedNameLocalPartFromPrefix: prefix).
				namespacedAttributes size > oldSize
					ifFalse: [
						XMLNamespaceException signal:
							'Attribute "', each, '" has alias for namespace "', namespace, '"']]].! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 10/9/2014 06:53'!
validateUnprefixedName: aName
	"catch names like ':name' with an empty prefix"
	(aName size > 0
		and: [(aName at: 1) == $:])
		ifTrue: [XMLNamespaceException signal: 'Name "', aName, '" contains ":" character'].! !

Object subclass: #XMLNamespaceScopeAllocator
	instanceVariableNames: 'scopes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Namespaces'!

!XMLNamespaceScopeAllocator commentStamp: '<historical>' prior: 0!
This class handles the allocation of namespace scopes using a stack. Sending #enterScope to an instance will cause it to push a new scope onto the stack that inherits all mappings from the previous, enclosing scope and makes it the current scope. Sending #exitScop causes that scope to be removed and the parent scope to again be the current scope.!

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'monty 10/7/2014 03:06'!
currentScope
	^ self scopes lastOrNil ifNil: [self emptyScope]! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'monty 2/1/2014 04:44'!
currentScope: aScope
	self scopes addLast: aScope! !

!XMLNamespaceScopeAllocator methodsFor: 'private' stamp: 'JAAyer 3/13/2010 13:44'!
emptyScope
	^ XMLNamespaceScope new! !

!XMLNamespaceScopeAllocator methodsFor: 'scope' stamp: 'monty 2/8/2014 02:33'!
enterScope
	^ self scopes addLast: self currentScope copy! !

!XMLNamespaceScopeAllocator methodsFor: 'scope' stamp: 'monty 2/8/2014 02:39'!
exitScope
	^ self scopes removeLast! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'monty 12/6/2013 06:47'!
scopes
	^ scopes ifNil: [scopes := OrderedCollection new: 10]! !

Object subclass: #XMLNestedStreamReader
	instanceVariableNames: 'stream nestedStreams peekChar writeBuffer normalizedLineEndingChar currentLineNumber readLimit'
	classVariableNames: 'CarriageReturn LineFeed'
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLNestedStreamReader commentStamp: '<historical>' prior: 0!
This class represents a stream reader that implements pushBack: using nested streams. This enables subsitution/replacement to be performed without modifying the underlying collections streamed-over or having to copy them. It also performs line-ending normalization, transforming CR and CRLF sequences into a single LF character.

(The code in this class is optimized; refactor with care.)!

!XMLNestedStreamReader class methodsFor: 'class initialization' stamp: 'monty 2/10/2014 06:33'!
initialize
	"self initialize"

	super initialize.

	CarriageReturn := Character cr.
	LineFeed := Character lf.! !

!XMLNestedStreamReader class methodsFor: 'instance creation' stamp: 'monty 1/4/2014 08:05'!
on: anInitialStream readLimit: aReadLimit
	^ self new
		setStream: anInitialStream
		readLimit: aReadLimit! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'monty 1/29/2014 23:15'!
atEnd
	^ peekChar == nil
		and: [stream atEnd
			and: [self hasNestedStreams not
				or: [nestedStreams allSatisfy: [:each | each atEnd]]]]! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'monty 2/6/2014 01:01'!
atQuote
	^ self peek == $"
		or: [peekChar == $']! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 1/17/2014 22:14'!
basicNext
	"Returns next character in the stream after performing line-ending normalization."

	| nextChar |

	(nextChar := stream next)
		ifNil: [
			[stream atEnd and: [self hasNestedStreams]]
				whileTrue: [self popCurrentStream].
			nextChar := stream next].

	nextChar == CarriageReturn
		ifTrue: [
			nextChar := normalizedLineEndingChar.
			[stream atEnd and: [self hasNestedStreams]]
				whileTrue: [self popCurrentStream].
			(stream peek == LineFeed)
				ifTrue: [stream next]]
		ifFalse: [
			(nextChar == LineFeed)
				ifTrue: [nextChar := normalizedLineEndingChar]].

	^ nextChar.! !

!XMLNestedStreamReader methodsFor: 'file open/close' stamp: 'monty 2/20/2014 01:50'!
close
	stream close.
	nestedStreams ifNotNil: [nestedStreams reverseDo: [:each | each close]].! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 7/14/2010 00:39'!
currentLineNumber
	^ currentLineNumber! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'monty 1/4/2014 08:07'!
hasNestedStreams
	^ nestedStreams notNil and: [nestedStreams notEmpty]! !

!XMLNestedStreamReader methodsFor: 'initialization' stamp: 'monty 1/4/2014 08:10'!
initialize
	super initialize.

	nestedStreams := nil.
	peekChar := nil.
	writeBuffer := WriteStream on: (String new: 128).
	currentLineNumber := 1.
	normalizedLineEndingChar := LineFeed.! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 12/6/2013 06:47'!
maxNestedStreams
	^ 10! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 10/14/2014 01:10'!
next
	| nextChar |

	peekChar
		ifNil: [nextChar := self basicNext]
		ifNotNil: [
			nextChar := peekChar.
			peekChar := nil].

	nextChar == normalizedLineEndingChar
		ifTrue: [currentLineNumber := currentLineNumber + 1].
	readLimit ifNotNil: [
		(readLimit := readLimit - 1) >= 0
			ifFalse: [XMLLimitException signal: 'Security read limit on input exceeded']].

	^ nextChar.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 2/4/2014 02:55'!
nextInteger
	^ XMLIntegerReader readFrom: self.! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'monty 2/1/2014 04:26'!
nextMatchAll: aString
	| i nextChar |

	"inlined peek"
	(peekChar ifNil: [peekChar := self basicNext]) == (aString at: 1)
		ifFalse: [^ false].

	"we unrolled the first iteration with the above #peek"
	i := 2.
	self next.
	[(i <= aString size)
		and: [(aString at: i) == (nextChar := self next)]]
		whileTrue: [i := i + 1].
	(i > aString size)
		ifTrue: [^ true].

	self pushBack:
		(nextChar
			ifNotNil: [
				(aString copyFrom: 1 to: i)
					at: i put: nextChar;
					yourself]
			ifNil: [aString copyFrom: 1 to: i - 1]).
	^ false.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 1/16/2014 09:08'!
normalizedLineEndingChar
	^ normalizedLineEndingChar! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 12/18/2013 17:25'!
normalizedLineEndingChar: aCharacter
	normalizedLineEndingChar := aCharacter! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 2/2/2014 02:43'!
peek
	^ peekChar ifNil: [peekChar := self basicNext]! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 1/4/2014 08:10'!
popCurrentStream
	stream close.
	stream := nestedStreams removeLast.! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 2/3/2014 20:17'!
pushBack: aString
	aString size > 0
		ifTrue: [self pushStream: (ReadStream on: aString)]! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 12/6/2013 06:45'!
pushBackPeekChar
	| peekCharString |

	peekCharString := String with: peekChar.
	peekChar := nil.
	self pushBack: peekCharString.! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 12/13/2013 05:05'!
pushStream: aStream
	self pushStream: aStream onClose: nil! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 10/14/2014 01:10'!
pushStream: aStream onClose: aBlock
	peekChar
		ifNotNil: [self pushBackPeekChar].

	nestedStreams
		ifNil: [nestedStreams := OrderedCollection with: stream]
		ifNotNil: [
			(nestedStreams size >= self maxNestedStreams)
				ifTrue: [XMLLimitException signal: 'Cannot further nest input streams'].
			nestedStreams addLast: stream].

	self setStream: aStream onClose: aBlock! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 12/13/2013 05:13'!
setStream: aNewStream onClose: aBlock
	stream :=
		XMLPeekableStreamAdapter
			on: aNewStream
			onClose: aBlock! !

!XMLNestedStreamReader methodsFor: 'initialization' stamp: 'monty 1/4/2014 08:18'!
setStream: anInitialStream readLimit: aReadLimit
	self
		setStream: anInitialStream
		onClose: nil.

	readLimit := aReadLimit.! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'monty 2/10/2014 06:39'!
skipSeparators
	"inlined peek"
	[(peekChar ifNil: [peekChar := self basicNext]) notNil
		and: [peekChar isSeparator]]
		whileTrue: [self next].! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 1/4/2014 08:09'!
skipTo: aDelimiter
	[self atEnd] whileFalse: [
		self next == aDelimiter
			ifTrue: [^ true]].
	^ false.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 3/11/2010 08:01'!
stream
	^ stream! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAA 3/22/2011 11:49'!
upTo: aDelimiter
	| nextChar |

	writeBuffer reset.
	[self atEnd or: [(nextChar := self next) == aDelimiter]]
		whileFalse: [writeBuffer nextPut: nextChar].

	^ writeBuffer contents.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAA 3/22/2011 11:49'!
upToAll: aDelimitingString
	"Answer a subcollection from the current access position to the occurrence
	(if any, but not inclusive) of delimitingString. If delimitingString is not
	in the stream, answer the entire rest of the stream."

	writeBuffer reset.
	[self atEnd or: [self nextMatchAll: aDelimitingString]]
		whileFalse: [writeBuffer nextPut: self next].
	^ writeBuffer contents.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 11/13/2013 02:59'!
upToEnd
	writeBuffer reset.
	[self atEnd] whileFalse: [writeBuffer nextPut: self next].
	^ writeBuffer contents.! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAAyer 12/14/2010 02:39'!
useConverterForEncoding: anEncodingName
	| converterClass |

	converterClass :=
		(Smalltalk
			at: #TextConverter
			ifAbsent: [^ self])
				defaultConverterClassForEncoding: anEncodingName asLowercase.
		
	converterClass
		ifNotNil: [stream converter: converterClass new].! !

Object subclass: #XMLNode
	instanceVariableNames: 'parent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNode commentStamp: '<historical>' prior: 0!
This is a base class for XML nodes. It contains testing messages and methods to set and access the parent node and sibling nodes. It also provides a number messages to control printing.!

XMLNode subclass: #XMLAttribute
	instanceVariableNames: 'name value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLAttribute commentStamp: 'monty 2/8/2014 02:27' prior: 0!
The class represents an attribute node, to be stored in an attribute node list. You generally won't need to interact with these objects directly; the various #attribute- messages element nodes respond to will normally suffice. The name of an attribute node can be namespaced using prefixes; however, the prefix must be mapped to a namespace URI in the containing element node. Unprefixed attributes have no namespace, not even a default namespace URI if one is in scope, per the XML namespace standard.!

!XMLAttribute class methodsFor: 'instance creation' stamp: 'monty 10/18/2014 04:03'!
name: aName value: aValue
	^ self new	
		setName: aName
		value: aValue
		element: nil! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 2/8/2014 02:25'!
element
	^ parent! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 2/8/2014 02:25'!
element: anElement
	parent := anElement! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 4/12/2010 00:37'!
expandedName
	^ self nodeName expandWith: self namespaceURI! !

!XMLAttribute methodsFor: 'testing' stamp: 'JAAyer 6/26/2010 07:31'!
hasPrefix
	^ self nodeName isPrefixed! !

!XMLAttribute methodsFor: 'testing' stamp: 'JAAyer 7/22/2010 08:29'!
isNamed: aName
	^ self nodeName isSameAs: aName! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:16'!
localName
	^ self nodeName localPart! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 7/12/2010 00:14'!
name
	^ self nodeName qualified! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 10/14/2014 07:07'!
name: aName
	self renameWith: [self nodeName qualified: aName]! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 22:02'!
namespaceURI
	(self element notNil and: [self hasPrefix])
		ifFalse: [^ ''].
	
	^ self element resolvePrefix: self prefix.! !

!XMLAttribute methodsFor: 'private' stamp: 'monty 10/14/2014 07:01'!
nodeName
	^ name ifNil: [name := XMLNodeName new]! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 3/17/2010 03:48'!
prefix
	^ self nodeName prefix! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 10/14/2014 07:07'!
prefix: aPrefix
	self renameWith: [self nodeName prefix: aPrefix]! !

!XMLAttribute methodsFor: 'private' stamp: 'monty 10/18/2014 05:56'!
renameWith: aBlock
	| oldName |

	oldName := self name.
	aBlock value.
	(self name ~= oldName
		and: [self hasParent])
			ifTrue: [
				self parent
					renamedAttribute: self
					from: oldName
					to: self name].! !

!XMLAttribute methodsFor: 'initialization' stamp: 'monty 10/18/2014 04:03'!
setName: aName value: aValue element: anElement
	name := XMLNodeName qualified: aName.
	value := aValue.
	parent := anElement.! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 03:02'!
value
	^ value ifNil: [value := '']! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 3/5/2010 08:18'!
value: aString
	value := aString! !

!XMLAttribute methodsFor: 'printing' stamp: 'monty 2/9/2014 04:31'!
writeXMLOn: aWriter
	aWriter
		writeName: self name
		escapedContentValue: self value! !

XMLNode subclass: #XMLComment
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLComment commentStamp: '<historical>' prior: 0!
This class represents comments preserved as nodes. Comments are ignored during parsing by default. If you want them preserved as comment nodes, use #preservesCommentNodes: with the DOM parser before parsing.!

!XMLComment class methodsFor: 'instance creation' stamp: 'JAAyer 8/11/2010 22:14'!
string: aString
	^ self new string: aString! !

!XMLComment methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:15'!
isComment
	^ true! !

!XMLComment methodsFor: 'accessing' stamp: 'JAAyer 8/11/2010 22:14'!
string
	^ string ifNil: [string := '']! !

!XMLComment methodsFor: 'accessing' stamp: 'JAAyer 8/11/2010 22:14'!
string: aString
	string := aString! !

!XMLComment methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter comment: self string! !

XMLNode subclass: #XMLDeclarationNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLDeclarationNode commentStamp: 'JAAyer 8/11/2010 21:03' prior: 0!
This is a generic class to represent markup declarations in the internal subset. Each subclass represents a specific kind of declaration, and each is immutable.!

XMLDeclarationNode subclass: #XMLAttributeDeclaration
	instanceVariableNames: 'element attribute type defaultPragma defaultValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLAttributeDeclaration commentStamp: 'monty 2/2/2014 16:47' prior: 0!
This is a class for <!!ATTLIST> declarations with definitions for single attributes. (An ATTLIST with multiple attribute defs is the same as a series of separate ATTLISTs for each def)!

!XMLAttributeDeclaration class methodsFor: 'instance creation' stamp: 'monty 12/30/2013 23:59'!
element: anElementName attribute: anAttributeName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	^ self new
		setElement: anElementName
		attribute: anAttributeName
		type: aType
		defaultPragma: aDefaultPragma
		defaultValue: aDefaultValue! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:53'!
attribute
	^ attribute ifNil: [attribute := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:14'!
defaultPragma
	^ defaultPragma ifNil: [defaultPragma := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:11'!
defaultValue
	^ defaultValue ifNil: [defaultValue := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:21'!
element
	^ element ifNil: [element := '']! !

!XMLAttributeDeclaration methodsFor: 'initialization' stamp: 'monty 12/31/2013 00:02'!
setElement: anElementName attribute: anAttributeName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	element := anElementName.
	attribute := anAttributeName.
	type := aType.
	defaultPragma := aDefaultPragma.
	defaultValue := aDefaultValue.! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:42'!
type
	^ type ifNil: [type := '']! !

!XMLAttributeDeclaration methodsFor: 'printing' stamp: 'monty 12/31/2013 00:16'!
writeXMLOn: aWriter
	aWriter attributeDeclaration
		element: self element;
		attribute: self attribute;
		type: self type;
		defaultPragma: self defaultPragma;
		defaultValue: self defaultValue! !

!XMLDeclarationNode methodsFor: 'testing' stamp: 'monty 12/22/2013 06:10'!
isDeclaration
	^ true! !

XMLDeclarationNode subclass: #XMLElementDeclaration
	instanceVariableNames: 'contentModel element'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLElementDeclaration commentStamp: 'monty 2/2/2014 16:46' prior: 0!
This is a class for <!!ELEMENT> declarations.!

!XMLElementDeclaration class methodsFor: 'instance creation' stamp: 'monty 12/21/2013 03:14'!
element: aName contentModel: aContentModelString
	^ self new setElement: aName contentModel: aContentModelString! !

!XMLElementDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
contentModel
	^ contentModel ifNil: [contentModel := '']! !

!XMLElementDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:21'!
element
	^ element ifNil: [element := '']! !

!XMLElementDeclaration methodsFor: 'initialization' stamp: 'monty 12/21/2013 03:15'!
setElement: aName contentModel: aContentModelString
	element := aName.
	contentModel := aContentModelString.! !

!XMLElementDeclaration methodsFor: 'printing' stamp: 'monty 12/21/2013 03:25'!
writeXMLOn: aWriter
	aWriter elementDeclaration
		element: self element;
		contentModel: self contentModel! !

XMLDeclarationNode subclass: #XMLEntityDeclaration
	instanceVariableNames: 'name replacement publicId systemId ndata'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLEntityDeclaration commentStamp: 'monty 2/2/2014 16:46' prior: 0!
This is an abstract class for general and parameter <!!ENTITY> declarations.!

!XMLEntityDeclaration class methodsFor: 'instance creation' stamp: 'JAAyer 7/20/2010 10:53'!
name: aName publicID: aPublicId systemID: aSystemId
	^ self new
		setName: aName
		replacement: nil
		publicID: aPublicId
		systemID: aSystemId
		ndata: nil! !

!XMLEntityDeclaration class methodsFor: 'instance creation' stamp: 'JAAyer 7/20/2010 10:52'!
name: aName replacement: aReplacement
	^ self new
		setName: aName
		replacement: aReplacement
		publicID: nil
		systemID: nil
		ndata: nil! !

!XMLEntityDeclaration methodsFor: 'testing' stamp: 'monty 11/30/2013 21:53'!
isParameter
	^ false! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:22'!
name
	^ name ifNil: [name := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
ndata
	^ ndata ifNil: [ndata := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
publicID
	^ publicId ifNil: [publicId := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
replacement
	^ replacement ifNil: [replacement := '']! !

!XMLEntityDeclaration methodsFor: 'initialization' stamp: 'JAAyer 7/20/2010 10:51'!
setName: aName replacement: aReplacement publicID: aPublicId systemID: aSystemId ndata: aNotationName
	name := aName.
	replacement := aReplacement.
	publicId := aPublicId.
	systemId := aSystemId.
	ndata := aNotationName.! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
systemID
	^ systemId ifNil: [systemId := '']! !

!XMLEntityDeclaration methodsFor: 'printing' stamp: 'monty 11/13/2013 19:35'!
writeXMLOn: aWriter
	aWriter entityDeclaration
		name: self name;
		isParameter: self isParameter;
		replacement: self replacement;
		publicID: self publicID;
		systemID: self systemID;
		ndata: self ndata! !

XMLEntityDeclaration subclass: #XMLGeneralEntityDeclaration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLGeneralEntityDeclaration commentStamp: 'monty 2/2/2014 16:47' prior: 0!
This is a class for internal/external parsed/unparsed general <!!ENTITY> declarations.!

!XMLGeneralEntityDeclaration class methodsFor: 'instance creation' stamp: 'monty 11/14/2013 19:41'!
name: aName publicID: aPublicId systemID: aSystemId ndata: aNotationName
	^ self new
		setName: aName
		replacement: nil
		publicID: aPublicId
		systemID: aSystemId
		ndata: aNotationName! !

XMLEntityDeclaration subclass: #XMLParameterEntityDeclaration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLParameterEntityDeclaration commentStamp: 'monty 2/2/2014 16:46' prior: 0!
This is class for internal/external parameter <!!ENTITY> declarations.!

!XMLParameterEntityDeclaration methodsFor: 'testing' stamp: 'monty 11/13/2013 19:34'!
isParameter
	^ true! !

XMLDeclarationNode subclass: #XMLNotationDeclaration
	instanceVariableNames: 'name publicId systemId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNotationDeclaration commentStamp: 'monty 2/2/2014 16:46' prior: 0!
This is class for <!!NOTATION> declarations.
!

!XMLNotationDeclaration class methodsFor: 'instance creation' stamp: 'JAAyer 7/21/2010 00:55'!
name: aName publicID: aPublicId systemID: aSystemId
	^ self new setName: aName publicID: aPublicId systemID: aSystemId! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:22'!
name
	^ name ifNil: [name := '']! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
publicID
	^ publicId ifNil: [publicId := '']! !

!XMLNotationDeclaration methodsFor: 'initialization' stamp: 'JAAyer 7/21/2010 00:54'!
setName: aName publicID: aPublicId systemID: aSystemId
	name := aName.
	publicId := aPublicId.
	systemId := aSystemId.! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
systemID
	^ systemId ifNil: [systemId := '']! !

!XMLNotationDeclaration methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter notationDeclaration
		name: self name;
		publicID: self publicID;
		systemID: self systemID! !

!XMLNode methodsFor: 'private' stamp: 'monty 2/11/2014 00:13'!
acceptsContent
	^ false! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:15'!
addContent: aStringOrStringNode
	XMLDOMException signal: 'Node cannot contain content'! !

!XMLNode methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 10:59'!
allNodesDo: aBlock
	aBlock value: self! !

!XMLNode methodsFor: 'converting' stamp: 'monty 2/6/2014 20:06'!
asString
	"for compatibility if Object>>asString isn't defined"
	^ self printString! !

!XMLNode methodsFor: 'printing' stamp: 'monty 10/12/2014 21:50'!
canonicallyPrintOn: aStream
	self
		withNewXMLWriterOn: aStream
		do: [ :writer | writer
										isCanonical: true;
										writeWith: [ :wr | self writeXMLOn: wr ] ]
		whenAbsent: [self printOn: aStream]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 11/30/2013 19:07'!
canonicallyPrinted
	^ String streamContents: [:stream | self canonicallyPrintOn: stream]! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:16'!
contentString
 	"This returns the combined content string of all descendants with spaces inserted wher necessary."
	^ String streamContents: [:stream | self printContentOn: stream]! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 18:49'!
hasChildren
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/25/2010 14:37'!
hasParent
	^ self parent notNil! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:16'!
isCData
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:15'!
isComment
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/30/2010 19:19'!
isContentNode
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 12/22/2013 06:10'!
isDeclaration
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/19/2010 23:50'!
isDocument
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/11/2010 22:59'!
isElement
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/13/2010 21:43'!
isPI
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/12/2010 22:33'!
isStringNode
	^ false! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:56'!
nextNode
	^ self parent ifNotNil: [self parent nodeAfter: self]! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
parent
	^ parent! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
parent: aParentNode
	parent := aParentNode! !

!XMLNode methodsFor: 'printing' stamp: 'DamienCassou 4/30/2013 17:07'!
prettyPrintOn: aStream
	self
		withNewXMLWriterOn: aStream
		do: [ :writer | writer
									enablePrettyPrinting;
									writeWith: [ :wr | self writeXMLOn: wr ] ]
		whenAbsent: [ self printOn: aStream ]! !

!XMLNode methodsFor: 'printing' stamp: 'JAAyer 11/6/2010 01:18'!
prettyPrinted
	^ String streamContents: [:stream | self prettyPrintOn: stream]! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:56'!
previousNode
	^ self parent ifNotNil: [self parent nodeBefore: self]! !

!XMLNode methodsFor: 'printing' stamp: 'JAAyer 10/20/2010 19:11'!
printContentOn: aStream! !

!XMLNode methodsFor: 'printing' stamp: 'DamienCassou 4/30/2013 17:41'!
printOn: aStream
	self
		withNewXMLWriterOn: aStream
		do: [ :writer | writer writeWith: [ :wr | self writeXMLOn: wr] ]
		whenAbsent: [ super printOn: aStream ]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 10/12/2014 21:26'!
printWithoutSelfClosingTagsOn: aStream
	self
		withNewXMLWriterOn: aStream
		do: [ :writer |
			writer
				outputsSelfClosingTags: false;
				writeWith: [ :wr | self writeXMLOn: wr ]]
		whenAbsent: [ self printOn: aStream ]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 10/12/2014 21:27'!
printedWithoutSelfClosingTags
	^ String streamContents: [:stream | self printWithoutSelfClosingTagsOn: stream]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 10/12/2014 21:52'!
withNewXMLWriterOn: aStream do: aOneArgBlock
	^ self
		withNewXMLWriterOn: aStream
		do: aOneArgBlock
		whenAbsent: [ Error signal: 'The XMLWriter package is required for that' ]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 10/12/2014 21:38'!
withNewXMLWriterOn: aStream do: aOneArgBlock whenAbsent: aZeroArgBlock
	^ (self class environment hasClassNamed: #XMLWriter)
		ifTrue: [ aOneArgBlock value: ((self class environment classNamed: #XMLWriter) on: aStream)]
		ifFalse: [ aZeroArgBlock value ]! !

!XMLNode methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	self subclassResponsibility! !

XMLNode subclass: #XMLNodeWithChildren
	instanceVariableNames: 'nodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeWithChildren commentStamp: '<historical>' prior: 0!
This class represents a node that can contain child nodes. You probably will only use its API through one its subclasses, specifically the document, element and doctype declaration node classes.

This class provides message to access, add and remove child nodes under the "accessing" category. The nodes themselves are stored in some type of XMLObservableList, the exact type of which varies depending on the subclass and can be accessed directly using #nodes. You can modify the #nodes list directly if you like, but adding nodes to it or removing nodes from it will have the effect of adding them to removing them from the parent node. If this is not what you want, then make sure you copy the return value of #nodes before manipulating it.

The "enumerating" category provides two different modes of enumeration: the #nodes* enumerating messages enumerate child nodes of the receiver only, while the #allNode* forms recursively enumerate all descendent nodes, including the receiver itself, using depth-first traversal.

The inner XML of a node with children can be accessed as a string using #innerXML and set using #innerXML:.!

XMLNodeWithChildren subclass: #XMLDoctypeDeclaration
	instanceVariableNames: 'root publicId systemId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLDoctypeDeclaration commentStamp: '<historical>' prior: 0!
This class represents the <!!DOCTYPE ...> declaration one often finds at the top of an XML document. It stores the root element name, formal public identifier (FPI) if any, system identifier (a URI) and then any nodes comprising the internal subset.!

!XMLDoctypeDeclaration class methodsFor: 'instance creation' stamp: 'JAAyer 7/21/2010 07:47'!
root: aRootElementName publicID: aPublicId systemID: aSystemId
	^ self new
		setRoot: aRootElementName
		publicID: aPublicId
		systemID: aSystemId! !

!XMLDoctypeDeclaration methodsFor: 'private' stamp: 'monty 11/26/2013 21:51'!
nodeListClass
	^ XMLInternalSubsetList! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 10/10/2014 07:12'!
publicID
	^ publicId ifNil: [publicId := '']! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 10/10/2014 07:12'!
root
	^ root ifNil: [root := '']! !

!XMLDoctypeDeclaration methodsFor: 'initialization' stamp: 'JAAyer 7/21/2010 07:45'!
setRoot: aRootElementName publicID: aPublicId systemID: aSystemId
	root := aRootElementName.
	publicId := aPublicId.
	systemId := aSystemId.! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 10/10/2014 07:13'!
systemID
	^ systemId ifNil: [systemId := '']! !

!XMLDoctypeDeclaration methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeXMLOn: aWriter
	aWriter doctype
		root: self root;
		publicID: self publicID;
		systemID: self systemID;
		withInternalSubset: [self writeInnerXMLOn: aWriter]! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:22'!
addNode: aNode
	^ self nodes addLast: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 8/14/2010 21:05'!
addNode: aNode after: aPriorNode
	^ self nodes add: aNode after: aPriorNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 8/14/2010 21:05'!
addNode: aNode before: aSubsequentNode
	^ self nodes add: aNode before: aSubsequentNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:22'!
addNodeFirst: aNode
	^ self nodes addFirst: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 05:12'!
addNodes: aNodeCollection
	^ self nodes addAll: aNodeCollection! !

!XMLNodeWithChildren methodsFor: 'notifications' stamp: 'monty 2/2/2014 02:40'!
addedNode: aNode
	aNode parent: self! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 12/4/2010 12:32'!
allNodes
	^ self allNodesCollect: [:each | each]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 10/14/2014 22:49'!
allNodesCollect: aBlock
	| collectedNodes |

	collectedNodes := self newNodeListForCollect.
	self allNodesDo: [:each | collectedNodes addLast: (aBlock value: each)].
	^ collectedNodes.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:17'!
allNodesDetect: aBlock
	^ self allNodesDetect: aBlock ifNone: [nil]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:21'!
allNodesDetect: aBlock ifNone: anExceptionBlock
	self allNodesDo: [:each | (aBlock value: each) ifTrue: [^ each]].
	^ anExceptionBlock value.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 10/19/2014 08:44'!
allNodesDo: aBlock
	aBlock value: self.
	self hasChildren
		ifTrue: [self nodesDo: [:each | each allNodesDo: aBlock]]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 10/14/2014 22:49'!
allNodesSelect: aBlock
	| selectedNodes |

	selectedNodes := self newNodeListForSelect.
	self allNodesDo: [:each |
		(aBlock value: each)
			ifTrue: [selectedNodes addLast: each]].
	^ selectedNodes.! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 11/26/2013 21:52'!
firstNode
	^ self nodes first! !

!XMLNodeWithChildren methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 18:48'!
hasChildren
	"Direct variable access avoids possibly unnecessary lazy initialization of the node list"

	^ nodes notNil and: [nodes notEmpty]! !

!XMLNodeWithChildren methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:23'!
includesNode: aNode
	^ self nodes includes: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 10/12/2014 21:45'!
innerXML
	self hasChildren
		ifTrue: [
			self
				withNewXMLWriterOn: (String new: self nodes size * 100) writeStream
				do: [ :writer |
					self writeInnerXMLOn: writer.
					^ writer contents]]
		ifFalse: [^ '']! !

!XMLNodeWithChildren methodsFor: 'parsing' stamp: 'JAAyer 11/26/2010 19:23'!
innerXML: aStringOrStream
	self removeNodes.

	(self innerXMLParserOn: aStringOrStream) parseDocument.! !

!XMLNodeWithChildren methodsFor: 'private' stamp: 'JAAyer 11/26/2010 19:23'!
innerXMLParserOn: aStringOrStream
	^ XMLDOMParser on: aStringOrStream withRootNode: self! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 11/26/2013 21:53'!
lastNode
	^ self nodes last! !

!XMLNodeWithChildren methodsFor: 'private' stamp: 'monty 10/14/2014 22:49'!
newNodeListForCollect
	^ XMLOrderedList new! !

!XMLNodeWithChildren methodsFor: 'private' stamp: 'monty 10/14/2014 22:49'!
newNodeListForSelect
	^ XMLOrderedList new! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:26'!
nodeAfter: aNode
	^ self nodes after: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:27'!
nodeBefore: aNode
	^ self nodes before: aNode! !

!XMLNodeWithChildren methodsFor: 'private' stamp: 'monty 11/26/2013 21:51'!
nodeListClass
	self subclassResponsibility! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 10/18/2014 01:00'!
nodes
	^ nodes ifNil: [nodes := self nodeListClass observer: self]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:13'!
nodesCollect: aBlock
	^ self nodes collect: aBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 13:45'!
nodesDetect: aBlock
	^ self nodesDetect: aBlock ifNone: [nil]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:14'!
nodesDetect: aBlock ifNone: anExceptionBlock
	^ self nodes detect: aBlock ifNone: anExceptionBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 7/20/2010 10:27'!
nodesDo: aBlock
	self nodes do: aBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:13'!
nodesSelect: aBlock
	^ self nodes select: aBlock! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNode: aNode
	^ self nodes remove: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNodes
	self nodes removeAll! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNodes: aNodeCollection
	^ self nodes removeAll: aNodeCollection! !

!XMLNodeWithChildren methodsFor: 'notifications' stamp: 'monty 2/2/2014 02:38'!
removedNode: aNode
	aNode parent: nil! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 11/5/2010 22:39'!
replaceNode: aNode with: aReplacementNode
	^ self nodes
		at: (self nodes indexOf: aNode)
		put: aReplacementNode! !

!XMLNodeWithChildren methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeInnerXMLOn: aWriter
	self nodesDo: [:each | each writeXMLOn: aWriter]! !

!XMLNodeWithChildren methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeXMLOn: aWriter
	self writeInnerXMLOn: aWriter! !

XMLNodeWithChildren subclass: #XMLNodeWithElements
	instanceVariableNames: 'configuration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeWithElements commentStamp: '<historical>' prior: 0!
This class represents a node with elements. You will likely use its API through its subclasses instead.

Instances provide "accessing" messages to retrieve child elements by their name and namespace information. The #elementAt: forms return the first matching element, while the #elementsAt: forms return all matching child elements.

As with node enumeration in the superclass, there are two different modes of enumeration: the #elements* enumerating messages that enumerate child elements only, and the #allElements* forms that enumerate all descendent elements, including the receiver if it is an element (but not if it is a document), using depth-first traversal.

The #findElementNamed:* forms can be used to search using depth-first traversal for a specific element. Searching will start with the receiver if it is an element.

All element name matching is done using both the qualified and local name of elements, so 'prefix:element-name' will only match 'prefix:element-name' while 'element-name' will match 'element-name', 'prefix:element-name' or 'different-prefix:element-name' and so on.!

XMLNodeWithElements subclass: #XMLDocument
	instanceVariableNames: 'version encoding doctypeDeclaration isStandalone'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLDocument commentStamp: '<historical>' prior: 0!
This class represents a document node, which is often the root of a node tree. It can contain XML version and encoding information, a DOCTYPE declaration with a possible internal subset and a root element child node.!

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/10/2010 05:16'!
doctypeDeclaration
	^ doctypeDeclaration! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/10/2010 05:16'!
doctypeDeclaration: aDoctypeDeclaration
	doctypeDeclaration := aDoctypeDeclaration! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 13:34'!
document
	^ self! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 09:26'!
document: aDocument
	self shouldNotImplement! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 03:03'!
encoding	
	^ encoding ifNil: [encoding := 'UTF-8']! !

!XMLDocument methodsFor: 'accessing' stamp: 'mir 1/17/2002 12:57'!
encoding: aString	
	encoding := aString! !

!XMLDocument methodsFor: 'testing' stamp: 'JAAyer 2/19/2010 23:51'!
isDocument
	^ true! !

!XMLDocument methodsFor: 'testing' stamp: 'monty 11/19/2013 05:38'!
isStandalone
	^ isStandalone ifNil: [isStandalone := false]! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 11/19/2013 05:40'!
isStandalone: aBoolean
	isStandalone := aBoolean! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 11/25/2010 15:28'!
root
	^ self firstElement! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 12/6/2013 06:33'!
root: anElement
	self
		removeNodes: self elements;
		addNode: anElement! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 01:48'!
version	
	^ version! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 19:39'!
version: aVersionNumber
	version := aVersionNumber! !

!XMLDocument methodsFor: 'printing' stamp: 'monty 11/19/2013 05:47'!
writeXMLOn: aWriter
	self version ifNotNil: [
		aWriter xml
			version: self version;
			encoding: self encoding;
			standalone: self isStandalone].
	self doctypeDeclaration
		ifNotNil: [self doctypeDeclaration writeXMLOn: aWriter].
	
	self writeInnerXMLOn: aWriter.! !

XMLNodeWithElements subclass: #XMLElement
	instanceVariableNames: 'name attributes namespaceScope document'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLElement commentStamp: '<historical>' prior: 0!
The class represents an element node. An element consists of a name and optionally attributes, namespace declarations and any number child nodes and elements.

The name can consist of a local name only or a qualified name, which is a namespace prefix and local name separated by a colon. If the name is qualfied and namespace support is enabled (the default), then the prefix must be mapped to a namespace URI in the element or its parent at the time the name is set. The class-side instance creation #name:namespaceURI:* messages and #name:namespaces:* messages allow you set both simulaniously, as does the instance-side #name:namespaceURI: message. If namespace support is disabled, prefixes are ignored (not validated). You can test the name of an element using #isNamed: and #isNamedAny:, which test the qualified and local names of an element for a match.

Namespaces can be declared using #declareNamespace:uri:, using #declareNamespaces: with a collection of associations (where an empty prefix represents the default namespace) or simultaniously while changing the element's name using #name:namespaceURI:. The default namespace can be accessed through #defaultNamespace and #defaultNamespace:. When you print an element, the minimum number of xmlns declarations necessary for it to be valid will be printed. You can query an element's namespace information using #namespaceURI, #prefix, #expandedName and other messages under "namespaces."" For an element to be namespaced, it must have a qualified name with a valid prefix or have a default namespace declaration in scope for it.

The #attribute- messages provide a Dictionary-like prototcol for manipulating attribute nodes. Attribute value accessors return empty strings when the specified value is not present. The underlying attribute node list can be accessed using #attributeNodes, and an order-preserving dictionary of attribute names/values can be obtained using #attributes.

See the comment for the superclass for how to access child nodes and elements.!

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 10/30/2010 22:09'!
name: aLocalName
	^ self new name: aLocalName! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:05'!
name: aLocalName attributes: aDictionaryOrArray
	^ (self named: aLocalName)
		setAttributes: aDictionaryOrArray! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaceURI: aUri
	^ self new name: aQualifiedName namespaceURI: aUri! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaceURI: aUri attributes: aDictionaryOrArray
	^ (self name: aQualifiedName namespaceURI: aUri)
		setAttributes: aDictionaryOrArray! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaces: aNamespaceDictionaryOrArray
	^ self new
		declareNamespaces: aNamespaceDictionaryOrArray;
		name: aQualifiedName! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaces: aNamespaceDictionaryOrArray attributes: anAttributeDictionaryOrArray
	^ (self
		name: aQualifiedName
		namespaces: aNamespaceDictionaryOrArray)
			setAttributes: anAttributeDictionaryOrArray! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 3/11/2010 23:10'!
named: aLocalName
	^ self new name: aLocalName! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 2/11/2014 00:15'!
addContent: aStringOrStringNode
	"Adds a string or string node. If the last child node is already a string node,
	the characters being added are concatenated to it."

	(self hasChildren and: [self lastNode acceptsContent])
		ifTrue: [self lastNode addContent: aStringOrStringNode]
		ifFalse: [self addNode: aStringOrStringNode asXMLString]! !

!XMLElement methodsFor: 'notifications' stamp: 'monty 10/18/2014 00:39'!
addedAttribute: anAttribute
	anAttribute element: self.
	self validateAttributes.! !

!XMLElement methodsFor: 'notifications' stamp: 'monty 2/2/2014 04:13'!
addedElement: anElement
	super addedElement: anElement.
	(self hasNamespaces
		and: [self namespaceScope ~= anElement namespaceScope])
		ifTrue: [anElement inheritNamespacesFrom: self]! !

!XMLElement methodsFor: 'enumerating' stamp: 'monty 10/19/2014 08:44'!
allElementsDo: aBlock
	"See superclass version."

	aBlock value: self.
	self hasChildren
		ifTrue: [self elementsDo: [:each | each allElementsDo: aBlock]]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:15'!
attributeAssociations
	^ self attributeNodes asNameValueAssociations! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:33'!
attributeAt: aName
	^ self attributeAt: aName ifAbsent: ['']! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:33'!
attributeAt: aName ifAbsent: aBlock
	^ self attributeNodes attributeValueAt: aName ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:33'!
attributeAt: aName ifAbsentPut: aBlock
	^ self attributeNodes attributeValueAt: aName ifAbsentPut: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:34'!
attributeAt: aName put: aValue
	^ self attributeNodes attributeValueAt: aName put: aValue! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:08'!
attributeNames
	^ self attributeNodes attributeNames! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:14'!
attributeNamesAndValuesDo: aTwoArgumentBlock
	self attributeNodes attributeNamesAndValuesDo: aTwoArgumentBlock! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:09'!
attributeNamesDo: aBlock
	self attributeNodes attributeNamesDo: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:27'!
attributeNodeAt: aName
	^ self attributeNodes attributeNodeAt: aName! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:27'!
attributeNodeAt: aName ifAbsent: aBlock
	^ self attributeNodes attributeNodeAt: aName ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:28'!
attributeNodeAt: aLocalName namespaceURI: aUri
	^ self attributeNodes attributeNodeAt: aLocalName namespaceURI: aUri! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:28'!
attributeNodeAt: aLocalName namespaceURI: aUri ifAbsent: aBlock
	^ self attributeNodes attributeNodeAt: aLocalName namespaceURI: aUri ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 10/18/2014 01:01'!
attributeNodes
	^ attributes ifNil: [attributes := XMLAttributeList observer: self]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:16'!
attributes
	^ self attributeNodes asDictionary! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
contentString: aString
	self
		removeNodes;
		addContent: aString! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 10/10/2014 05:56'!
declareNamespaces: aDictionaryOrArray
	aDictionaryOrArray associationsDo: [:each |
		self
			declarePrefix: each key
			uri: each value]! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 10/12/2014 03:16'!
declarePrefix: aPrefix uri: aUri
	| oldMapping |

	(oldMapping := self namespaceScope prefixMappingsAt: aPrefix ifAbsent: [nil]) = aUri
		ifFalse: [
			self
				updateEnclosedScopesWith: [:scope | scope mapPrefix: aPrefix to: aUri]
				whileTrue: [:scope |
					(scope includesPrefix: aPrefix) not
						or: [scope includesPrefix: aPrefix mappedTo: oldMapping]]]! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 10/10/2014 06:14'!
declaresDefaultNamespace
	"Did the receiver declare a new default namespace rather than inheriting one?"

	^ self namespaceScope hasDefaultNamespace
		and: [self isRoot or: [self defaultNamespace ~= self parent defaultNamespace]]! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 10/10/2014 06:38'!
declaresPrefix: aPrefix uri: aUri
	"Did the receiver declare a new prefix/uri pair rather than inheriting one?"

	^ (self isDeclaredPrefix: aPrefix uri: aUri)
		and: [self isRoot or: [(self parent isDeclaredPrefix: aPrefix uri: aUri) not]]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 6/20/2010 23:08'!
defaultNamespace
	^ self namespaceScope defaultNamespace! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 10/10/2014 06:13'!
defaultNamespace: aUri
	| oldNamespace |

	oldNamespace := self defaultNamespace.
	self
		updateEnclosedScopesWith: [:scope | scope defaultNamespace: aUri]
		whileTrue: [:scope | scope defaultNamespace = oldNamespace]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/12/2010 10:46'!
document
	^ document! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 10/14/2014 07:43'!
document: aDocument
	document := aDocument.

	self hasChildren
		ifTrue: [self elementsDo: [:each | each document: aDocument]]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 4/12/2010 00:37'!
expandedName
	^ self nodeName expandWith: self namespaceURI! !

!XMLElement methodsFor: 'testing' stamp: 'monty 10/14/2014 07:51'!
hasAttributes
	^ attributes notNil and: [attributes notEmpty]! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 10/16/2010 06:07'!
hasNamespaces
	^ namespaceScope notNil and: [namespaceScope hasNamespaces]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 6/26/2010 07:31'!
hasPrefix
	^ self nodeName isPrefixed! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 8/10/2010 04:11'!
hasStringNodes
	^ self nodes anySatisfy: [:each | each isStringNode]! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 12/6/2010 21:23'!
includesAttribute: aName
	^ self attributeNodes includesAttribute: aName.! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/10/2014 23:56'!
inheritNamespacesFrom: anElement
	self allElementsDo: [:each |
		each namespaceScope inheritMappingsFrom:
			anElement namespaceScope]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/24/2010 12:54'!
isDeclaredPrefix: aPrefix
	^ self namespaceScope includesPrefix: aPrefix! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/24/2010 12:54'!
isDeclaredPrefix: aPrefix uri: aUri
	^ self namespaceScope includesPrefix: aPrefix mappedTo: aUri! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 4/11/2010 22:59'!
isElement
	^ true! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 11/26/2010 12:04'!
isNamed: aName
	"Does the receiver's qualified or local name match the argument?'"
	^ self nodeName isSameAs: aName! !

!XMLElement methodsFor: 'testing' stamp: 'monty 10/14/2014 07:53'!
isNamedAny: aNameCollection
	"Does the receiver's qualified or local name match any of the names supplied as the argument?'"
	aNameCollection do: [:each |
		"inline isNamed:"
		(self nodeName isSameAs: each)
			ifTrue: [^ true]].
	^ false.! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 11/26/2010 12:03'!
isRoot
	"Is the receiver the root element?"
	^ (self hasParent and: [self parent isElement]) not! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:16'!
localName
	^ self nodeName localPart! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/12/2010 00:14'!
name
	^ self nodeName qualified! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 10/14/2014 07:07'!
name: aName
	self renameWith: [self nodeName qualified: aName]! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 2/1/2014 06:47'!
name: aQualifiedName namespaceURI: aUri
	self
		declarePrefix: (aQualifiedName qualifiedNamePrefix) uri: aUri;
		name: aQualifiedName.! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 6/20/2010 23:10'!
namespaceScope
	^ namespaceScope ifNil: [namespaceScope := XMLNamespaceScope new]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 3/14/2010 12:52'!
namespaceURI
	^ self resolvePrefix: self prefix! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:56'!
nextElement
	^ self parent ifNotNil: [self parent elementAfter: self]! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/14/2014 07:02'!
nodeName
	^ name ifNil: [name := XMLNodeName new]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 3/17/2010 03:48'!
prefix
	^ self nodeName prefix! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 10/14/2014 07:07'!
prefix: aPrefix
	self renameWith: [self nodeName prefix: aPrefix]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:57'!
previousElement
	^ self parent ifNotNil: [self parent elementBefore: self]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 12/6/2010 21:24'!
removeAttribute: aName
	^ (self attributeNodes removeAttributeNodeAt: aName) value.! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:54'!
removeAttributeNode: aNode
	^ self attributeNodes remove: aNode! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 08:42'!
removeAttributes
	^ self attributeNodes removeAll! !

!XMLElement methodsFor: 'notifications' stamp: 'monty 10/18/2014 00:43'!
removedAttribute: anAttribute
	anAttribute element: nil! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/18/2014 01:35'!
renameWith: aBlock
	| oldName |

	oldName := self name.
	aBlock value.
	self name = oldName
		ifFalse: [
			self validateName.
			self hasParent
				ifTrue: [
					self parent
						renamedElement: self
						from: oldName
						to: self name]].! !

!XMLElement methodsFor: 'notifications' stamp: 'monty 10/18/2014 05:56'!
renamedAttribute: anAttribute from: anOldName to: aNewName
	self attributeNodes	
		renamedAttribute: anAttribute
		from: anOldName
		to: aNewName.
	self validateAttributes.! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 6/20/2010 23:08'!
resolvePrefix: aPrefix
	^ self namespaceScope resolvePrefix: aPrefix! !

!XMLElement methodsFor: 'initialization' stamp: 'JAAyer 7/12/2010 14:40'!
setAttributes: aDictionaryOrArray
	self attributeNodes removeAll.
	aDictionaryOrArray associationsDo: [:each |
		self attributeNodes
			addName: each key
			value: each value].! !

!XMLElement methodsFor: 'initialization' stamp: 'monty 12/27/2013 03:42'!
setNamespaceScope: aNamespaceScope
	namespaceScope := aNamespaceScope! !

!XMLElement methodsFor: 'initialization' stamp: 'monty 10/18/2014 04:42'!
setQualifiedName: aQualifiedName prefix: aPrefix localName: aLocalName attributes: anAttributeDictionary
	name := XMLNodeName qualified: aQualifiedName localPart: aLocalName.

	anAttributeDictionary isEmpty "avoid ifNotEmpty: closure overhead"
		ifFalse: [
			"The setFrom method is used to avoid revalidating the
			attributes when adding them to the list"
			(attributes :=
				(XMLAttributeList new: anAttributeDictionary size)
					observer: self)
				setFromDictionary: anAttributeDictionary].! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:08'!
stringNodes
	^ self nodes select: [:each | each isStringNode]! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 7/22/2010 08:08'!
stringNodesDo: aBlock
	self nodes do: [:each | each isStringNode ifTrue: [aBlock value: each]]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 01:44'!
strings
	^ self stringNodes collect: [:each | each string]! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 8/9/2010 01:44'!
stringsDo: aBlock
	self stringNodesDo: [:each | aBlock value: each string]! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 10/9/2014 19:51'!
undeclarePrefix: aPrefix
	self namespaceScope unmapPrefix: aPrefix.
	self
		validateName;
		validateAttributes.! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 11/24/2010 12:23'!
updateEnclosedScopesWith: aBlock whileTrue: aConditionalBlock
	"Updates the receiver's scope and the scopes of all descendants with aBlock
	where aConditional is true. It stops descending down a path of a subtree
	as soon as aConditional is false, but continues down other paths."

	(aConditionalBlock value: self namespaceScope)
		ifTrue: [aBlock value: self namespaceScope]
		ifFalse: [^ self].

	self elementsDo: [:each |
		each
			updateEnclosedScopesWith: aBlock
			whileTrue: aConditionalBlock]! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/9/2014 21:08'!
validateAttributes
	self usesNamespaces
		ifTrue: [
			self namespaceScope
				validateAttributePrefixes: self attributeNodes attributesByName]! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/9/2014 06:42'!
validateName
	self usesNamespaces
		ifTrue: [
			self namespaceScope
				validateAndResolvePrefix: self prefix
				ofElement: self name]! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/12/2014 00:58'!
writeAttributesOn: aTagWriter
	self attributeNodes do: [:each |
		aTagWriter
			attributeAt: each name
			put: each value]! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/12/2014 00:58'!
writeNamespaceDeclarationsOn: aTagWriter
	self namespaceScope prefixesAndURIsDo: [:prefix :uri |
		((self declaresPrefix: prefix uri: uri)
			or: [aTagWriter isEmbedded not])
			ifTrue: [
				aTagWriter
					xmlnsAt: prefix
					put: uri]]! !

!XMLElement methodsFor: 'printing' stamp: 'monty 10/14/2014 08:08'!
writeXMLOn: aWriter
	| tagWriter |

	tagWriter := aWriter tag: self name.

	self hasNamespaces
		ifTrue: [self writeNamespaceDeclarationsOn: tagWriter].
	self hasAttributes 
		ifTrue: [self writeAttributesOn: tagWriter].

	tagWriter with: [
		self hasChildren
			ifTrue: [self writeInnerXMLOn: aWriter]].! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
addElement: anElement
	self addNode: anElement! !

!XMLNodeWithElements methodsFor: 'notifications' stamp: 'monty 2/2/2014 04:17'!
addedElement: anElement
	self addedNode: anElement.
	anElement
		configuration: self configuration;
		document: self document.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 12/4/2010 12:32'!
allElements
	^ self allElementsCollect: [:each | each]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 10/14/2014 22:49'!
allElementsCollect: aBlock
	| collectedElements |

	collectedElements := self newNodeListForCollect.
	self allElementsDo: [:each | collectedElements addLast: (aBlock value: each)].
	^ collectedElements.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 10:47'!
allElementsDetect: aBlock
	"Descend depth-first visiting each element with aBlock until one such evaluation
	of aBlock with an element is true, then return that element. If no evaluation is
	true, then return nil."

	^ self allElementsDetect: aBlock ifNone: [nil]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 14:20'!
allElementsDetect: aBlock ifNone: anExceptionBlock
	"Descend depth-first visiting each element with aBlock until one such evaluation
	of aBlock with an element is true, then return that element. If no evaluation is
	true, then return nil."

	self allElementsDo: [:each | (aBlock value: each) ifTrue: [^ each]].
	^ anExceptionBlock value.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 10/14/2014 22:36'!
allElementsDo: aBlock
	"Descend depth-first visiting each element with aBlock."

	self hasChildren
		ifTrue: [self elementsDo: [:each | each allElementsDo: aBlock]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/25/2010 13:58'!
allElementsNamed: aName
	^ self allElementsSelect: [:each | each isNamed: aName]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 13:17'!
allElementsNamed: aName do: aBlock
	self allElementsDo: [:each |
		(each isNamed: aName)
			ifTrue: [aBlock value: each]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/18/2014 04:38'!
allElementsNamed: aName namespaceURI: aUri
	^ self allElementsSelect: [:each |
		(each isNamed: aName)
			and: [each namespaceURI = aUri]]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 10/14/2014 22:49'!
allElementsSelect: aBlock
	| selectedElements |

	selectedElements := self newNodeListForSelect.
	self allElementsDo: [:each |
		(aBlock value: each)
			ifTrue: [selectedElements addLast: each]].
	^ selectedElements.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:49'!
configuration
	^ configuration ifNil: [
		self configuration: XMLDOMConfiguration new.
		configuration]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/14/2014 07:43'!
configuration: aConfiguration
	configuration := aConfiguration.

	self hasChildren
		ifTrue: [self elementsDo: [:each | each configuration: aConfiguration]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:09'!
contentNodes
	^ self nodes select: [:each | each isContentNode]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 7/22/2010 08:09'!
contentNodesDo: aBlock
	self nodes do: [:each | each isContentNode ifTrue: [aBlock value: each]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 16:14'!
contentStringAt: aName
	^ (self elementAt: aName ifAbsent: [^ '']) contentString! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 7/12/2010 03:19'!
document
	self subclassResponsibility! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 09:26'!
document: aDocument 
	self subclassResponsibility! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 2/14/2014 05:31'!
elementAfter: anElement
	^ self nodes elementAfter: anElement! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:31'!
elementAt: aName
	^ self nodes elementAt: aName! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName ifAbsent: aBlock
	^ self nodes elementAt: aName ifAbsent: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName namespaceURI: aUri
	^ self nodes elementAt: aName namespaceURI: aUri! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName namespaceURI: aUri ifAbsent: aBlock
	^ self nodes elementAt: aName namespaceURI: aUri ifAbsent: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 2/14/2014 05:31'!
elementBefore: anElement
	^ self nodes elementBefore: anElement! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 5/4/2010 13:18'!
elements
	^ self nodes elements! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementsAt: aName
	^ self nodes elementsAt: aName! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:32'!
elementsAt: aName do: aBlock
	(self nodes elementsAt: aName) do: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:33'!
elementsAt: aName namespaceURI: aUri
	^ self nodes elementsAt: aName namespaceURI: aUri! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsCollect: aBlock
	^ self nodes elementsCollect: aBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsDetect: aBlock
	^ self nodes elementsDetect: aBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsDetect: aBlock ifNone: anExceptionBlock
	^ self nodes elementsDetect: aBlock ifNone: anExceptionBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 6/26/2010 00:55'!
elementsDo: aBlock
	self nodes elementsDo: aBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:36'!
elementsSelect: aBlock
	^ self nodes elementsSelect: aBlock! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 11/25/2010 17:24'!
findElementNamed: aName
	^ self allElementsDetect: [:each | each isNamed: aName]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'monty 10/18/2014 04:39'!
findElementNamed: aName namespaceURI: aUri
	^ self allElementsDetect: [:each |
		(each isNamed: aName)
			and: [each namespaceURI = aUri]]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'monty 10/18/2014 04:39'!
findElementNamed: aName namespaceURI: aUri with: aBlock
	^ self allElementsDetect: [:each |
		(each isNamed: aName)
			and: [each namespaceURI = aUri
				and: [aBlock value: each]]]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 11/25/2010 13:48'!
findElementNamed: aName with: aBlock
	^ self allElementsDetect: [:each | (each isNamed: aName) and: [aBlock value: each]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 2/10/2014 23:09'!
firstElement
	^ self nodes firstElement! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 10/19/2010 11:13'!
hasContentNodes
	^ self nodes anySatisfy: [:each | each isContentNode]! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 11/24/2010 09:27'!
hasElements
	"Faster version used when traversing to avoid initializing the node list
	in leaf elements"
	^ nodes notNil and: [nodes hasElements]! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 12/6/2010 21:21'!
includesElement: aName
	^ self nodes includesElementAt: aName! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'monty 12/27/2013 03:37'!
innerXMLParserOn: aStringOrStream
	^ self configuration parserHandlerClass
		on: aStringOrStream
		withRootNode: self! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:15'!
isContentNode
	^ true! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 2/10/2014 23:09'!
lastElement
	^ self nodes lastElement! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 8/7/2010 20:11'!
namespaceScope
	^ XMLNamespaceScope new! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'monty 11/26/2013 22:23'!
nodeListClass
	^ self configuration nodeListClass! !

!XMLNodeWithElements methodsFor: 'printing' stamp: 'JAAyer 5/5/2010 15:38'!
printContentOn: aStream
	self contentNodesDo: [:each | each printContentOn: aStream]! !

!XMLNodeWithElements methodsFor: 'notifications' stamp: 'monty 2/2/2014 04:17'!
removedElement: anElement
	self removedNode: anElement.
	anElement document: nil.! !

!XMLNodeWithElements methodsFor: 'notifications' stamp: 'monty 10/18/2014 01:08'!
renamedElement: anElement from: anOldName to: aNewName
	self nodes
		renamedElement: anElement
		from: anOldName
		to: aNewName! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 11/23/2010 14:48'!
usesNamespaces
	"Uses direct access and isNil test for performance"
	^ configuration isNil or: [configuration usesNamespaces]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 12:59'!
usesNamespaces: aBoolean
	self configuration usesNamespaces: aBoolean! !

XMLNode subclass: #XMLPI
	instanceVariableNames: 'target data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLPI commentStamp: 'JAAyer 6/25/2010 14:06' prior: 0!
This class represents a processing instruction.
!

!XMLPI class methodsFor: 'instance creation' stamp: 'JAAyer 2/25/2010 01:48'!
target: targetName data: aString
	^ self new
		target: targetName;
		data: aString! !

!XMLPI methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 03:02'!
data
	^ data ifNil: [data := '']! !

!XMLPI methodsFor: 'accessing' stamp: 'mir 1/17/2002 13:02'!
data: aString
	data := aString! !

!XMLPI methodsFor: 'testing' stamp: 'JAAyer 4/13/2010 21:43'!
isPI
	^ true! !

!XMLPI methodsFor: 'accessing' stamp: 'JAAyer 7/21/2010 12:42'!
target
	^ target ifNil: [target := '']! !

!XMLPI methodsFor: 'accessing' stamp: 'mir 1/17/2002 13:02'!
target: aString
	target := aString! !

!XMLPI methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter pi
		target: self target;
		data: self data! !

XMLNode subclass: #XMLString
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLString commentStamp: '<historical>' prior: 0!
This class represents string nodes, or nodes that contain no markup. String nodes can be created directly using the class-side #string: message or by sending a String object #asXMLString. When printing, they escape special characters in their contents.!

XMLString subclass: #XMLCData
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLCData commentStamp: '<historical>' prior: 0!
This class represents a preserved CDATA section from a parsed document. It behaves like its superclass, except that its printed form differs in that it contains a "<!![CDATA[" prefix and a "]]>" suffix and its contents are NOT escaped. By default, CDATA sections are converted to regular string nodes during parsing. To preserve them as CDATA nodes, use #preservesCDataNodes: with the DOM parser before parsing.!

!XMLCData methodsFor: 'private' stamp: 'monty 2/11/2014 00:14'!
acceptsContent
	^ false! !

!XMLCData methodsFor: 'converting' stamp: 'monty 2/11/2014 00:12'!
asXMLString
	^ XMLString string: self string! !

!XMLCData methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:16'!
isCData
	^ true! !

!XMLCData methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter cdata: self string! !

!XMLString class methodsFor: 'instance creation' stamp: 'JAAyer 2/25/2010 01:48'!
string: aString
	^ self new string: aString! !

!XMLString methodsFor: 'private' stamp: 'monty 2/11/2014 00:14'!
acceptsContent
	^ true! !

!XMLString methodsFor: 'accessing' stamp: 'monty 1/1/2014 07:23'!
addContent: aStringOrStringNode
	string :=
		string
			ifNil: [aStringOrStringNode asString]
			ifNotNil: [string, aStringOrStringNode asString]! !

!XMLString methodsFor: 'converting' stamp: 'monty 1/1/2014 07:14'!
asString
	^ self string! !

!XMLString methodsFor: 'converting' stamp: 'JAAyer 7/22/2010 05:19'!
asXMLString
	^ self! !

!XMLString methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:26'!
isContentNode
	^ true! !

!XMLString methodsFor: 'testing' stamp: 'JAAyer 4/12/2010 22:35'!
isStringNode
	^ true! !

!XMLString methodsFor: 'printing' stamp: 'JAAyer 9/2/2010 13:25'!
printContentOn: aStream
	| lastChar |

	self string ifEmpty: [^ self].

	((lastChar := aStream peekLast) isNil
		or: [lastChar isSeparator
			or: [self string first isSeparator]])
		ifFalse: [aStream space].

	aStream nextPutAll: self string.! !

!XMLString methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 05:19'!
string
	^ string ifNil: [string := '']! !

!XMLString methodsFor: 'accessing' stamp: 'monty 1/1/2014 07:23'!
string: aString
	string := aString asString! !

!XMLString methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter string: self string! !

Object subclass: #XMLNodeFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeFactory commentStamp: '<historical>' prior: 0!
This class is the default node factory. Its accessor messages return the class objects used by the DOM parser to build a node tree. You can override these messages to return different class objects and inject an instance of your factory using the DOM parser's #nodeFactory: accessor before parsing.!

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:58'!
attributeDeclarationClass
	^ XMLAttributeDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
cdataClass
	^ XMLCData! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:20'!
classForElement: aQualifiedName prefix: aPrefix uri: aNamespaceUri localName: aLocalName
	^ self elementClass! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
commentClass
	^ XMLComment! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:19'!
doctypeDeclarationClass
	^ XMLDoctypeDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:19'!
documentClass
	^ XMLDocument! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
elementClass
	^ XMLElement! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
elementDeclarationClass
	^ XMLElementDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
generalEntityDeclarationClass
	^ XMLGeneralEntityDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
notationDeclarationClass
	^ XMLNotationDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
parameterEntityDeclarationClass
	^ XMLParameterEntityDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
piClass
	^ XMLPI! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
stringClass
	^ XMLString! !

XMLNodeFactory subclass: #XMLPluggableElementFactory
	instanceVariableNames: 'elementClass elementClassesByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLPluggableElementFactory commentStamp: '<historical>' prior: 0!
This class is a pluggable factory that can map elements to different XMLElement subclasses based on the name and namespace information of those elements. You have to create an instance, configure it to handle certain elements with certain classes, and then inject your instance into a DOM parser using #nodeFactory: before parsing. Here is an example of its use:

	doc := (XMLDOMParser on: someXML)
		nodeFactory:
			(XMLPluggableElementFactory new
				elementClass: GenericElement;
				handleElement: 'user' withClass: UserElement;
				handleElement: 'report' withClass: ReportElement;
				handleElement: 'report' namespaceURI: 'urn:specialreprot' withClass: SpecialReportElement)
		parseDocument.
		
The #handleElementsMatchingClass*: forms try to match elements to the specified classes based on the names of those classes when stripped of any prefix (like XML) and "Element" suffix and converted to both camel case and a hyphenated form. So this:
	XMLPluggableElementFactory new
		handleElementsMatchingClass: MYRootUserElement
		....
will handle 'rootUser' and 'root-user' elements with the MYRootUserElement class.!

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'JAAyer 11/14/2010 21:57'!
classForElement: aName prefix: aPrefix uri: aUri localName: aLocalName
	^ (self classForElement: aName uri: aUri)
		ifNil: [
			aPrefix
				ifEmpty: [self elementClass]
				ifNotEmpty: [
					(self classForElement: aLocalName uri: aUri)
						ifNil: [self elementClass]]]! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/14/2010 21:55'!
classForElement: aName uri: aUri
	| classesForElement |

	classesForElement :=
		self elementClassesByName
			at: aName
			ifAbsent: [^ nil].

	^ classesForElement
		at: aUri
		ifAbsent: [
			classesForElement
				at: ''
				ifAbsent: [nil]].! !

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'JAAyer 11/15/2010 05:05'!
elementClass
	^ elementClass ifNil: [elementClass := super elementClass]! !

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'JAAyer 11/15/2010 05:04'!
elementClass: anElementClass
	elementClass := anElementClass! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/13/2010 08:36'!
elementClassesByName
	^ elementClassesByName ifNil: [elementClassesByName := Dictionary new]! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/13/2010 19:37'!
handleElement: anElementName namespaceURI: aUri withClass: aClass
	(self elementClassesByName
		at: anElementName
		ifAbsentPut: [Dictionary new])
			at: aUri put: aClass! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/13/2010 19:37'!
handleElement: anElementName withClass: aClass
	self
		handleElement: anElementName
		namespaceURI: ''
		withClass: aClass! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/20/2010 11:39'!
handleElements: anElementNameCollection namespaceURI: aNamespaceURI withClass: aClass
	anElementNameCollection do: [:each |
		self
			handleElement: each
			namespaceURI: aNamespaceURI
			withClass: aClass]! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/20/2010 11:36'!
handleElements: anElementNameCollection withClass: aClass
	anElementNameCollection do: [:each | self handleElement: each withClass: aClass]! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/22/2010 11:10'!
handleElementsMatchingClass: aClass
	| className |

	(className := self toNameWithoutPrefixAndElementSuffix: aClass name asString)
		ifEmpty: [^ self].

	self
		handleElement: (self toCamelCasedName: className)
		withClass: aClass.
	self
		handleElement: (self toHyphenatedName: className)
		withClass: aClass.! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/20/2010 11:26'!
handleElementsMatchingClasses: aClassCollection
	aClassCollection do: [:each | self handleElementsMatchingClass: each]! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/20/2010 11:29'!
toCamelCasedName: aString
	^ aString allButFirst copyWithFirst: aString first asLowercase! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/20/2010 11:34'!
toHyphenatedName: aString
	| nameStream hyphenatedStream previousChar nextChar |

	nameStream := aString readStream.
	hyphenatedStream := (String new: nameStream size) writeStream.
	[nameStream atEnd]
		whileFalse: [
			((nextChar := nameStream next) isUppercase
				and: [previousChar notNil and: [previousChar isLowercase]])
				ifTrue: [hyphenatedStream nextPut: $-].
			hyphenatedStream nextPut: nextChar asLowercase.
			previousChar := nextChar].
	^ hyphenatedStream contents.! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/22/2010 11:10'!
toNameWithoutPrefixAndElementSuffix: aString
	| nameStream |

	nameStream := aString readStream.
	[nameStream atEnd not and: [nameStream peek isUppercase]]
		whileTrue: [nameStream next].
	nameStream atEnd
		ifTrue: [^ ''].
	nameStream back.

	^ nameStream upToEnd copyReplaceAll: 'Element' with: ''.! !

Object subclass: #XMLNodeName
	instanceVariableNames: 'qualified localPart'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeName commentStamp: 'monty 10/14/2014 07:17' prior: 0!
The class represents a name for an element or attribute. It stores the local part and qualified name. You should not need to use this class directly.!

!XMLNodeName class methodsFor: 'instance creation' stamp: 'monty 10/1/2014 15:20'!
qualified: aQualifiedName
	^ self new setQualified: aQualifiedName! !

!XMLNodeName class methodsFor: 'instance creation' stamp: 'monty 10/3/2014 04:12'!
qualified: aQualifiedName localPart: aLocalPart
	^ self new
		setQualified: aQualifiedName
		localPart: aLocalPart! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 15:01'!
expandWith: aUri
	^ aUri
		ifEmpty: [self localPart]
		ifNotEmpty: ['{', aUri, '}', self localPart]! !

!XMLNodeName methodsFor: 'testing' stamp: 'monty 10/3/2014 04:19'!
isPrefixed
	^ self qualified size > self localPart size! !

!XMLNodeName methodsFor: 'testing' stamp: 'monty 10/8/2014 00:08'!
isSameAs: aNameString
	^ aNameString
		isQualifiedName: self qualified
		orLocalName: self localPart! !

!XMLNodeName methodsFor: 'accessing' stamp: 'monty 10/3/2014 20:27'!
localPart
	^ localPart ifNil: [localPart := self qualified qualifiedNameLocalPart]! !

!XMLNodeName methodsFor: 'accessing' stamp: 'monty 10/4/2014 00:08'!
prefix
	| prefixSize |

	"faster than qualifiedNamePrefix if we already have the localPart"
	(prefixSize := self qualified size - self localPart size - 1) > 0
		ifTrue: [^ qualified copyFrom: 1 to: prefixSize]
		ifFalse: [^ '']! !

!XMLNodeName methodsFor: 'accessing' stamp: 'monty 10/14/2014 06:58'!
prefix: aPrefix
	qualified := 
		aPrefix
			ifEmpty: [self localPart]
			ifNotEmpty: [aPrefix, ':', self localPart]! !

!XMLNodeName methodsFor: 'printing' stamp: 'JAAyer 6/27/2010 15:15'!
printOn: aStream
	aStream nextPutAll: self qualified! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:23'!
qualified
	^ qualified ifNil: [qualified := '']! !

!XMLNodeName methodsFor: 'accessing' stamp: 'monty 10/14/2014 07:17'!
qualified: aQualifiedName
	qualified := aQualifiedName.
	localPart := aQualifiedName qualifiedNameLocalPart.! !

!XMLNodeName methodsFor: 'initialization' stamp: 'monty 1/28/2014 19:42'!
setQualified: aQualifiedName
	qualified := aQualifiedName! !

!XMLNodeName methodsFor: 'initialization' stamp: 'monty 10/3/2014 04:12'!
setQualified: aQualifiedName localPart: aLocalPart
	qualified := aQualifiedName.
	localPart := aLocalPart.! !

Object subclass: #XMLParser
	instanceVariableNames: 'tokenizer driver'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLParser commentStamp: '<historical>' prior: 0!
This class is a facade that parses XML using a driver and tokenizer. See SAXHandler and XMLDOMParser for ready-made front ends to it.!

!XMLParser class methodsFor: 'instance creation' stamp: 'JAAyer 11/28/2010 07:09'!
driver: aDriver on: aStringOrStream
	^ self new setDriver: aDriver tokenizer: (aDriver tokenizerOn: aStringOrStream)! !

!XMLParser methodsFor: 'testing' stamp: 'JAAyer 11/27/2010 07:17'!
atEnd
	^ self tokenizer atEnd! !

!XMLParser methodsFor: 'accessing' stamp: 'monty 11/14/2013 23:34'!
currentLineNumber
	^ self tokenizer currentLineNumber! !

!XMLParser methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 07:07'!
driver
	^ driver! !

!XMLParser methodsFor: 'parsing' stamp: 'monty 2/18/2014 02:54'!
parseDocument
	[[self tokenizer atEnd]
		whileFalse: [self tokenizer nextToken]]
		on: XMLParserException
		do: [:error | self passParserException: error]! !

!XMLParser methodsFor: 'parsing' stamp: 'monty 2/18/2014 02:54'!
parseDocumentWhile: aBlock
	[[self tokenizer atEnd or: [aBlock value not]]
		whileFalse: [self tokenizer nextToken]]
		on: XMLParserException
		do: [:error | self passParserException: error]! !

!XMLParser methodsFor: 'parsing' stamp: 'monty 2/18/2014 02:54'!
parseToken
	^ [self tokenizer nextToken]
		on: XMLParserException
		do: [:error | self passParserException: error]! !

!XMLParser methodsFor: 'private' stamp: 'monty 2/18/2014 02:54'!
passParserException: anError
	self tokenizer streamReader close.
	anError
		appendLineNumber: self currentLineNumber;
		pass.! !

!XMLParser methodsFor: 'initialization' stamp: 'JAAyer 11/28/2010 07:09'!
setDriver: aDriver tokenizer: aTokenizer
	driver := aDriver.
	tokenizer := aTokenizer.! !

!XMLParser methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 07:16'!
tokenizer
	^ tokenizer! !

Object subclass: #XMLParserDriver
	instanceVariableNames: 'uris'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLParserDriver commentStamp: '<historical>' prior: 0!
This is a base class for parser drivers. Drivers are sent messages by a tokenizer to handle various XML tokens as they are read. To create a driver, subclass this class and override the handler messages and testing messages that control tokenization.!

XMLParserDriver subclass: #DTDExternalSubsetParserDriver
	instanceVariableNames: 'saxDriver dtd documentUri'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!DTDExternalSubsetParserDriver commentStamp: '<historical>' prior: 0!
This class is a driver for parsing the external subset of a DTD.!

!DTDExternalSubsetParserDriver class methodsFor: 'instance creation' stamp: 'monty 11/19/2013 06:10'!
saxDriver: aSaxDriver documentURI: anExternalSubsetUri
	^ self new
		setSAXDriver: aSaxDriver
		documentURI: anExternalSubsetUri ! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 11/19/2013 06:11'!
documentURI
	^ documentUri! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 2/3/2014 17:58'!
dtd
	^ dtd! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 11/19/2013 06:04'!
externalEntityResolver
	^ self saxDriver externalEntityResolver! !

!DTDExternalSubsetParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/30/2013 23:55'!
handleAttributeDeclaration: anElement name: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	self isValidating
		ifTrue: [
			self dtd
				addValidatorForAttributeIn: anElement
				named: aName
				type: aType
				defaultPragma: aDefaultPragma
				defaultValue: aDefaultValue]! !

!DTDExternalSubsetParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/10/2013 06:32'!
handleElementDeclaration: aName contentModel: aContentModel
	self isValidating
		ifTrue: [
			self dtd
				addValidatorForElement: aName
				contentModel: aContentModel]! !

!DTDExternalSubsetParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/14/2013 02:15'!
handleGeneralEntityDeclaration: aName publicID: aPublicId systemID: aSystemId ndata: aNotation
	aNotation ifNotEmpty: [
		self dtd
			addUnparsedEntity: aName
			ndata: aNotation.
		^ self].

	self dtd
		addExternalGeneralEntity: aName
		resolver: self externalEntityResolver
		uri: aSystemId
		from: self currentURI.! !

!DTDExternalSubsetParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/13/2013 18:52'!
handleGeneralEntityDeclaration: aName replacement: aReplacement
	self dtd
		addGeneralEntity: aName
		replacement: aReplacement
		from: self currentURI! !

!DTDExternalSubsetParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/19/2013 02:20'!
handleNotationDeclaration: aName publicID: aPublicId systemID: aSystemId
	self dtd addNotation: aName! !

!DTDExternalSubsetParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/13/2013 18:55'!
handleParameterEntityDeclaration: aName publicID: aPublicId systemID: aSystemId
	self dtd
		addExternalParameterEntity: aName
		resolver: self externalEntityResolver
		uri: aSystemId
		from: self currentURI! !

!DTDExternalSubsetParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/13/2013 18:53'!
handleParameterEntityDeclaration: aName replacement: aReplacement
	self dtd
		addParameterEntity: aName
		replacement:aReplacement
		from: self currentURI! !

!DTDExternalSubsetParserDriver methodsFor: 'handling tokens' stamp: 'monty 1/24/2014 22:32'!
handleParameterEntityReference: aName
	| entity |

	entity := self dtd parameterEntityAt: aName.
	(entity isNil and: [self isValidating])
		ifTrue: [(XMLValidationException undeclaredParameterEntity: aName) signal].

	^ entity.! !

!DTDExternalSubsetParserDriver methodsFor: 'testing' stamp: 'monty 2/7/2014 23:33'!
isParsingExternalSubset
	^ true! !

!DTDExternalSubsetParserDriver methodsFor: 'testing' stamp: 'monty 2/3/2014 17:58'!
isValidating
	^ self saxDriver isValidating! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 1/16/2014 19:10'!
maxEntityReplacementDepth
	^ self saxDriver maxEntityReplacementDepth! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 1/16/2014 23:07'!
readLimit
	^ self saxDriver externalSubsetReadLimit! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 11/19/2013 06:04'!
saxDriver
	^ saxDriver! !

!DTDExternalSubsetParserDriver methodsFor: 'initialization' stamp: 'monty 2/3/2014 17:58'!
setSAXDriver: aSaxdriver documentURI: aUri
	saxDriver := aSaxdriver.
	dtd := saxDriver dtd.
	documentUri := aUri.! !

!DTDExternalSubsetParserDriver methodsFor: 'testing' stamp: 'monty 11/30/2013 21:55'!
usesNamespaces
	^ self saxDriver usesNamespaces! !

XMLParserDriver subclass: #SAXParserDriver
	instanceVariableNames: 'saxHandler elementNester scopeAllocator dtd validator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAXParserDriver commentStamp: 'monty 2/9/2014 02:43' prior: 0!
SAXParserDriver together with SAXHandler provides a SAX-like API for XML parsing. The driver processes XML tokens and signals SAX events to a SAXHandler. To use it, create a subclass of SAXHandler and override messages to handle SAX events of interest. See "instance creation" on the class-side of SAXHandler for ways to instantiate such a parser.!

!SAXParserDriver class methodsFor: 'instance creation' stamp: 'monty 2/3/2014 17:57'!
saxHandler: aHandler
	^ self new saxHandler: aHandler! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 12/27/2013 04:13'!
currentElement
	^ self elementNester currentElement! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 20:36'!
currentScope
	^ self usesNamespaces
		ifTrue: [self scopeAllocator currentScope]
		ifFalse: [nil]! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 12/27/2013 03:40'!
currentScope: aNamespaceScope
	self usesNamespaces
		ifTrue: [self scopeAllocator currentScope: aNamespaceScope]! !

!SAXParserDriver methodsFor: 'namespaces' stamp: 'monty 2/8/2014 03:11'!
declareNamespaces: aDictionary inScope: aScope
	aDictionary keysAndValuesDo: [:prefix :uri |
		aScope mapPrefix: prefix to: uri.
		self saxHandler startPrefixMapping: prefix uri: uri]! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 1/19/2014 04:44'!
documentReadLimit
	^ self saxHandler documentReadLimit! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 11/16/2013 04:35'!
documentURI
	^ self saxHandler documentURI! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 2/3/2014 19:30'!
dtd
	^ dtd ifNil: [dtd := DTDDoctypeDefinition new]! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 10/5/2014 15:29'!
elementNester
	^ elementNester ifNil: [elementNester := self elementNesterClass new]! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 12/27/2013 04:08'!
elementNesterClass
	^ XMLWellFormedElementNester! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 2/11/2014 01:53'!
endNestingOfElement: anElementName withScopedAttributes: anAttributeDictionary
	| base |

	base :=
		anAttributeDictionary
			at: 'xml:base'
			ifAbsent: [
				self elementNester endElement: anElementName.
				^ self].
	(self elementNester
		endElement: anElementName;
		scopedAttributeAt: 'xml:base') = base
			ifFalse: [self removeURI: base].! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 2/3/2014 17:42'!
externalEntityResolver
	^ self saxHandler externalEntityResolver! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 10/3/2014 00:19'!
externalSubsetParserOnURI: aUri
	^ self saxHandler parserClass
		driver:
			(DTDExternalSubsetParserDriver
				saxDriver: self
				documentURI: aUri asString)
		on: (self externalEntityResolver resolveEntityURI: aUri)
	! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 1/19/2014 04:45'!
externalSubsetReadLimit
	^ self saxHandler externalSubsetReadLimit! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/30/2013 23:56'!
handleAttributeDeclaration: anElement name: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	self isValidating
		ifTrue: [
			self dtd
				addValidatorForAttributeIn: anElement
				named: aName
				type: aType
				defaultPragma: aDefaultPragma
				defaultValue: aDefaultValue].
	self saxHandler
		attributeDeclaration: anElement
		name: aName
		type: aType
		defaultPragma: aDefaultPragma
		defaultValue: aDefaultValue! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 10/4/2014 07:00'!
handleCData: aString
	self saxHandler
		isInCDataSection: true;
		startCData;
		characters: aString;
		endCData;
		isInCDataSection: false! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/11/2013 22:26'!
handleComment: aString
	self saxHandler comment: aString! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/10/2013 06:32'!
handleElementDeclaration: aName contentModel: aContentModel
	self isValidating
		ifTrue: [
			self dtd
				addValidatorForElement: aName
				contentModel: aContentModel].

	self saxHandler
		elementDeclaration: aName
		contentModel: aContentModel! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/16/2013 05:10'!
handleEndContentEntityReplacement: aName
	self saxHandler endEntity: aName! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 10/6/2014 21:47'!
handleEndDTD
	(self resolvesExternalEntities
		and: [self dtd hasExternalSubset])
		ifTrue: [self parseExternalSubset: self dtd externalSubsetURI].

	self saxHandler
		endDTD;
		isInDTD: false.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 2/2/2014 02:44'!
handleEndDocument
	self elementNester endNesting.
	self isValidating
		ifTrue: [self validator validateEndDocument].

	self saxHandler endDocument.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 10/3/2014 20:26'!
handleEndTag: anElementName
	"performance critical method"
	| prefix currentScope |

	self elementNester hasScopedAttributes
		ifTrue: [
			self
				endNestingOfElement: anElementName
				withScopedAttributes: self elementNester currentScopedAttributes]
		ifFalse: [self elementNester endElement: anElementName].

	prefix := anElementName qualifiedNamePrefix.
	self saxHandler 
		endElement: anElementName
		prefix: prefix
		uri:
			(self usesNamespaces
				ifTrue: [
					currentScope := self scopeAllocator exitScope.
					currentScope resolvePrefix: prefix]
				ifFalse: [''])
		localName: (anElementName qualifiedNameLocalPartFromPrefix: prefix).

	currentScope ifNotNil: [
		currentScope hasNamespaces
			ifTrue: [self undeclareNamespacesInScope: currentScope]].
	self isValidating
		ifTrue: [self validator validateEndTag: anElementName].! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 2/9/2014 02:28'!
handleGeneralEntityDeclaration: aName publicID: aPublicId systemID: aSystemId ndata: aNotation
	aNotation
		ifNotEmpty: [
			self dtd
				addUnparsedEntity: aName
				ndata: aNotation]
		ifEmpty: [
			self resolvesExternalEntities
				ifTrue: [
					self dtd
						addExternalGeneralEntity: aName
						resolver: self externalEntityResolver
						uri: aSystemId
						from: self currentURI]].

	self saxHandler
		generalEntityDeclaration: aName
		publicID: aPublicId
		systemID: aSystemId
		ndata: aNotation.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/13/2013 18:49'!
handleGeneralEntityDeclaration: aName replacement: aReplacement
	self dtd
		addGeneralEntity: aName
		replacement: aReplacement
		from: self currentURI.

	self saxHandler
		generalEntityDeclaration: aName
		replacement: aReplacement.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 2/11/2014 01:23'!
handleGeneralEntityReference: aName
	| entity |

	(entity := self dtd generalEntityAt: aName)
		ifNil: [
			self isValidating
				ifTrue: [(XMLValidationException undeclaredGeneralEntity: aName) signal].
			self preservesUndeclaredEntityReferences
				ifTrue: [entity := DTDBypassedGeneralEntity named: aName].
			self saxHandler skippedEntity: aName].
	^ entity.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/19/2013 02:19'!
handleNotationDeclaration: aName publicID: aPublicId systemID: aSystemId
	self isValidating
		ifTrue: [self dtd addNotation: aName].

	self saxHandler
		notationDeclaration: aName
		publicID: aPublicId
		systemID: aSystemId.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 2/3/2014 17:42'!
handlePCData: aString
	self isValidating
		ifTrue: [self validator validatePCData: aString].

	self saxHandler characters: aString.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 2/2/2014 04:03'!
handlePI: piTarget data: piData
	self saxHandler 
		processingInstruction: piTarget
		data: piData! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 2/9/2014 02:28'!
handleParameterEntityDeclaration: aName publicID: aPublicId systemID: aSystemId
	self resolvesExternalEntities
		ifTrue: [
			self dtd
				addExternalParameterEntity: aName
				resolver: self externalEntityResolver
				uri: aSystemId
				from: self currentURI].

	self saxHandler
		parameterEntityDeclaration: aName
		publicID: aPublicId
		systemID: aSystemId.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/13/2013 18:54'!
handleParameterEntityDeclaration: aName replacement: aReplacement
	self dtd
		addParameterEntity: aName
		replacement:aReplacement
		from: self currentURI.

	self saxHandler
		parameterEntityDeclaration: aName
		replacement: aReplacement.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 2/11/2014 01:23'!
handleParameterEntityReference: aName
	| entity |

	(entity := self dtd parameterEntityAt: aName)
		ifNil: [
			self isValidating
				ifTrue: [(XMLValidationException undeclaredParameterEntity: aName) signal].
			self preservesUndeclaredEntityReferences
				ifTrue: [entity := DTDBypassedParameterEntity named: aName]].
	^ entity.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 1/26/2014 04:21'!
handleReservedAttributes: anAttributeDictionary
	(anAttributeDictionary at: 'xml:base' ifAbsent: [nil])
		ifNotNil: [:uri | self currentURI: uri].
	self isValidating
		ifTrue: [self validator validateXMLIDAttributeIn: anAttributeDictionary].

	self elementNester
		setScopedAttributesNamed: #('xml:space' 'xml:base' 'xml:lang')
		from: anAttributeDictionary.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/18/2013 22:08'!
handleStartContentEntityReplacement: aName
	self saxHandler startEntity: aName! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 10/5/2014 20:31'!
handleStartDTD: aRoot publicID: aPublicId systemID: aSystemId
	self dtd
		rootElement: aRoot
		externalSubsetURI: aSystemId.
	
	self saxHandler
		isInDTD: true;
		startDTD: aRoot
		publicID: aPublicId
		systemID: aSystemId.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'mir 8/14/2000 18:29'!
handleStartDocument
	self saxHandler startDocument! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 10/9/2014 06:34'!
handleStartTag: anElementName attributes: anAttributeDictionary namespaces: aNamespaceDictionary
	"performance critical method"
	| prefix namespaceUri currentScope |

	self elementNester startElement: anElementName.

	prefix := anElementName qualifiedNamePrefix.
	self usesNamespaces
		ifTrue: [
			currentScope := self scopeAllocator enterScope.
			aNamespaceDictionary
				ifNotNil: [self declareNamespaces: aNamespaceDictionary inScope: currentScope].
			namespaceUri := currentScope validateAndResolvePrefix: prefix ofElement: anElementName.
			anAttributeDictionary isEmpty "avoid ifNotEmpty: closure overhead"
				ifFalse: [currentScope validateAttributePrefixes: anAttributeDictionary]].
	self isValidating
		ifTrue: [
			self elementNester isCurrentElementRoot
				ifTrue: [self validateRootElement: anElementName].
			self validator
				validateStartTag: anElementName
				attributes: anAttributeDictionary].

	self saxHandler
		startElement: anElementName
		prefix: prefix
		uri: (namespaceUri ifNil: [''])
		localName: (anElementName qualifiedNameLocalPartFromPrefix: prefix)
		attributes: anAttributeDictionary.
	currentScope
		ifNotNil: [self saxHandler currentScope: currentScope].! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 1/24/2014 22:32'!
handleUnparsedEntityReference: aName
	| notationName |

	notationName := self dtd unparsedEntityAt: aName.
	self isValidating
		ifTrue: [
			notationName
				ifNil: [(XMLValidationException undeclaredUnparsedEntity: aName) signal].
			(self dtd includesNotation: notationName)
				ifFalse: [XMLValidationException signal: 'Undeclared notation "', notationName, '"']].
	^ notationName.

	! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 1/21/2014 05:22'!
handleWhitespace: aString
	(self elementNester hasScopedAttributes
		and: [(self elementNester scopedAttributeAt: 'xml:space') = 'preserve'])
		ifTrue: [self saxHandler characters: aString]
		ifFalse: [self saxHandler ignorableWhitespace: aString]! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 1/5/2014 00:52'!
handleXMLVersion: aVersion encoding: anEncoding standalone: aYesOrNo
	| versionNumber standaloneBoolean |

	aVersion ifNotEmpty: [versionNumber := aVersion asNumber].
	standaloneBoolean := aYesOrNo = 'yes'.

	self dtd isStandalone: standaloneBoolean.

	self saxHandler
		xmlVersion: versionNumber
		encoding: anEncoding
		standalone: standaloneBoolean! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 2/3/2014 17:43'!
isValidating
	^ self saxHandler isValidating! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 1/19/2014 04:45'!
maxEntityReplacementDepth
	^ self saxHandler maxEntityReplacementDepth! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 1/10/2014 02:29'!
parseExternalSubset: aUri
	| parser |

	parser :=
		self externalSubsetParserOnURI:
			(self externalEntityResolver
				combineURI: self documentURI
				with: aUri).
	parser parseDocument.! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 2/11/2014 01:23'!
preservesUndeclaredEntityReferences
	^ self saxHandler preservesUndeclaredEntityReferences! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 1/19/2014 04:45'!
readLimit
	^ self documentReadLimit! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 2/9/2014 02:28'!
resolvesExternalEntities
	^ self saxHandler resolvesExternalEntities! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 12/27/2013 04:14'!
rootElement
	^ self elementNester rootElement! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 12/27/2013 04:11'!
rootElement: anElementName
	self elementNester rootElement: anElementName! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 08:00'!
saxHandler
	^ saxHandler! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 2/3/2014 17:57'!
saxHandler: aHandler
	saxHandler := aHandler! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 12/27/2013 03:40'!
scopeAllocator
	^ scopeAllocator ifNil: [scopeAllocator := XMLNamespaceScopeAllocator new]! !

!SAXParserDriver methodsFor: 'namespaces' stamp: 'monty 10/10/2014 05:53'!
undeclareNamespacesInScope: aScope
	| currentScope |

	currentScope := self scopeAllocator currentScope.
	aScope prefixesAndURIsDo: [:prefix :uri |
		(currentScope includesPrefix: prefix mappedTo: uri)
			ifFalse: [self saxHandler endPrefixMapping: prefix]].! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 2/3/2014 17:43'!
usesNamespaces
	^ self saxHandler usesNamespaces! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 2/7/2014 23:38'!
validateRootElement: aStartTag
	(self saxHandler requiresDocumentConstraints
		and: [self validator hasElementValidators not])
		ifTrue: [XMLValidationException signal: 'Document validation constraints required'].

	self validator validateRootElement: aStartTag.! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 11/19/2013 01:10'!
validator
	^ validator ifNil: [validator := DTDDocumentValidator dtd: self dtd]! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 1/20/2014 21:45'!
currentURI
	^ self uris
		ifEmpty: [self documentURI]
		ifNotEmpty: [self uris last]! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 1/20/2014 02:25'!
currentURI: aUri
	self uris addLast: aUri! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 11/16/2013 04:35'!
documentURI
	^ nil! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 11/30/2013 04:17'!
dtd
	^ nil! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/30/2013 23:46'!
handleAttributeDeclaration: anElement name: anName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleCData: aString! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleComment: aString! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 03:46'!
handleElementDeclaration: aName contentModel: aContentModel! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/16/2013 05:10'!
handleEndContentEntityReplacement: aName
! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/12/2013 17:36'!
handleEndDTD! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleEndDocument! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleEndTag: anElementName! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 03:08'!
handleGeneralEntityDeclaration: aName publicID: aPublicId systemID: aSystemId ndata: aNotation! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 03:10'!
handleGeneralEntityDeclaration: aName replacement: aReplacement! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 20:44'!
handleGeneralEntityReference: aName
	^ nil! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 03:56'!
handleNotationDeclaration: aName publicID: aPublicId systemID: aSystemId! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handlePCData: aString! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handlePI: piTarget data: piData! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 03:09'!
handleParameterEntityDeclaration: aName publicID: aPublicId systemID: aSystemId! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 03:10'!
handleParameterEntityDeclaration: aName replacement: aReplacement! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 20:44'!
handleParameterEntityReference: aName
	^ nil! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 1/4/2014 07:42'!
handleParseError: anErrorString
	XMLWellFormednessException signal: anErrorString! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 2/18/2014 05:13'!
handleReservedAttributes: anAttributeDictionary! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/16/2013 05:10'!
handleStartContentEntityReplacement: aName! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/12/2013 17:15'!
handleStartDTD: aRoot publicID: aPublicId systemID: aSystemId! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleStartDocument! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleStartTag: anElementName attributes: anAttributeDictionary namespaces: aNamespaceDictionary! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/16/2013 03:47'!
handleUnparsedEntityReference: aName
	^ nil! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleWhitespace: aString! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleXMLVersion: aVersion encoding: anEncoding standalone: aYesOrNo! !

!XMLParserDriver methodsFor: 'testing' stamp: 'monty 2/8/2014 02:03'!
isParsingExternalSubset
	^ false! !

!XMLParserDriver methodsFor: 'testing' stamp: 'monty 2/9/2014 02:41'!
isValidating
	^ false! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 1/16/2014 19:45'!
readLimit
	^ nil! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 2/2/2014 07:33'!
removeURI: aUri
	"remove: removes the first match"
	self uris removeAt:
		(self uris
			lastIndexOf: aUri
			ifAbsent: [^ self])! !

!XMLParserDriver methodsFor: 'private' stamp: 'monty 2/10/2014 02:43'!
tokenizerClass
	^ XMLWellFormedParserTokenizer! !

!XMLParserDriver methodsFor: 'initialization' stamp: 'monty 1/17/2014 01:28'!
tokenizerOn: aStringOrStream
	^ self tokenizerClass
		driver: self
		on: aStringOrStream
		readLimit: self readLimit! !

!XMLParserDriver methodsFor: 'private' stamp: 'monty 1/20/2014 02:25'!
uris
	^ uris ifNil: [uris := OrderedCollection new]! !

!XMLParserDriver methodsFor: 'testing' stamp: 'monty 2/9/2014 02:41'!
usesNamespaces
	^ true! !

Object subclass: #XMLParserLimits
	instanceVariableNames: 'documentReadLimit externalSubsetReadLimit maxEntityReplacementDepth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLParserLimits commentStamp: 'monty 1/25/2014 04:00' prior: 0!
This class places security limits on parser input. You can set these using the configuration messages in SAXHandler or remove them using removeLimits.!

!XMLParserLimits class methodsFor: 'accessing' stamp: 'monty 10/3/2014 18:56'!
defaultDocumentReadLimit
	^ 10000000! !

!XMLParserLimits class methodsFor: 'accessing' stamp: 'monty 10/3/2014 23:22'!
defaultExternalSubsetReadLimit
	^ (self defaultDocumentReadLimit / 4) asInteger! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 18:43'!
documentReadLimit
	^ documentReadLimit! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 18:44'!
documentReadLimit: aLimit
	documentReadLimit := aLimit! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 18:44'!
externalSubsetReadLimit
	^ externalSubsetReadLimit! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 18:44'!
externalSubsetReadLimit: aLimit
	externalSubsetReadLimit := aLimit! !

!XMLParserLimits methodsFor: 'initialization' stamp: 'monty 10/3/2014 23:22'!
initialize
	super initialize.

	documentReadLimit := self class defaultDocumentReadLimit.
	externalSubsetReadLimit := self class defaultExternalSubsetReadLimit.
	maxEntityReplacementDepth := 3.! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 18:45'!
maxEntityReplacementDepth
	^ maxEntityReplacementDepth! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 18:45'!
maxEntityReplacementDepth: aLimit
	maxEntityReplacementDepth := aLimit! !

!XMLParserLimits methodsFor: 'removing' stamp: 'monty 1/16/2014 18:45'!
removeAll
	documentReadLimit := nil.
	externalSubsetReadLimit := nil.
	maxEntityReplacementDepth := nil.! !

Object subclass: #XMLParserTokenizer
	instanceVariableNames: 'driver context streamReader writeStream nameStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

XMLParserTokenizer class
	instanceVariableNames: 'predefinedEntities'!

!XMLParserTokenizer commentStamp: 'monty 2/9/2014 02:44' prior: 0!
This is a base class for tokenizers. It reads tokens from a stream using a NestedStreamReader. Sending it #nextToken causes a token to be read and one or more handler messages to be dispatched to a driver.

Be careful changing the code in this class or subclass because it is optimized.!

XMLParserTokenizer class
	instanceVariableNames: 'predefinedEntities'!

!XMLParserTokenizer class methodsFor: 'instance creation' stamp: 'monty 1/17/2014 01:27'!
driver: aDriver on: aStringOrStream
	^self
		driver: aDriver
		on: aStringOrStream
		readLimit: nil! !

!XMLParserTokenizer class methodsFor: 'instance creation' stamp: 'monty 1/17/2014 01:26'!
driver: aDriver on: aStringOrStream readLimit: aReadLimit
	^self new
		setDriver: aDriver
		stream:
			(aStringOrStream isStream
				ifTrue: [aStringOrStream]
				ifFalse: [aStringOrStream readStream])
		readLimit: aReadLimit! !

!XMLParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 12/19/2013 07:23'!
initialize
	"self initialize"

	super initialize.
	self initializePredefinedEntities.! !

!XMLParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 12/7/2013 00:38'!
initializePredefinedEntities! !

!XMLParserTokenizer class methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:37'!
predefinedEntities
	^ predefinedEntities! !

!XMLParserTokenizer methodsFor: 'testing' stamp: 'monty 1/28/2014 05:14'!
atEnd
	^ context atEnd! !

!XMLParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 2/3/2014 22:40'!
characterFromCodePoint: aCodePoint
	^ (aCodePoint notNil and: [aCodePoint > 0])
		ifTrue: [
			[aCodePoint asCharacter]
				on: Error
				do: [:error | nil]]
		ifFalse: [nil]! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:35'!
currentLineNumber
	^ streamReader currentLineNumber! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:35'!
driver
	^ driver! !

!XMLParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:35'!
expectNext: aCharacter
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:37'!
expectNextAll: aString
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:36'!
expectTerminator: aCharacter
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:38'!
expectUpToAll: aString
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/30/2014 18:25'!
nextAttributeValue
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/30/2014 00:39'!
nextAttributeValueDelimitedBy: aQuote
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/9/2013 01:15'!
nextCDataSection
	self expectNextAll: '[CDATA['.

	driver handleCData: (self expectUpToAll: ']]>').
! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 2/2/2014 23:29'!
nextComment
	| comment nextChar |

	"Skip first -"
	streamReader next.
	self expectNext: $-.

	writeStream reset.
	[(nextChar := streamReader next) == nil
		or: [nextChar == $-
			and: [(streamReader peek) == $-]]]
		whileFalse: [writeStream nextPut: nextChar].
	comment := writeStream contents.

	self
		expectNext: $-;
		expectTerminator: $>.

	driver handleComment: comment.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 2/14/2014 23:24'!
nextContentMarkupToken
	streamReader peek == $?
		ifTrue: [^ self nextPIOrXMLDecl].
	streamReader peek == $!!
		ifTrue: [
			streamReader next.
			^ streamReader peek == $-
				ifTrue: [self nextComment]
				ifFalse: [self nextCDataSection]].

	self nextTag.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 03:15'!
nextContentToken
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 2/3/2014 22:26'!
nextDecodedCharReference
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/12/2013 00:08'!
nextDocTypeDecl
	|  root externalId |

	context enterDoctype.
	self expectNextAll: 'DOCTYPE'.
	streamReader skipSeparators.

	root := self nextName.
	streamReader skipSeparators.
	externalId := self nextExternalIDSystemLiteralRequired: true.
	driver
		handleStartDTD: root
		publicID: externalId key
		systemID: externalId value.

	streamReader skipSeparators.
	streamReader peek == $[
		ifTrue: [self nextInternalSubsetStart]
		ifFalse: [self nextEndDocTypeDecl].! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/7/2013 03:38'!
nextEndDocTypeDecl
	streamReader skipSeparators.
	self expectTerminator: $>.
	context enterProlog.

	driver handleEndDTD.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 2/18/2014 05:27'!
nextEndDocument
	context atEnd
		ifFalse: [
			streamReader close.
			context enterAtEnd.
			driver handleEndDocument]! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/10/2013 07:31'!
nextEndInternalSubset
	self expectTerminator: $].

	context enterProlog.
	self nextEndDocTypeDecl.! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/12/2013 00:36'!
nextExternalIDSystemLiteralRequired: aBoolean
	| publicId systemId |

	(streamReader atEnd not
		and: [streamReader peek == $P
			or: [streamReader peek == $S]])
		ifTrue: [
			streamReader peek == $P
				ifTrue: [
					self
						expectNextAll: 'PUBLIC';
						skipSeparatorsReplacingParameterEntities.
					publicId := self nextPubidLiteral.

					self skipSeparatorsReplacingParameterEntities.
					systemId := self nextSystemLiteralRequired: aBoolean]
				ifFalse: [
					self
						expectNextAll: 'SYSTEM';
						skipSeparatorsReplacingParameterEntities.
					systemId := self nextSystemLiteralRequired: true]].

	^ (publicId ifNil: ['']) -> (systemId ifNil: ['']).! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/11/2013 01:47'!
nextGeneralEntityReference
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/7/2013 03:21'!
nextInternalSubsetStart
	"skip ["
	streamReader next.
	context enterInternalSubset.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 01:14'!
nextName
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 2/3/2014 01:12'!
nextPCDataToken
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 2/14/2014 23:45'!
nextPIOrXMLDecl
	| piTarget piData |

	"Skip ?"
	streamReader next.
	piTarget := self nextPITarget.
	(piTarget = 'xml'
		and: [context isInProlog
			or: [context isInTextDeclaration]])
		ifTrue: [^ self nextXMLDecl]
		ifFalse: [
			context isInExternalSubsetTextDeclaration
				ifTrue: [context enterExternalSubset]].
	streamReader skipSeparators.
	piData := self expectUpToAll: '?>'.

	driver handlePI: piTarget data: piData.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:16'!
nextPITarget
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/11/2013 01:47'!
nextParameterEntityReference
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 03:19'!
nextPrologToken
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/7/2013 03:18'!
nextPubidLiteral
	self subclassResponsiblity! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 2/7/2014 23:42'!
nextStartDocument
	driver isParsingExternalSubset
		ifTrue: [context enterExternalSubsetTextDeclaration]
		ifFalse: [context enterProlog].
	driver handleStartDocument.! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/10/2013 07:21'!
nextSubsetMarkupToken
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/7/2013 03:16'!
nextSubsetToken
	streamReader skipSeparators.
	streamReader atEnd
		ifTrue: [^ self].

	streamReader peek == $%
		ifTrue: [^ self replaceParameterEntityReference].
	(context isInInternalSubset and: [streamReader peek == $]])
		ifTrue: [^ self nextEndInternalSubset].

	self nextSubsetMarkupToken.! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/7/2013 03:22'!
nextSystemLiteralRequired: aBoolean
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 04:30'!
nextTag
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/30/2014 03:19'!
nextToken
	(streamReader atEnd and: [context isInitialized])
		ifTrue: [^ self nextEndDocument].

	context isInContent
		ifTrue: [^ self nextContentToken].
	context isInSubset
		ifTrue: [^ self nextSubsetToken].
	context isInProlog
		ifTrue: [^ self nextPrologToken].
	context notInitialized 
		ifTrue: [^ self nextStartDocument].

	self nextEndDocument.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/28/2014 23:35'!
nextUnparsedEntityReference
	| name |

	name := self nextName.
	self expectNext: $;.

	^ (driver handleUnparsedEntityReference: name) ifNil: [''].! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:33'!
nextXMLAttributeName: aName
	self expectNextAll: aName.
	streamReader skipSeparators.
	self expectNext: $=.
	streamReader skipSeparators.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 08:09'!
nextXMLDecl
	| version encoding standalone isTextDeclaration |

	isTextDeclaration  := context isInTextDeclaration.
	version := self nextXMLVersionAttributeRequired: isTextDeclaration not.
	(encoding := self nextXMLEncodingAttributeRequired: isTextDeclaration)
		ifNotEmpty: [streamReader useConverterForEncoding: encoding].
	isTextDeclaration
		ifFalse: [standalone := self nextXMLStandaloneAttribute].

	streamReader skipSeparators.
	self
		expectNext: $?;
		expectTerminator: $>.

	isTextDeclaration
		ifTrue: [
			context isInExternalSubsetTextDeclaration
				ifTrue: [context enterExternalSubset]]
		ifFalse: [
			driver
				handleXMLVersion: version
				encoding: encoding
				standalone: standalone].! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:47'!
nextXMLEncodingAttributeRequired: aBoolean
	streamReader skipSeparators.
	(aBoolean or: [streamReader peek == $e])
		ifFalse: [^ ''].

	^ self
		nextXMLAttributeName: 'encoding';
		nextXMLEncodingAttributeValue.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:47'!
nextXMLEncodingAttributeValue
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:43'!
nextXMLStandaloneAttribute
	streamReader skipSeparators.
	(streamReader peek == $s)
		ifFalse: [^ ''].

	^ self
		nextXMLAttributeName: 'standalone';
		nextXMLStandaloneAttributeValue.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:44'!
nextXMLStandaloneAttributeValue
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:33'!
nextXMLVersionAttributeRequired: aBoolean
	streamReader skipSeparators.
	(aBoolean or: [streamReader peek == $v])
		ifFalse: [^ ''].

	^ self
		nextXMLAttributeName: 'version';
		nextXMLVersionAttributeValue.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:34'!
nextXMLVersionAttributeValue
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 1/16/2014 09:08'!
normalizedLineEndingChar
	^ streamReader normalizedLineEndingChar! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/18/2013 17:26'!
normalizedLineEndingChar: aCharacter
	streamReader normalizedLineEndingChar: aCharacter! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/7/2013 04:31'!
replaceParameterEntityReference
	streamReader next.
	self nextParameterEntityReference! !

!XMLParserTokenizer methodsFor: 'initialization' stamp: 'monty 2/10/2014 01:08'!
setDriver: aDriver stream: aStream readLimit: aReadLimit
	driver := aDriver.
	streamReader :=
		(XMLNestedStreamReader
			on: aStream
			readLimit: aReadLimit).
	writeStream := WriteStream on: (String new: 128).
	nameStream := WriteStream on: (String new: 16).
	context := self tokenContextClass new.! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 2/2/2014 23:43'!
skipSeparatorsReplacingParameterEntities
	"this can be used to replace references within declarations"

	streamReader skipSeparators.
	context isInExternalSubset
		ifFalse: [^ self].
	[streamReader peek == $%]
		whileTrue: [
			self replaceParameterEntityReference.
			streamReader skipSeparators].! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:35'!
streamReader
	^ streamReader! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:35'!
tokenContext
	^ context! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:36'!
tokenContextClass
	self subclassResponsibility! !

XMLParserTokenizer subclass: #XMLWellFormedParserTokenizer
	instanceVariableNames: 'activeEntities'
	classVariableNames: 'AdditionalNameChars AttributeTypeDelimiters EncodingNameChars NameChars NormalizedSpace PubidChars UnaryOperatorChars ValueListChars'
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLWellFormedParserTokenizer commentStamp: 'monty 1/17/2014 01:50' prior: 0!
This is tokenizer for well-formed XML that supports DTD subsets and entity replacement.!

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 2/10/2014 01:17'!
initialize
	"self initialize"

	super initialize.
	self
		initializeAdditionalNameChars;
		initializeNameChars;
		initializePubidChars;
		initializeAttributeTypeDelimiters;
		initializeValueListChars;
		initializeUnaryOperatorChars;
		initializeEncodingNameChars;
		initializeNormalizedSpace! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'JAA 3/22/2011 12:43'!
initializeAdditionalNameChars
	"This creates a character set that contains all those defined in the XML spec for the
	NameChar production rule minus those also defined for NameStartChar
	(NameChar - NameStartChar)"

	(AdditionalNameChars := BitmapCharacterSet new: 16r2041)
		add: $-;
		add: $.;
		addAll: ($0 to: $9);
		add: (Character value: 16rB7).
	16r0300 to: 16r036F do: [:each | AdditionalNameChars add: (Character value: each)].
	16r203F to: 16r2040 do: [:each | AdditionalNameChars add: (Character value: each)].! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 1/21/2014 23:44'!
initializeAttributeTypeDelimiters
	AttributeTypeDelimiters := BitmapCharacterSet with: $# with: $' with: $" with: $>! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 1/21/2014 23:45'!
initializeEncodingNameChars
	(EncodingNameChars := BitmapCharacterSet new)
		addAll: ($a to: $z);
		addAll: ($A to: $Z);
		addAll: ($0 to: $9);
		add: $-;
		add: $_;
		add: $.! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 2/1/2014 02:50'!
initializeNameChars
	"This creates a character set that coveres the entire range of characters speicified for
	the NameChar production rule in the XML spec."

	(NameChars := BitmapCharacterSet new: 16rF0000)
		add: $:;
		addAll: ($A to: $Z);
		add: $_;
		addAll: ($a to: $z).
	16rC0 to: 16rD6 do: [:each | NameChars add: (Character value: each)].
	16rD8 to: 16rF6 do: [:each | NameChars add: (Character value: each)].
	16rF8 to: 16r2FF do: [:each | NameChars add: (Character value: each)].
	16r370 to: 16r37D do: [:each | NameChars add: (Character value: each)].
	16r37F to: 16r1FFF do: [:each | NameChars add: (Character value: each)].
	16r200C to: 16r200D do: [:each | NameChars add: (Character value: each)].
	16r2070 to: 16r218F do: [:each | NameChars add: (Character value: each)].
	16r2C00 to: 16r2FEF do: [:each | NameChars add: (Character value: each)].
	16r3001 to: 16rD7FF do: [:each | NameChars add: (Character value: each)].
	16rF900 to: 16rFDCF do: [:each | NameChars add: (Character value: each)].
	16rFDF0 to: 16rFFFD do: [:each | NameChars add: (Character value: each)].
	16r10000 to: 16rEFFFF do: [:each | NameChars add: (Character value: each)].

	AdditionalNameChars do: [:each | NameChars add: each].! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 2/10/2014 01:18'!
initializeNormalizedSpace
	NormalizedSpace := Character space! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 12/1/2013 00:43'!
initializePredefinedEntities
	(predefinedEntities := Dictionary new)
		at: 'lt' put: $<;
		at: 'gt' put: $>;
		at: 'amp' put: $&;
		at: 'apos' put: $';
		at: 'quot' put: $"! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 11/12/2013 01:44'!
initializePubidChars
	(PubidChars := BitmapCharacterSet new)
		add: Character space;
		add: Character cr;
		add: Character lf;
		addAll: ($a to: $z);
		addAll: ($A to: $Z);
		addAll: ($0 to: $9);
		addAll: '-''()+,./:=?;!!*#@$_%'! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 1/21/2014 23:45'!
initializeUnaryOperatorChars
	UnaryOperatorChars := BitmapCharacterSet with: $? with: $* with: $+! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 1/21/2014 23:45'!
initializeValueListChars
	ValueListChars := BitmapCharacterSet with: $( with: $| with: $)! !

!XMLWellFormedParserTokenizer class methodsFor: 'testing' stamp: 'monty 2/10/2014 03:44'!
isName: aString
	aString ifEmpty: [^ false].

	aString doWithIndex: [:each :i | 
		((NameChars includes: each)
			and: [i > 1 or: [(AdditionalNameChars includes: each) not]])
			ifFalse: [^ false]].
	^ true.! !

!XMLWellFormedParserTokenizer class methodsFor: 'testing' stamp: 'monty 12/28/2013 23:18'!
isNmtoken: aString
	^ aString notEmpty and: [aString allSatisfy: [:each | self isNmtokenChar: each]]! !

!XMLWellFormedParserTokenizer class methodsFor: 'testing' stamp: 'monty 2/1/2014 02:50'!
isNmtokenChar: aCharacter
	^ NameChars includes: aCharacter! !

!XMLWellFormedParserTokenizer methodsFor: 'accessing' stamp: 'monty 1/17/2014 05:28'!
activeEntities
	^ activeEntities ifNil: [activeEntities := OrderedCollection new]! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 1/23/2014 04:25'!
addActiveEntity: anEntity
	(self hasActiveEntityNamed: anEntity name)
		ifTrue: [self parseError: 'Illegal self-referential entity ', anEntity asReference].
	self maxEntityReplacementDepth
		ifNotNil: [:maxDepth |
			(self activeEntities size + 1> maxDepth)
				ifTrue: [self entityDepthError: 'Cannot further nest entity replacements']].
	
	self activeEntities addLast: anEntity.
	anEntity isExternal
		ifTrue: [driver currentURI: anEntity uri].! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 12/13/2013 18:30'!
endReplacementForGeneralEntity: anEntity isInContent: aBoolean
	self removeActiveEntity: anEntity.
	aBoolean
		ifTrue: [driver handleEndContentEntityReplacement: anEntity name].! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 12/13/2013 18:30'!
endReplacementForParameterEntity: anEntity
	self removeActiveEntity: anEntity! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 1/24/2014 00:51'!
entityDepthError: anErrorString
	XMLLimitException signal: anErrorString! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 12/10/2013 07:28'!
errorExpected: anExpectedString
	self parseError: 'Expected ', anExpectedString! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 12/9/2013 18:29'!
errorExpected: anExpectedCharacterOrString butGot: aReceivedCharacterOrString
	| expectedString receivedString |

	expectedString := anExpectedCharacterOrString asString.	
	(receivedString := (aReceivedCharacterOrString ifNil: ['']) asString)
		ifEmpty: [receivedString := 'nothing'].

	self errorExpected: expectedString, ' but got ', receivedString.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 1/4/2014 06:50'!
expectDigit
	| nextChar |

	((nextChar := streamReader next) notNil
		and: [nextChar isDigit])
		ifFalse: [
			self
				errorExpected: 'digit'
				butGot: nextChar].
	^ nextChar.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 1/4/2014 06:52'!
expectLetter
	| nextChar |

	((nextChar := streamReader next) notNil
		and: [nextChar isLetter])
		ifFalse: [
			self
				errorExpected: 'letter'
				butGot: nextChar].
	^ nextChar.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:35'!
expectNext: aCharacter
	| nextChar |

	(nextChar := streamReader next) == aCharacter
		ifFalse: [
			self
				errorExpected: aCharacter
				butGot: nextChar].
	^ nextChar.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:36'!
expectNextAll: anExpectedLiteral
	| nextChar |

	anExpectedLiteral doWithIndex: [:each :i |
		(nextChar := streamReader next) == each
			ifFalse: [
				self
					errorExpected: anExpectedLiteral
					butGot:
						(anExpectedLiteral
							copyReplaceFrom: i
							to: anExpectedLiteral size
							with: (nextChar ifNil: ['']) asString)]].
	^ anExpectedLiteral.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/9/2013 06:51'!
expectQuote
	| nextChar |

	(((nextChar := streamReader next) == $")
		or: [nextChar == $'])
		ifFalse: [
			self
				errorExpected: 'quote character delimiter'
				butGot: nextChar].
	^ nextChar.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:36'!
expectTerminator: aCharacter
	| nextChar |

	(nextChar := streamReader next) == aCharacter
		ifFalse: [
			self
				errorExpected: aCharacter asString, ' terminator'
				butGot: nextChar].
	^ nextChar.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 1/28/2014 01:29'!
expectUpToAll: aString
	| isTerminated |

	"upToAll: can't distinguish between a missing terminator or terminator at the end of a string"
	isTerminated := false.
	writeStream reset.
	[streamReader atEnd
		or: [(isTerminated := streamReader nextMatchAll: aString)]]
		whileFalse: [writeStream nextPut: streamReader next].

	isTerminated
		ifFalse: [self errorExpected: 'terminating ', aString].

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'testing' stamp: 'monty 12/13/2013 19:55'!
hasActiveEntities
	^ activeEntities notNil and: [activeEntities notEmpty]! !

!XMLWellFormedParserTokenizer methodsFor: 'testing' stamp: 'monty 12/13/2013 19:58'!
hasActiveEntityNamed: aName
	^ self activeEntities anySatisfy: [:each | each name = aName]! !

!XMLWellFormedParserTokenizer methodsFor: 'testing' stamp: 'monty 12/13/2013 19:59'!
hasActiveExternalEntity
	^ self activeEntities anySatisfy: [:each | each isExternal]! !

!XMLWellFormedParserTokenizer methodsFor: 'accessing' stamp: 'monty 1/17/2014 05:28'!
maxEntityReplacementDepth
	^ driver maxEntityReplacementDepth! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 2/6/2014 01:58'!
nextAttlistDeclaration
	| element defaultPragma |

	self
		expectNextAll: 'ATTLIST';
		skipSeparatorsReplacingParameterEntities.

	element := self nextName.
	[self skipSeparatorsReplacingParameterEntities.
	(streamReader atEnd or: [streamReader peek == $>])]
		whileFalse: [
			driver
				handleAttributeDeclaration: element
				name: self nextName
				type: self nextAttributeType
				defaultPragma: (defaultPragma := self nextAttributeDefaultPragma)
				defaultValue: (self nextAttributeDefaultValueForPragma: defaultPragma)].

	self
		skipSeparatorsReplacingParameterEntities;
		expectTerminator: $>.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 1/28/2014 05:38'!
nextAttributeDefaultPragma
	self skipSeparatorsReplacingParameterEntities.
	streamReader peek == $#
		ifFalse: [^ ''].
	streamReader next.

	^ streamReader peek == $R
		ifTrue: [self expectNextAll: 'REQUIRED']
		ifFalse: [
			streamReader peek == $I
				ifTrue: [self expectNextAll: 'IMPLIED']
				ifFalse: [self expectNextAll: 'FIXED']].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/31/2013 00:32'!
nextAttributeDefaultValueForPragma: aPragma
	self skipSeparatorsReplacingParameterEntities.

	^ (aPragma isEmpty or: [aPragma first == $F]) "FIXED"
		ifTrue: [self nextAttributeValue]
		ifFalse: [''].
! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 1/28/2014 01:30'!
nextAttributeType
	| nextChar |

	writeStream reset.
	[self skipSeparatorsReplacingParameterEntities.
	(nextChar := streamReader peek) isNil
		or: [AttributeTypeDelimiters includes: nextChar]]
		whileFalse: [
			(ValueListChars includes: nextChar)
				ifTrue: [
					writeStream position > 0
						ifTrue: [writeStream space].
					writeStream nextPut: streamReader next]
				ifFalse: [self nextNmtokenOn: writeStream]].
	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 2/6/2014 01:05'!
nextAttributeValue
	| quote value |

	((quote := streamReader next) == $"
		or: [quote == $'])
		ifFalse: [
			quote == $&
				ifTrue: [^ self nextUnparsedEntityReference]
				ifFalse: [self errorExpected: 'start of quoted attribute value']].
	
	context
		saveCurrent;
		enterLiteralValue.
	value := self nextAttributeValueDelimitedBy: quote.
	context resetPrevious.

	(streamReader next == quote)
		ifFalse: [self errorExpected: 'end of quoted attribute value'].

	^ value.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 2/10/2014 01:18'!
nextAttributeValueDelimitedBy: aQuote
	| nextChar |

	writeStream reset.
	[(nextChar := streamReader peek) == aQuote
		or: [nextChar == $<
			or: [nextChar == nil]]]
		whileFalse: [
			(streamReader next) == $&
				ifTrue: [
					streamReader peek == $#
						ifTrue: [writeStream nextPut: self nextDecodedCharReference]
						ifFalse: [
							(nextChar := self nextGeneralEntityReference)
								ifNotNil: [writeStream nextPut: nextChar]]]
				ifFalse: [
					nextChar isSeparator
						ifTrue: [nextChar := NormalizedSpace].
					writeStream nextPut: nextChar]].
	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 1/28/2014 01:30'!
nextConditionalSection
	"skip ["
	streamReader next.

	writeStream reset.
	self nextConditionalSectionOn: writeStream.
	streamReader pushBack: writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/28/2013 22:29'!
nextConditionalSectionOn: aWriteStream
	self
		skipSeparatorsReplacingParameterEntities;
		expectNext: $I.

	^ streamReader peek == $N
		ifTrue: [self nextIncludeSectionOn: aWriteStream]
		ifFalse: [self nextIgnoreSection].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 2/3/2014 01:12'!
nextContentToken
	streamReader peek == $<
		ifTrue: [
			streamReader next.
			^ self nextContentMarkupToken].
	self nextPCDataToken.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 2/18/2014 05:36'!
nextDecodedCharReference
	| codePoint |

	"skip #"
	streamReader next.
	(codePoint := streamReader nextInteger)
		ifNil: [self errorExpected: 'integer'].
	self expectNext: $;.

	^ (self characterFromCodePoint: codePoint)
		ifNil: [self parseError: 'Invalid character code point ', codePoint printString].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 1/30/2014 03:21'!
nextElementDeclaration
	| name nextChar contentModel |

	self
		expectNextAll: 'LEMENT';
		skipSeparatorsReplacingParameterEntities.

	name := self nextName.
	writeStream reset.
	[self skipSeparatorsReplacingParameterEntities.
	(streamReader atEnd or: [(nextChar := streamReader peek) == $>])]
		whileFalse: [
			nextChar == $#
				ifTrue: [writeStream nextPutAll: (self expectNextAll: '#PCDATA')]
				ifFalse: [
					(nextChar == $, or: [ValueListChars includes: nextChar])
						ifTrue: [writeStream nextPut: streamReader next]
						ifFalse: [self nextNameOn: writeStream]].
			(UnaryOperatorChars includes: streamReader peek)
				ifTrue: [writeStream nextPut: streamReader next]].
	contentModel := writeStream contents.

	self
		skipSeparatorsReplacingParameterEntities;
		expectTerminator: $>.

	driver
		handleElementDeclaration: name
		contentModel: contentModel.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/28/2014 05:17'!
nextEndDocument
	context isInInternalSubset
		ifTrue: [self errorExpected: 'end of internal subset'].
	super nextEndDocument.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/10/2013 07:32'!
nextEndInternalSubset
	self hasActiveEntities
		ifTrue: [self parseError: 'Parameter entity replacement cannot terminate internal subset'].

	super nextEndInternalSubset.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/9/2013 01:27'!
nextEndTag
	| tagName |

	"Skip /"
	streamReader next.
	tagName := self nextName.
	streamReader skipSeparators.
	self expectTerminator: $>.

	driver handleEndTag: tagName.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/11/2013 01:44'!
nextEntityDeclaration
	| isParameterEntity name  |

	self expectNextAll: 'NTITY'.

	"this is to distinguish a parameter reference from declaration"
	isParameterEntity := false.
	[streamReader skipSeparators.
	streamReader peek == $% and: [isParameterEntity not]]
		whileTrue: [
			streamReader next.
			(context isInExternalSubset
				and: [streamReader atEnd not
					and: [streamReader peek isSeparator not]])
				ifTrue: [self nextParameterEntityReference]
				ifFalse: [isParameterEntity := true]].

	self skipSeparatorsReplacingParameterEntities.
	name := self nextName.
	self skipSeparatorsReplacingParameterEntities.
	streamReader atQuote
		ifTrue: [
			self
				nextInternalEntityDeclaration: name
				isParameterEntity: isParameterEntity]
		ifFalse: [
			self
				nextExternalEntityDeclaration: name
				isParameterEntity: isParameterEntity].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 10/19/2014 05:57'!
nextEntityValue
	| quote value  |

	quote := streamReader next.
	context
		saveCurrent;
		enterLiteralValue.
	value := self nextEntityValueDelimitedBy: quote.
	context resetPrevious.
	self expectNext: quote.

	^ value.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 2/10/2014 19:56'!
nextEntityValueDelimitedBy: aQuote
	| nextChar |

	writeStream reset.
	[(nextChar := streamReader peek) == aQuote
		or: [nextChar == nil]]
		whileFalse: [
			(streamReader next) == $&
				ifTrue: [
					writeStream nextPut: 
						(streamReader peek == $#
							ifTrue: [self nextDecodedCharReference]
							ifFalse: [$&])]
				ifFalse: [
					nextChar == $%
						ifTrue: [self nextParameterEntityReference]
						ifFalse: [writeStream nextPut: nextChar]]].
	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/12/2013 00:09'!
nextExternalEntityDeclaration: aName isParameterEntity: aBoolean
	| externalId notation |

	externalId := self nextExternalIDSystemLiteralRequired: true.
	aBoolean
		ifTrue: [
			driver
				handleParameterEntityDeclaration: aName
				publicID: externalId key
				systemID: externalId value]
		ifFalse: [
			self skipSeparatorsReplacingParameterEntities.
			streamReader peek == $>
				ifTrue: [notation := '']
				ifFalse: [
					self
						expectNextAll: 'NDATA';
						skipSeparatorsReplacingParameterEntities.
					notation := self nextName].
			driver
				handleGeneralEntityDeclaration: aName
				publicID: externalId key
				systemID: externalId value
				ndata: notation].
	self
		skipSeparatorsReplacingParameterEntities;
		expectTerminator: $>.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 2/10/2014 19:54'!
nextGeneralEntityReference
	| name entity replacement startedInContent |

	name := self nextName.
	self expectNext: $;.
	self class predefinedEntities
		at: name
		ifPresent: [:character | ^ character].

	entity := driver handleGeneralEntityReference: name.
	(replacement := self replacementFromGeneralEntity: entity)
		ifNotEmpty: [
			startedInContent := context isInContent.
			self
				pushBackReplacement: replacement
				forEntity: entity
				onClose: [
					self
						endReplacementForGeneralEntity: entity
						isInContent: startedInContent].
			startedInContent
				ifTrue: [driver handleStartContentEntityReplacement: name]].
	^ nil.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/10/2013 06:50'!
nextIgnoreSection
	| openSections |

	self expectNextAll: 'GNORE'.
	streamReader skipSeparators.
	self expectNext: $[.

	openSections := 1.
	[openSections > 0 and: [streamReader atEnd not]]
		whileTrue: [
			(streamReader nextMatchAll: ']]>')
				ifTrue: [openSections := openSections - 1]
				ifFalse: [
					(streamReader nextMatchAll: '<!![')
						ifTrue: [openSections := openSections + 1]
						ifFalse: [streamReader next]]].

	openSections > 0
		ifTrue: [self errorExpected: 'terminating ]]>'].

	^ ''.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/28/2013 22:28'!
nextIncludeSectionOn: aWriteStream
	| isOpen  |

	self expectNextAll: 'NCLUDE'.
	streamReader skipSeparators.
	self expectNext: $[.

	isOpen := true.
	[isOpen and: [streamReader atEnd not]]
		whileTrue: [
			(streamReader nextMatchAll: '<!![')
				ifTrue: [self nextConditionalSectionOn: aWriteStream]
				ifFalse: [
					(streamReader nextMatchAll: ']]>')
						ifTrue: [isOpen := false]
						ifFalse: [aWriteStream nextPut: streamReader next]]].

	isOpen
		ifTrue: [self errorExpected: 'terminating ]]>'].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/11/2013 01:44'!
nextInternalEntityDeclaration: aName isParameterEntity: aBoolean
	| value |

	value := self nextEntityValue.
	self
		skipSeparatorsReplacingParameterEntities;
		expectTerminator: $>.

	aBoolean
		ifTrue: [driver handleParameterEntityDeclaration: aName replacement: value]
		ifFalse: [driver handleGeneralEntityDeclaration: aName replacement: value].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 2/10/2014 22:27'!
nextName
	| nextChar |

	nameStream reset.
	"inlined from nextNameOn:"
	((nextChar := streamReader next) notNil
		and: [(NameChars includes: nextChar)
			and: [(AdditionalNameChars includes: nextChar) not]])
		ifFalse: [self errorExpected: 'name'].
	nameStream nextPut: nextChar.

	[(nextChar := streamReader peek) notNil
		and: [NameChars includes: nextChar]]
		whileTrue: [nameStream nextPut: streamReader next].
	^ nameStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 2/10/2014 22:26'!
nextNameOn: aWriteStream
	| nextChar |

	((nextChar := streamReader next) notNil
		and: [(NameChars includes: nextChar)
			and: [(AdditionalNameChars includes: nextChar) not]])
		ifFalse: [self errorExpected: 'name'].
	aWriteStream nextPut: nextChar.

	[(nextChar := streamReader peek) notNil
		and: [NameChars includes: nextChar]]
		whileTrue: [aWriteStream nextPut: streamReader next].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 2/1/2014 02:53'!
nextNmtokenOn: aWriteStream
	"matches the nmtoken production"
	| nextChar |

	((nextChar := streamReader peek) notNil
		and: [(NameChars includes: nextChar)])
		ifFalse: [self errorExpected: 'name token'].
	aWriteStream nextPut: streamReader next.
		
	[(nextChar := streamReader peek) notNil
		and: [NameChars includes: nextChar]]
		whileTrue: [aWriteStream nextPut: streamReader next].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/12/2013 00:10'!
nextNotationDeclaration
	| name externalId |

	self
		expectNextAll: 'NOTATION';
		skipSeparatorsReplacingParameterEntities.

	name := self nextName.
	self skipSeparatorsReplacingParameterEntities.

	externalId := self nextExternalIDSystemLiteralRequired: false.
	self
		skipSeparatorsReplacingParameterEntities;
		expectTerminator: $>.

	driver
		handleNotationDeclaration: name
		publicID: externalId key
		systemID: externalId value.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 2/6/2014 01:40'!
nextPCDataToken
	| isIgnorableWhitespace nextChar |

	isIgnorableWhitespace := true.
	writeStream reset.
	[(nextChar := streamReader peek) == $<
		or: [nextChar == nil]]
		whileFalse: [
			(streamReader next) == $&
				ifTrue: [
					(nextChar :=
						streamReader peek == $#
							ifTrue: [self nextDecodedCharReference]
							ifFalse: [self nextGeneralEntityReference])
						ifNotNil: [
							writeStream nextPut: nextChar.
							isIgnorableWhitespace := false]]
				ifFalse: [
					isIgnorableWhitespace
						ifTrue: [isIgnorableWhitespace := nextChar isSeparator].
					writeStream nextPut: nextChar]].
	isIgnorableWhitespace
		ifTrue: [
			writeStream position > 0
				ifTrue: [driver handleWhitespace: writeStream contents]]
		ifFalse: [driver handlePCData: writeStream contents].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/16/2014 09:13'!
nextPITarget
	| target |

	target := self nextName.
	(target = 'xml'
		and: [context isInProlog not
			and: [context isInTextDeclaration not]])
		ifTrue: [self parseError: 'XML declaration outside of prolog or text declaration'].

	^ target.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 2/9/2014 02:45'!
nextParameterEntityReference
	| name entity replacement |

	name := self nextName.
	self expectNext: $;.

	entity := driver handleParameterEntityReference: name.
	(replacement := self replacementFromParameterEntity: entity)
		ifNotEmpty: [
			self
				pushBackReplacement: replacement
				forEntity: entity
				onClose: [self endReplacementForParameterEntity: entity]].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/13/2013 20:09'!
nextPrologToken
	streamReader skipSeparators.
	streamReader atEnd
		ifTrue: [^ self].

	self expectNext: $<.
	streamReader peek == $?
		ifTrue: [^ self nextPIOrXMLDecl].
	streamReader peek == $!!
		ifTrue: [
			streamReader next.
			streamReader peek == $-
				ifTrue: [^ self nextComment].
			^ self nextDocTypeDecl].

	context enterContent.
	self nextTag.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 1/28/2014 01:38'!
nextPubidLiteral
	| quote nextChar |

	quote := self expectQuote.
	writeStream reset.
	[(nextChar := streamReader peek) notNil
		and: [(PubidChars includes: nextChar)
			and: [nextChar ~~ quote]]]
		whileTrue: [writeStream nextPut: streamReader next].
	self expectNext: quote.

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/10/2013 07:20'!
nextSubsetDeclaration
	streamReader peek == $E
		ifTrue: [
			streamReader next.
			^ streamReader peek == $N
				ifTrue: [self nextEntityDeclaration]
				ifFalse: [self nextElementDeclaration]].
	streamReader peek == $A
		ifTrue: [^ self nextAttlistDeclaration].
	streamReader peek == $N
		ifTrue: [^ self nextNotationDeclaration].

	self errorExpected: 'declaration'.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 2/14/2014 23:22'!
nextSubsetMarkupToken
	self expectNext: $<.
	streamReader peek == $?
		ifTrue: [^ self nextPIOrXMLDecl]
		ifFalse: [
			context isInExternalSubsetTextDeclaration
				ifTrue: [context enterExternalSubset]].

	self expectNext: $!!.
	streamReader peek == $-
		ifTrue: [^ self nextComment].

	(streamReader peek == $[)
		ifTrue: [
			(context isInExternalSubset
				or: [self hasActiveExternalEntity])
				ifTrue: [^ self nextConditionalSection].
			self parseError: 'Cannot have INCLUDE/IGNORE sections in internal subset'].

	self nextSubsetDeclaration.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/6/2013 08:36'!
nextSystemLiteralRequired: aBoolean	
	| quote value |

	(aBoolean or: [streamReader atQuote])
		ifTrue: [
			quote := self expectQuote.
			value := streamReader upTo: quote]
		ifFalse: [value := ''].
	^ value.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 10/8/2014 21:00'!
nextTag
	"performance critical method"
	| tagName attributes reservedAttributes namespaces nextChar attributeName attributeValue |

	(streamReader peek == $/)
		ifTrue: [^ self nextEndTag].
	tagName := self nextName.

	attributes := OrderPreservingDictionary new: 10 withDefaultValue: ''.
	[streamReader skipSeparators.
	((nextChar := streamReader peek) == $>
		or: [nextChar == $/
			or: [nextChar == nil]])]
		whileFalse: [
			attributeName := self nextName.
			streamReader skipSeparators.
			self expectNext: $=.
			streamReader skipSeparators.
			attributeValue := self nextAttributeValue.
	
			((attributeName at: 1) == $x "fast test to skip the others"
				and: [(attributeName beginsWith: 'xml')
					and: [attributeName size >= 5]])
				ifTrue: [
					(driver usesNamespaces
						and: [(attributeName at: 4) == $n
							and: [(attributeName at: 5) == $s]])
						ifTrue: [
							"initialize only if needed"
							namespaces
								ifNil: [namespaces := OrderPreservingDictionary new: 5 withDefaultValue: ''].
							attributeName size > 6
								ifTrue: [
									namespaces
										at: (attributeName copyFrom: 7 to: attributeName size)
										put: attributeValue]
								ifFalse: [namespaces at: '' put: attributeValue]]
						ifFalse: [
							(attributeName at: 4) == $:
								ifTrue: [
									"the xml: attributes require special processing so are copied
									to a separate dictionary to not be affected by changes to the
									regular dictionary and quick nil test if any are present"
									(reservedAttributes ifNil: [reservedAttributes := Dictionary new])
										at: attributeName put: attributeValue].
								attributes at: attributeName put: attributeValue]]
				ifFalse: [attributes at: attributeName put: attributeValue]].

	driver
		handleStartTag: tagName
		attributes: attributes
		namespaces: namespaces.
	reservedAttributes
		ifNotNil: [driver handleReservedAttributes: reservedAttributes].

	streamReader peek == $/
		ifTrue: [
			streamReader next.
			driver handleEndTag: tagName].
	self expectTerminator: $>.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/31/2014 23:51'!
nextTextDeclaration
	"skip <"
	streamReader
		skipSeparators;
		next.

	context
		saveCurrent;
		enterTextDeclaration.
	self nextPIOrXMLDecl.
	context resetPrevious.	! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/28/2014 01:39'!
nextXMLEncodingAttributeValue
	| quote |

	quote := self expectQuote.

	writeStream
		reset;
		nextPut: self expectLetter.
	[streamReader atEnd not
		and: [EncodingNameChars includes: streamReader peek]]
		whileTrue: [writeStream nextPut: streamReader next].

	self expectNext: quote.
	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:44'!
nextXMLStandaloneAttributeValue
	| quote yesOrNo |

	quote := self expectQuote.
	yesOrNo := 
		streamReader peek == $y
			ifTrue: [self expectNextAll: 'yes']
			ifFalse: [self expectNextAll: 'no'].
	self expectNext: quote.

	^ yesOrNo.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/28/2014 01:39'!
nextXMLVersionAttributeValue
	| quote |

	quote := self expectQuote.

	writeStream
		reset;
		nextPut: (self expectNext: $1);
		nextPut: (self expectNext: $.);
		nextPut: self expectDigit.

	[streamReader atEnd
		or: [streamReader peek == quote]]
		whileFalse: [writeStream nextPut: self expectDigit].

	self expectNext: quote.
	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 12/7/2013 01:15'!
parseError: anErrorString
	driver handleParseError: anErrorString! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 1/17/2014 20:40'!
pushBackReplacement: aReplacement forEntity: anEntity onClose: aBlock
	self addActiveEntity: anEntity.

	streamReader
		pushStream: aReplacement readStream
		onClose: aBlock.

	(anEntity isExternal and: [anEntity hasTextDeclaration])
		ifTrue: [self nextTextDeclaration].! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 1/20/2014 22:14'!
removeActiveEntity: anEntity
	self activeEntities remove: anEntity.
	anEntity isExternal
		ifTrue: [driver removeURI: anEntity uri].! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 12/11/2013 01:50'!
replacementFromGeneralEntity: anEntity
	anEntity ifNil: [^ ''].

	(context isInLiteralValue and: [anEntity isExternal])
		ifTrue: [self parseError: 'External general entity ', anEntity asReference, ' in literal context'].

	^ context isInLiteralValue
		ifTrue: [anEntity replacementForLiteralContext]
		ifFalse: [anEntity replacementForContentContext].! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 12/11/2013 01:50'!
replacementFromParameterEntity: anEntity
	anEntity ifNil: [^ ''].

	^ context isInLiteralValue
		ifTrue: [anEntity replacementForLiteralContext]
		ifFalse: [anEntity replacementForDTDContext]! !

!XMLWellFormedParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:36'!
tokenContextClass
	^ XMLTokenContext! !

Object subclass: #XMLPeekableStreamAdapter
	instanceVariableNames: 'stream peekChar onClose'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLPeekableStreamAdapter commentStamp: '<historical>' prior: 0!
This class wraps any readable stream that at the very least supports #next and #atEnd and makes it peekable. Additionally, it ensures that #next and #peek can be used at the end of a stream safely without raising an exception (the Squeak and Pharo behavior).!

!XMLPeekableStreamAdapter class methodsFor: 'instance creation' stamp: 'monty 12/13/2013 05:07'!
on: aStream
	^ self
		on: aStream
		onClose: nil! !

!XMLPeekableStreamAdapter class methodsFor: 'instance creation' stamp: 'monty 12/13/2013 05:08'!
on: aStream onClose: aBlock
	^ self new
		setStream: aStream
		onClose: aBlock! !

!XMLPeekableStreamAdapter methodsFor: 'testing' stamp: 'monty 10/13/2014 19:31'!
atEnd
	^ peekChar == nil and: [stream atEnd]! !

!XMLPeekableStreamAdapter methodsFor: 'file open/close' stamp: 'monty 11/16/2013 04:22'!
close
	stream close.
	onClose ifNotNil: [onClose value].! !

!XMLPeekableStreamAdapter methodsFor: 'accessing' stamp: 'JAAyer 12/14/2010 02:34'!
converter: aConverter
	(stream respondsTo: #converter:)
		ifTrue: [stream converter: aConverter]! !

!XMLPeekableStreamAdapter methodsFor: 'accessing' stamp: 'monty 10/13/2014 19:47'!
next
	peekChar
		ifNotNil: [| nextChar |
			nextChar := peekChar.
			peekChar := nil.
			^ nextChar]
		ifNil: [
			^ stream atEnd
				ifTrue: [nil]
				ifFalse: [stream next]]! !

!XMLPeekableStreamAdapter methodsFor: 'accessing' stamp: 'monty 10/13/2014 19:31'!
peek
	^ peekChar
		ifNil: [
			stream atEnd
				ifFalse: [peekChar := stream next].
			peekChar]! !

!XMLPeekableStreamAdapter methodsFor: 'initialization' stamp: 'monty 12/13/2013 05:08'!
setStream: aStream onClose: aBlock
	stream := aStream.
	peekChar := nil.
	onClose := aBlock.! !

Object subclass: #XMLTokenContext
	instanceVariableNames: 'context previousContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLTokenContext commentStamp: '<historical>' prior: 0!
The context a token occurs in.!

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/12/2013 01:50'!
atEnd
	^ context == #atEnd! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/12/2013 01:52'!
enterAtEnd
	context := #atEnd! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/12/2013 01:51'!
enterContent
	context := #inContent! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/22/2013 18:59'!
enterDoctype
	context := #inDoctype! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/12/2013 01:52'!
enterExternalSubset
	context := #inExternalSubset! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/19/2013 05:13'!
enterExternalSubsetTextDeclaration
	context := #inExternalSubsetTextDeclaration! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/12/2013 01:52'!
enterInternalSubset
	context := #inInternalSubset! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/12/2013 07:18'!
enterLiteralValue
	context := #inLiteralValue! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/12/2013 02:25'!
enterProlog
	context := #inProlog! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 11/16/2013 05:31'!
enterTextDeclaration
	context := #inTextDeclaration! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/12/2013 01:51'!
isInContent
	^ context == #inContent! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/22/2013 18:59'!
isInDoctype
	^ context == #inDoctype! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/19/2013 05:11'!
isInExternalSubset
	^ context == #inExternalSubset
		or: [self isInExternalSubsetTextDeclaration]! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/19/2013 05:11'!
isInExternalSubsetTextDeclaration
	^ context == #inExternalSubsetTextDeclaration! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/12/2013 01:50'!
isInInternalSubset
	^ context == #inInternalSubset! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/12/2013 07:17'!
isInLiteralValue
	^ context == #inLiteralValue! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/12/2013 02:25'!
isInProlog
	^ context == #inProlog! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/12/2013 22:52'!
isInSubset
	^ self isInInternalSubset or: [self isInExternalSubset]! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/19/2013 06:13'!
isInTextDeclaration
	^ context == #inTextDeclaration
		or: [self isInExternalSubsetTextDeclaration]! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/12/2013 01:30'!
isInitialized
	^ context notNil! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/12/2013 01:36'!
notInitialized
	^ context isNil! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 1/31/2014 23:44'!
resetPrevious
	context := previousContext! !

!XMLTokenContext methodsFor: 'changing context' stamp: 'monty 1/31/2014 23:44'!
saveCurrent
	previousContext := context! !

Object subclass: #XMLURI
	instanceVariableNames: 'scheme host pathSegments queryString fragment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-URI'!

!XMLURI commentStamp: 'monty 2/2/2014 02:42' prior: 0!
A class for XML URIs. It uses the XMLURIParser to parse URIs and can combine URIs using /, can decode URI paths, and can retrieve contents using XMLHTTPRequest and XMLFileHandle.!

!XMLURI class methodsFor: 'instance creation' stamp: 'monty 1/10/2014 00:16'!
fromString: aString
	^ self new fromString: aString! !

!XMLURI methodsFor: 'combining' stamp: 'monty 1/12/2014 22:05'!
/ aUriOrString
	| uriToCombineWith |

	uriToCombineWith := aUriOrString asXMLURI.
	^ (uriToCombineWith isRelative)
		ifTrue: [self copy combineWithURI: uriToCombineWith]
		ifFalse: [uriToCombineWith].! !

!XMLURI methodsFor: 'comparing' stamp: 'monty 2/6/2014 06:32'!
= anObject
	self == anObject
		ifTrue: [^ true].

	^ self class == anObject class
		and: [scheme = anObject scheme
			and: [host = anObject host
				and: [pathSegments = anObject pathSegments
					and: [queryString = anObject queryString
						and: [fragment = anObject fragment]]]]].! !

!XMLURI methodsFor: 'private' stamp: 'monty 1/16/2014 08:54'!
addPathSegment: aSegment
	(aSegment = '.' or: [aSegment = '..'])
		ifTrue: [
			(aSegment = '..' and: [pathSegments notEmpty])
				ifTrue: [pathSegments removeLast].
			^ self].

	(aSegment notEmpty
		and: [pathSegments isEmpty
			and: [self hasHost]])
		ifTrue: [pathSegments addFirst: ''].

	pathSegments addLast: aSegment.! !

!XMLURI methodsFor: 'private' stamp: 'monty 1/17/2014 02:43'!
addPathSegmentsFrom: aUri
	aUri pathSegments
		ifEmpty: [^ self].

	aUri hasAbsolutePath
		ifTrue: [pathSegments removeAll]
		ifFalse: [
			pathSegments
				ifNotEmpty: [pathSegments removeLast]].

	aUri pathSegments do: [:each | self addPathSegment: each].! !

!XMLURI methodsFor: 'converting' stamp: 'monty 1/21/2014 23:37'!
asRetrievableResource
	^ self isLocalURI
		ifTrue: [XMLFileHandle path: self unescapedPath]
		ifFalse: [XMLHTTPRequest url: self asString]! !

!XMLURI methodsFor: 'converting' stamp: 'monty 1/13/2014 00:42'!
asString
	"for compatibility if Object>>asString isn't defined"
	^ self printString! !

!XMLURI methodsFor: 'converting' stamp: 'monty 1/10/2014 00:56'!
asXMLURI
	^ self! !

!XMLURI methodsFor: 'private' stamp: 'monty 1/17/2014 02:26'!
combineWithURI: aUri
	aUri hasHost
		ifTrue: [host := aUri host].

	self addPathSegmentsFrom: aUri.
	queryString := aUri queryString.
	fragment := aUri fragment.! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 1/7/2014 02:02'!
fragment
	^ fragment! !

!XMLURI methodsFor: 'initialization' stamp: 'monty 1/10/2014 06:56'!
fromString: aString
	| parser  |

	parser := XMLURIParser on: aString.
	self
		setScheme: parser parseScheme
		host: parser parseHost
		path: parser parsePath
		queryString: parser parseQueryString
		fragment: parser parseFragment.! !

!XMLURI methodsFor: 'retrieving' stamp: 'monty 1/10/2014 00:13'!
get
	^ self getUpToLimit: nil! !

!XMLURI methodsFor: 'retrieving' stamp: 'monty 1/10/2014 00:13'!
getUpToLimit: aMaxSize
	^ self asRetrievableResource getUpToLimit: aMaxSize! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/10/2014 06:51'!
hasAbsolutePath
	^ pathSegments notEmpty
		and: [pathSegments first isEmpty]! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/16/2014 08:56'!
hasHost
	^ host notEmpty! !

!XMLURI methodsFor: 'comparing' stamp: 'monty 1/13/2014 05:05'!
hash
	^ host hash bitXor: pathSegments hash! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 1/7/2014 01:50'!
host
	^ host! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/7/2014 01:54'!
isFileURI
	^ scheme = 'file'! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/10/2014 00:47'!
isLocalURI
	^ self isFileURI or: [self isRelative]! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/10/2014 06:54'!
isRelative
	^ scheme isEmpty
		or: [host isEmpty
			and: [self isFileURI not]]! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 1/20/2014 23:55'!
path
	^ self pathPrintStringUnescaped: false! !

!XMLURI methodsFor: 'printing' stamp: 'monty 1/21/2014 00:31'!
pathPrintStringUnescaped: aBoolean
	| pathStream |

	pathSegments
		ifEmpty: [^ ''].

	pathStream := (String new: 16) writeStream.
	self
		printPathSegmentsOn: pathStream
		unescaped: aBoolean.
	^ pathStream contents.! !

!XMLURI methodsFor: 'private' stamp: 'monty 1/10/2014 00:07'!
pathSegments
	^ pathSegments! !

!XMLURI methodsFor: 'copying' stamp: 'monty 1/10/2014 00:13'!
postCopy
	pathSegments := pathSegments copy! !

!XMLURI methodsFor: 'printing' stamp: 'monty 1/9/2014 05:19'!
printFragmentOn: aStream
	fragment ifNotEmpty: [
		aStream
			nextPut: $#;
			nextPutAll: fragment]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 1/10/2014 18:49'!
printHostOn: aStream
	host ifNotEmpty: [
		scheme ifEmpty: [aStream nextPutAll: '//'].
		aStream nextPutAll: host]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 1/21/2014 00:34'!
printOn: aStream
	self
		printSchemeOn: aStream;
		printHostOn: aStream;
		printPathSegmentsOn: aStream unescaped: false;
		printQueryStringOn: aStream;
		printFragmentOn: aStream
		! !

!XMLURI methodsFor: 'printing' stamp: 'monty 1/21/2014 01:01'!
printPathSegmentsOn: aStream unescaped: aBoolean
	pathSegments
		do: [:each |
			aBoolean
				ifTrue: [self printUnescaped: each on: aStream]
				ifFalse: [aStream nextPutAll: each]]
		separatedBy: [aStream nextPut: $/]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 1/9/2014 05:19'!
printQueryStringOn: aStream
	queryString ifNotEmpty: [
		aStream
			nextPut: $?;
			nextPutAll: queryString]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 1/9/2014 05:19'!
printSchemeOn: aStream
	scheme ifNotEmpty: [
		aStream
			nextPutAll: scheme;
			nextPutAll: '://']! !

!XMLURI methodsFor: 'printing' stamp: 'monty 1/21/2014 02:03'!
printUnescaped: aString on: aStream
	| readStream nextChar charCode |

	readStream := aString readStream.
	[readStream atEnd]
		whileFalse: [
			(nextChar := readStream next) == $%
				ifTrue: [
					charCode := 
						XMLIntegerReader
							readFrom: readStream
							withBase: 16
							upTo: 2.
					nextChar := (charCode ifNil: [$%]) asCharacter].
			aStream nextPut: nextChar].! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 1/7/2014 03:38'!
queryString
	^ queryString! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 1/7/2014 01:50'!
scheme
	^ scheme! !

!XMLURI methodsFor: 'private' stamp: 'monty 1/10/2014 06:46'!
setPathSegmentsFromString: aString
	pathSegments := OrderedCollection new.

	aString ifEmpty: [^ self].

	$/ split: aString do: [:segment | pathSegments addLast: segment].! !

!XMLURI methodsFor: 'initialization' stamp: 'monty 1/10/2014 00:21'!
setScheme: aScheme host: aHost path: aPath queryString: aQueryString fragment: aFragment
	scheme := aScheme.
	host := aHost.
	self setPathSegmentsFromString: aPath.
	queryString := aQueryString.
	fragment := aFragment.! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 1/21/2014 00:32'!
unescapedPath
	^ self pathPrintStringUnescaped: true! !

Object subclass: #XMLURIParser
	instanceVariableNames: 'uriStream writeStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-URI'!

!XMLURIParser commentStamp: 'monty 1/7/2014 04:42' prior: 0!
This is a basic URI parser that can split a URI into scheme, host (with port), path, query string, and fragment.!

!XMLURIParser class methodsFor: 'instance creation' stamp: 'monty 1/7/2014 01:22'!
on: aString
	^ self new uri: aString! !

!XMLURIParser methodsFor: 'private' stamp: 'monty 1/7/2014 04:18'!
parseDoubleSlashOrReset
	2 timesRepeat: [
		(uriStream atEnd not
			and: [uriStream peek == $/])
				ifTrue: [uriStream next]
				ifFalse: [
					uriStream reset.
					^ false]].
	^ true.! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 1/10/2014 06:59'!
parseFragment
	^ self
		parseUpToAnyNotSkipping: ''
		startingAfter: $#! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 1/8/2014 02:27'!
parseHost
	^ (uriStream position > 0
		or: [self parseDoubleSlashOrReset])
		ifTrue: [(self parseUpToAnyNotSkipping: '/?#') asLowercase]
		ifFalse: [''].! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 1/7/2014 01:34'!
parsePath
	^ self parseUpToAnyNotSkipping: '?#'! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 1/10/2014 06:59'!
parseQueryString
	^ self
		parseUpToAnyNotSkipping: '#'
		startingAfter: $?! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 1/7/2014 04:18'!
parseScheme
	| scheme |

	scheme := uriStream upTo: $:.
	^ self parseDoubleSlashOrReset
		ifTrue: [scheme asLowercase]
		ifFalse: [''].

	
	
		! !

!XMLURIParser methodsFor: 'private' stamp: 'monty 1/7/2014 01:38'!
parseUpToAnyNotSkipping: aCollection
	^ self
		parseUpToAnyNotSkipping: aCollection
		startingAfter: nil! !

!XMLURIParser methodsFor: 'private' stamp: 'monty 1/7/2014 02:06'!
parseUpToAnyNotSkipping: aCollection startingAfter: aCharacter
	aCharacter ifNotNil: [
		(uriStream atEnd not
			and: [uriStream peek == aCharacter])
			ifFalse: [^ ''].
		uriStream next].

	^ aCollection
		ifEmpty: [uriStream upToEnd]
		ifNotEmpty: [
			writeStream reset.
			[uriStream atEnd
				or: [aCollection includes: uriStream peek]]
				whileFalse: [writeStream nextPut: uriStream next].
			writeStream contents].! !

!XMLURIParser methodsFor: 'accessing' stamp: 'monty 1/7/2014 06:29'!
uri: aString
	uriStream := aString readStream.
	writeStream := (String new: aString size) writeStream.! !
XMLNestedStreamReader initialize!
DTDAttributeDefinitionParser initialize!
XMLParserTokenizer initialize!
XMLWellFormedParserTokenizer initialize!
DTDValueListParser initialize!
XMLIntegerReader initialize!
DTDDFAContentModelParser initialize!
